;****************************************************************
;                        AI_INIT.ASM
;                   (programming template)
;****************************************************************
; Note: this file contains assembly language source for
;       initializing the XGA or 8514/A Adapter Interface software
;       as described in Section 6.2.1 and for performing the
;       control functions descibed in Section 6.4.1
;
;****************************************************************
;                          stack segment
;****************************************************************
STACK   SEGMENT stack

        DB      0400H DUP ('?')    ; Default stack is 1K
;
STACK   ENDS
;
;****************************************************************
;                segment for task state data
;****************************************************************
TASK    SEGMENT
;**********************|
;  AI state buffer     |
;**********************|
STATE_BUF       DB      360 DUP (00H)
;
TASK    ENDS
;
;****************************************************************
;                          data segment
;****************************************************************
DATA    SEGMENT
;**********************|
;   AI list address    |
;**********************|
AI_ADD          DD      0       ; Doubleword storage for address
                                ; of Adapter Interface services
;
;
HQDPS_DATA      DW      6       ; Length of data block
BUF_SIZE        DW      0       ; Buffer size
STK_SIZE        DW      0       ; Stack usage, in bytes
PAL_SIZE        DW      0       ; Palette buffer size, in bytes
;
HOPEN_DATA      DW      3       ; Length of data block
INIT_FLAGS      DB      0       ; 7 6 5 4 3 2 1 0 <= flags
                                ; | | ___________
                                ; | |      |_______ Reserved
                                ; | |_ Do not load palette
                                ; |___ Do not clear bit planes
AF_MODE         DB      0       ; Advanced function mode
                                ; No.   Pixels     Text
                                ; 00    1024x768   85x38
                                ; 01    640x480    80x34
                                ; 02    1024x768   128x54
                                ; 03    1024x768   146x51
RET_FLAGS       DB      0       ; Status
                                ; 0 if initialization successful
                                ; Not 0 if initialization failed
;
HINIT_DATA      DW      2       ; Length of data block
BUF_SEG         DW      0       ; Segment of AI buffer
;
HCLOSE_DATA     DW      0       ; Length field is zero for HCLOSE
HEGS_DATA       DW      0       ; Length field is zero for HEGS
DUMMY           DW      0       ; Dummy data area
;
;****************************************************************
;                      application's data
;****************************************************************
;         .
;         .
;         .
;
DATA    ENDS

;****************************************************************
;                          code segment
;****************************************************************
;
CODE    SEGMENT
        ASSUME  CS:CODE
;
START:
; Establish data and extra segment addressability
        MOV     AX,DATA         ; Address of DATA to AX
        MOV     DS,AX           ; and to DS
        ASSUME  DS:DATA         ; Assume from here on
;**********************|
;  get adapter address |
;**********************|
; The local procedure AI_VECTOR obtains the segment:offset
; address of the adapter handler
        CALL    AI_VECTOR       ; Local procedure
        JNC     OK_VECTOR       ; Go if no carry
;**********************|
;     error exit       |
;**********************|
AI_ERROR:
; HEGS (erase graphics screen)
        MOV     AX,13           ; Code number for this service
        LEA     BX,HEGS_DATA    ; Pointer to dummy data block
        CALL    AI_SERVICE
;**********************|
;     exit to DOS      |
;**********************|
DOS_EXIT:
        MOV     AH,4CH          ; DOS service request code
        MOV     AL,0            ; No error code returned
        INT     21H             ; TO DOS
;**********************|
;    AI installed      |
;**********************|
OK_VECTOR:
; Store segment and offset of AI handler
        MOV     WORD PTR AI_ADD,DX      ; Store offset of address
        MOV     WORD PTR AI_ADD+2,CX    ; and segment
; Entry point for AI services is now stored in a DS variable
;**********************|
;    initialize AI     |
;**********************|
; Call HQDPS service (query drawing process state)
        MOV     AX,31           ; Code number for this service
        LEA     BX,HQDPS_DATA   ; Pointer to data block
        CALL    AI_SERVICE
; The following information is stored by the query drawing
; process command
; 1. size of task state buffer
; 2. stack usage, in bytes
; 3. size of palette buffer
; This information may later be required by the application
;
; Call HOPEN service (enable adapter)
        MOV     INIT_FLAGS,0    ; Set initialization flags
                                ; to clear memory and load
                                ; default palette
        MOV     AF_MODE,0       ; Set 1024x768 mode number 0
        MOV     AX,8            ; Code number for this service
        LEA     BX,HOPEN_DATA   ; Pointer to data block
        CALL    AI_SERVICE
; The HOPEN command returns system information in the RET_FLAGS
; field of the parameter block.
        MOV     AL,RET_FLAGS    ; Not zero if open error
        CMP     AL,0            ; Test for no error
        JZ      OK_OPEN         ; Go if no error
        JMP     AI_ERROR        ; Error exit
;
; Call HINIT (Initialize adapter state)
OK_OPEN:
        MOV     AX,TASK         ; Segment for task state
        MOV     BUF_SEG,AX      ; Store segment of adapter state
                                ; buffer
        MOV     AX,48           ; Code number for this service
        LEA     BX,HINIT_DATA   ; Pointer to data block
        CALL    AI_SERVICE
; At this point the AI is initialized and ready for use
;****************************************************************
;                       application's code
;***************************************************************
;        .
;        .
;        .
;****************************************************************
;                           procedures
;***************************************************************
AI_VECTOR       PROC    NEAR
; Procedure to obtain the address vector to the XGA/8514/A
; AI. This procedure must be called before calls are made
; to the Adapter Interface services (by means of the AI_SERVICE
; procedure)
;
; On entry:
;          nothing
; On exit:
;          carry set if no AI installed
;          carry clear if AI present
;             CX => segment of AI link table
;             DX => offset of AI link table
;
;**********************|
;    get vector 7FH    |
;**********************|
; Use MS DOS service number 53, interrupt 21H, to obtain the
; vector for the XGA/8514-A AI interrupt (7FH)
        MOV     AH,53           ; MS DOS service number
        MOV     AL,7FH          ; AI interrupt
        INT     21H             ; MS DOS interrupt
; ES => segment of interrupt handler
; BX => offset of handler
        MOV     AX,ES           ; Segment to AX
        OR      AX,BX           ; OR segment and offset
        JNZ     OK_AI           ; Go if address not 0000:0000
;**********************|
;  ERROR - no AI       |
;**********************|
NO_AI:
        STC                     ; Error return
        RET
;**********************|
;  get AI address      |
;**********************|
; Service number 0105H, interrupt 7FH, returns the address of the
; XGA and 8514/A jump table
OK_AI:
        MOV     AX,0105H        ; Service request number
        INT     7FH             ; in XGA AI interrupt
        JC      NO_AI           ; Go if error code returned
; At this point CX:DX holds the address of the XGA/8514-A entry
; point (in segment:offset form)
        CLC                     ; No error code
        RET
AI_VECTOR       ENDP
;****************************************************************
;
AI_SERVICE      PROC    NEAR
; Procedure to access the services in the XGA and 8514/A Adapter
; Interface
;
; On entry:
;         AX = service number
;         DS:BX = address of parameter block
;
        PUSH    BP              ; Save base pointer
        MOV     BP,SP           ; Set BP to stack
; Push address of caller's parameter block
        PUSH    DS
        PUSH    BX              ; the offset
; Multiply by 4 to form offset as required by AI
        SHL     AX,1            ; AX time 2
        SHL     AX,1            ; again
        MOV     SI,AX           ; Offset to SI
        LES     BX,AI_ADD       ; Entry block address (ES:BX)
        CALL    DWORD PTR ES:[BX][SI]    ; Call AI service
        POP     BP              ; Restore caller's BP
        RET
AI_SERVICE      ENDP
;****************************************************************
;
CODE    ENDS
        END     START
