;*******************************************************************
;*******************************************************************
;                            CSETHP.ASM
;*******************************************************************
;*******************************************************************
;
; Display the character set in a Hewlett Packard PCL printer font
;
; Note: this program must be linked with the MATCH.LIB library
;       furnished with the MATCH program
;
;*******************************************************************
; External references in MATCH.LIB
EXTRN   KBR_WAIT:       FAR
EXTRN   BIO_SET_MODE:   FAR
EXTRN   BIO_CLEAR_SCR:  FAR
EXTRN   BIO_SHOW_MESS:  FAR
EXTRN   COARSE_FILL:    FAR
EXTRN   PIXEL_ADD:      FAR
EXTRN   SET_WRITE_MODE: FAR
EXTRN   OPEN_FILE:      FAR
EXTRN   CLOSE_FILE:     FAR
EXTRN   OPEN_CREATE:    FAR
EXTRN   READ_128:       FAR
EXTRN   FINE_CHARHP:    FAR
EXTRN   FINE_TEXT:      FAR
EXTRN   INDEX_HP:       FAR
EXTRN   FONT_TO_RAM:    FAR
;
; Data externals
EXTRN   MAP_Y:          BYTE
EXTRN   PIX_WIDTH:      WORD
;
INCLUDELIB MATCH
;***************************** stack *******************************
;
STACK   SEGMENT stack
;
               DB      0400H DUP ('?'); Default stack is 1K
STACK_TOP      EQU     THIS BYTE
;
STACK   ENDS
;***************************** data ********************************
;
DATA    SEGMENT BYTE    PUBLIC
;
; RAM storage for Hewlett Packard font
HP_FONT         DB      32000 DUP (00H)
; RAM storage for 1 x 16 built in ROM font
                DB      1       ; Bit map x dimension, in bytes
                DB      16      ; Bit map y dimension, in bits
FONT_1X16       DB      4100 DUP (00H)
;
;**********************|
;   input message      |
;**********************|
INPUT_MSG       DB      'Character set display utility',0AH,0DH
                DB      'Enter HP font filename: '
                DB      00H
;
;**********************|
;     HP font data     |
;**********************|
DOS_BUFFER      DB      14
                DB      16 DUP (00H)
;
HANDLE_1        DW      0       ; File handle
START_STR       DB      '*c33E'
                DB      0
DTA_BUF         DB      128 DUP (00H)   ; Disk transfer area
                DW      0
DATA_BUF        DB      128 DUP (00H)   ; Disk data storage area
                DW      0
;
CHAR_COUNT      DB      94      ; Number of characters to display
PIX_COUNT       DW      0       ; Consecutive pixel along the
                                ; current screen line
COL_COUNT       DW      0       ; x coordinate of the start of the
                                ; current row
ROW_COUNT       DW      0       ; y coordinate of the start of the
                                ; current row
I_HIGH          DW      0       ; Vertical pixels in the letter I
I_WIDE          DW      0       ; Horizontal pixels in I
;**********************|
;     text messages    |
;**********************|
FILE_ERR        DB      0       ; Start column
                DB      29      ; Start row
                DB      00000000B       ; Color code
                DB      'ERROR - Open Failed',00H
;
NAME_BLOCK      DW      10      ; Start pixel column (0 to 639)
                DW      440     ; Start pixel row (0 to 479)
                DW      8       ; Horizontal pixel displacement
                DW      0       ; Vertical pixel displacement
                DB      00000000B       ; Color code
                DB      'Hewlett Packard font filename: '
NAME_STRING     DB      16 DUP (00H)
                DB      0
DATA    ENDS
;*******************************************************************
;
CODE    SEGMENT BYTE
        ASSUME  CS:CODE,SS:STACK
;
START_MAIN:
;**********************|
;    initialize DS     |
;**********************|
        MOV     AX,DATA         ; AX to module data
        MOV     DS,AX           ; and to DS
        ASSUME  DS:DATA
;*********************|
; program preparations|
;*********************|
        LEA     SI,INPUT_MSG
        CALL    BIO_SHOW_MESS
;*********************|
;  input filename     |
;  and open file      |
;*********************|
        LEA     DX,DOS_BUFFER
        MOV     BL,0            ; Code for open operation
        CALL    OPEN_CREATE     ; Procedure in SOLUTION.LIB
        JNC     OK_OPEN         ; Go if no error code
;**********************|
;    disk open error   |
;**********************|
; Open operation failed. Post error message and exit
        LEA     SI,FILE_ERR
        CALL    BIO_SHOW_MESS
        CALL    KBR_WAIT
        JMP     DOS_EXIT
;**********************|
;   read font into RAM |
;**********************|
OK_OPEN:
        LEA     DI,HP_FONT      ; Storage buffer for HP font
        MOV     HANDLE_1,AX     ; Store file handle
NEW_128:
        MOV     BX,HANDLE_1
        LEA     DX,DATA_BUF     ; Buffer for data storage
        PUSH    DI              ; Save buffer pointer
        CALL    READ_128        ; Read sector into buffer
        POP     DI              ; Restore buffer pointer
        CMP     AX,0            ; Test for end of file
        JNE     MOVE_128        ; Go if not at end of file
;**********************|
;    end of file       |
;**********************|
        MOV     BX,HANDLE_1     ; File handle
        CALL    CLOSE_FILE      ; Library routine
        JMP     END_OF_READ
;**********************|
;   move sector to     |
;    font buffer       |
;**********************|
; At this point DATA_BUF holds 128 bytes from disk file
; DI --> storage position in the font's buffer
MOVE_128:
        MOV     CX,128          ; Byte counter
        LEA     SI,DATA_BUF     ; Pointer to data just read
PLACE_128:
        MOV     AL,[SI]         ; Byte from DATA_BUF
        MOV     [DI],AL         ; Into font's buffer
        INC     SI              ; Bump pointers
        INC     DI
        LOOP    PLACE_128       ; Continue until all sector read
; At this point the 128 bytes newly read from the disk file are
; stored in the font's buffer
        JMP     NEW_128
;**********************|
;  load 8 x 16 font    |
;**********************|
END_OF_READ:
        MOV     AH,11H          ; Service request number
        MOV     AL,30H          ; Font table information
        MOV     BH,06H          ; 8 x 16 font
        INT     10H
; ES:BP --> 8 x 16 ROM character table
;         DS:DI --> Start of RAM buffer for font
;         CX = word length of font to store
        LEA     DI,FONT_1X16    ; RAM storage for font
        MOV     CX,2048         ; Word length of font table
; DS:DI --> Start of RAM buffer for font
; CX = word length of font to store
        CALL    FONT_TO_RAM
;
;**********************|
;   initialize video   |
;     buffer address   |
;**********************|
        MOV     AX,0A000H       ; AX to base address of video buffer
        MOV     ES,AX           ; To ES
;
        MOV     AH,0            ; BIOS service request code
        MOV     AL,18           ; VGA mode number 18. 640 by 480
                                ; pixels in 16 colors
        INT     10H             ; BIOS video interrupt
;**********************|
;   set write mode 2   |
;**********************|
; This program uses write mode 2 for graphics display. In this write
; mode the color code is contained in a CPU register
        MOV     AL,2            ; Mode number
        CALL    SET_WRITE_MODE
;**********************|
;  draw color blocks   |
;**********************|
; Fill entire screen
        MOV     CH,0            ; Start column
        MOV     CL,0            ; Start row
        MOV     DH,80           ; Horizontal bytes
        MOV     DL,60           ; Vertical bytes
        MOV     AL,00000111B    ; Color = gray
        CALL    COARSE_FILL
;*****************************************************************
;                          display font
;*****************************************************************
; First determine font's dimensions
        LEA     DI,HP_FONT      ; Font's address in RAM
        MOV     BL,'I'          ; Capital I
        CALL    INDEX_HP        ; Local procedure
        MOV     AH,0
        MOV     AL,CS:MAP_Y     ; Pixel height of the letter N
        MOV     I_HIGH,AX       ; Store in DS variable
        MOV     AX,CS:PIX_WIDTH ; Character's pixel width
        MOV     I_WIDE,AX       ; Store in variable
;
        MOV     CX,I_WIDE
        ADD     CX,10           ; Start x pixel coordinate
        MOV     COL_COUNT,CX    ; Store x axis coordinate for
                                ; the start of the screen row
        MOV     DX,I_HIGH       ; Start y pixel coordinate
        ADD     DX,10
        MOV     ROW_COUNT,DX    ; Store y axis pixel coordinate
                                ; for the row start
        MOV     BL,21H          ; First character to display
NEXT_FONT_CHAR:
        LEA     DI,HP_FONT      ; Offset of font table
        PUSH    BX              ; Save registers
        PUSH    CX
        PUSH    DX
        CALL    FINE_CHARHP
        POP     DX              ; Restore registers
        POP     CX
        POP     BX
        DEC     CHAR_COUNT      ; Character counter
        JZ      END_OF_FONT     ; Go if last character
; Not the last character
        INC     BL              ; Bump character code
        MOV     AX,CS:PIX_WIDTH ; Character width to AX
        ADD     AX,I_WIDE       ; Width of the letter I
        ADD     PIX_COUNT,AX    ; Add to pixel counter
        CMP     PIX_COUNT,580   ; Maximum pixels per line
        JAE     NEXT_SC_ROW     ; Go to next row routine
        ADD     CX,AX           ; Index to next character
        JMP     NEXT_FONT_CHAR  ; Continue in the same row
NEXT_SC_ROW:
        MOV     PIX_COUNT,0     ; Reset counter
        MOV     CX,COL_COUNT    ; Address of the start of row
        MOV     DX,ROW_COUNT
        ADD     DX,I_HIGH       ; Index to next row
        ADD     DX,I_WIDE       ; Add a few more pixels
        MOV     ROW_COUNT,DX    ; Update counter
        JMP     NEXT_FONT_CHAR
;*****************************************************************
;                    display font name
;*****************************************************************
; First determine font's dimensions
END_OF_FONT:
        int     03h
        LEA     SI,DOS_BUFFER   ; Filename buffer
        INC     SI              ; Skip parameter bytes
        INC     SI
        LEA     DI,NAME_STRING  ; Location in message block
MOVE_NAME:
        MOV     AL,[SI]         ; Get filename character
        CMP     AL,0            ; Test for terminator
        JE      END_OF_NAME     ; Go if terminator
        MOV     [DI],AL         ; Place in text area
        INC     SI
        INC     DI              ; Bump pointers
        JMP     MOVE_NAME
END_OF_NAME:
;**********************|
;  display font text   |
;      filename        |
;**********************|
        LEA     SI,NAME_BLOCK   ; Formatted message block
        LEA     DI,FONT_1X16
        CALL    FINE_TEXT
;
        CALL    KBR_WAIT
;
;**********************|
;     exit to DOS      |
;**********************|
DOS_EXIT:
        MOV     AL,3            ; Set alphanumeric mode 3
        CALL    BIO_SET_MODE    ; Procedure in SOLUTION.LIB
;
        MOV     AH,4CH          ; DOS service request code
        MOV     AL,0            ; No error code returned
        INT     21H             ; exit to MS DOS
;
CODE    ENDS
        END     START_MAIN
