;****************************************************************
;****************************************************************
;                           IRGB256.ASM
;****************************************************************
;****************************************************************
;
; Demonstration program for a double-bit IRGB setting of the DAC
; color registers (Chapter 3)
; Bit map:
;               7 6 5 4 3 2 1 0  <= bits
;               |_| |_| |_| |_|
;                I   R   G   B
; 
;****************************************************************
;
EXTRN   KBR_WAIT:       FAR
EXTRN   GRAPHIC_TEXT:   FAR
;
INCLUDELIB      SOLUTION
INCLUDELIB      GRAPHSOL
;
;*************************** stack ******************************
;
STACK   SEGMENT stack
;
        DB      0400H DUP ('?')
;
STACK   ENDS
;
;*************************** data *******************************
;
DATA    SEGMENT
;
COUNT_4         DB      4
;
SCREEN_TEXT     DB      0       ; Start at row 0
                DB      0       ; and column 0
                DB      11111111B       ; Bright white
        DB      ' Double-bit IRGB encoding in 256-colors ',0FFH
        DB      '                                        ',0FFH
        DB      '  64 tones per stripe          INTENSITY',0FFH
        DB      '                                  BITS  ',0FFH
        DB      '                                 <= 00',0FFH,0FFH
        DB      '                                 <= 01',0FFH,0FFH
        DB      '                                 <= 10',0FFH,0FFH
        DB      '                                 <= 11',00H
;
;                      | R   G   B | R   G   B | R   G   B |
INTENSITY_0     DB      009,009,009,009,009,018,009,009,027 ; 2
                DB      009,009,036,009,018,009,009,018,018 ; 5
                DB      009,018,027,009,018,036,009,027,009 ; 8
                DB      009,027,018,009,027,027,009,027,036 ; 11
                DB      009,036,009,009,036,018,009,036,027 ; 14
                DB      009,036,036,018,009,009,018,009,018 ; 17
                DB      018,009,027,018,009,036,018,018,009 ; 20
                DB      018,018,018,018,018,027,018,018,036 ; 23
                DB      018,027,009,018,027,018,018,027,027 ; 26
                DB      018,027,036,018,036,009,018,036,018 ; 29
                DB      018,036,027,018,036,036,027,009,009 ; 32
                DB      027,009,018,027,009,027,027,009,036 ; 35
                DB      027,018,009,027,018,018,027,018,027 ; 38
                DB      027,018,036,027,027,009,027,027,018 ; 41
                DB      027,027,027,027,027,036,009,036,009 ; 44
                DB      027,036,018,027,036,027,027,036,036 ; 47
                DB      036,009,009,036,009,018,036,009,027 ; 50
                DB      036,009,036,036,018,009,036,018,018 ; 53
                DB      036,018,027,036,018,036,036,027,009 ; 56
                DB      036,027,018,036,027,027,036,027,036 ; 59
                DB      036,036,009,036,036,018,036,036,027 ; 62
                DB      036,036,036;                        ; 63
;
INTENSITY_1     DB      192 DUP (00)
INTENSITY_2     DB      192 DUP (00)
INTENSITY_3     DB      192 DUP (00)
;
DATA    ENDS
;
;
;*************************** code *******************************
;
CODE    SEGMENT PUBLIC
        ASSUME  CS:CODE
;
START:
; Initialize DS to establish addressability of data items
        MOV     AX,DATA
        MOV     DS,AX
        ASSUME  DS:DATA
        MOV     ES,AX           ; Set ES to data also
        ASSUME  ES:DATA
;**********************|
;       init VGA       |
;**********************|
; Set mode number 19 of VGA (color, 320 by 200 PELs)
        MOV     AH,0            ; BIOS service request number
        MOV     AL,19           ; Mode
        INT     10H
; Set write mode for 256 colors
        MOV     AL,0
        CALL    SET_WRITE_256
;**********************|
;  expand color table  |
;**********************|
; The data segment table named INTENSITY_0 contains the register
; values for 00 intensity bits. The values for 01, 10, and 11
; intensity bits are calculated by adding 9, 18, and 27 to the
; values in the INTENSITY_0 table
        LEA     SI,INTENSITY_0  ; Set pointer to source table
        LEA     DI,INTENSITY_1  ; Pointer to intensity 01 table
        LEA     BX,INTENSITY_2  ; Pointer to intensity 10 table
        LEA     BP,INTENSITY_3  ; Pointer to intensity 11 table
        MOV     CX,192          ; Number of values to move
        MOV     AH,9            ; Constant value to add
DO_TABLES:
        MOV     AL,[SI]         ; Value from intensity 00 table
        ADD     AL,AH           ; Add 9, once
        MOV     [DI],AL         ; Place in intensity 01 table
        ADD     AL,AH           ; Add 18
        MOV     [BX],AL         ; Place in intensity 10 table
        ADD     AL,AH           ; Add 27
        MOV     DS:[BP],AL      ; Place in intensity 11 table
        INC     SI              ; Bump all pointers
        INC     DI
        INC     BX
        INC     BP
        LOOP    DO_TABLES
;**********************|
;  set DAC registers   |
;**********************|
; Change first 64 DAC registers using BIOS service number 16
; of interrupt 10H
        MOV     AH,16           ; Service number
        MOV     AL,18           ; Sub-service to set group of
                                ; DAC color registers
        LEA     DX,INTENSITY_0  ; Pointer to table of 256 DAC
                                ; colors
        MOV     BX,0            ; Start with register
        MOV     CX,256          ; Set all 256 DAC registers
        INT     10H
;**********************|
;    text message      |
;**********************|
        LEA     DI,SCREEN_TEXT  ; Pointer to text message
        CALL    GRAPHIC_TEXT    ; Library procedure
; Set ES to VGA video buffer base
        MOV     AX,0A000H
        MOV     ES,AX
;**********************|
;    color stripes     |
;**********************|
; The color test consists of displaying columns of 8 pixels
; while incrementing the color code
;**********************|
;  display 64 stripes  |
;    of intensity 00   |
;**********************|
        MOV     AH,0            ; Initial color code
        MOV     DI,10240        ; Pointer to start of display area
        PUSH    DI              ; Save address
STRIPE_1:
        CALL    SET_PIXS
        INC     AH              ; Change color code
        CMP     AH,64           ; End of display range
        JE      INTEN_01        ; Finished
        JMP     STRIPE_1
;**********************|
;  display 64 stripes  |
;    of intensity 01   |
;**********************|
INTEN_01:
        POP     DI              ; Restore address
        ADD     DI,5120         ; Pointer to start of display area
        PUSH    DI              ; Save address
STRIPE_2:
        CALL    SET_PIXS
        INC     AH              ; Change color code
        CMP     AH,128          ; End of display range
        JE      INTEN_02        ; Finished
        JMP     STRIPE_2
;**********************|
;  display 64 stripes  |
;    of intensity 10   |
;**********************|
INTEN_02:
        POP     DI              ; Restore address
        ADD     DI,5120         ; Pointer to start of display area
        PUSH    DI              ; Save address
STRIPE_3:
        CALL    SET_PIXS
        INC     AH              ; Change color code
        CMP     AH,192          ; End of display range
        JE      INTEN_03        ; Finished
        JMP     STRIPE_3
;**********************|
;  display 64 stripes  |
;    of intensity 11   |
;**********************|
INTEN_03:
        POP     DI              ; Restore address
        ADD     DI,5120         ; Pointer to start of display area
        PUSH    DI              ; Save address
STRIPE_4:
        CALL    SET_PIXS
        INC     AH              ; Change color code
        CMP     AH,0            ; End of display range
        JE      END_STRIPE      ; Finished
        JMP     STRIPE_4
END_STRIPE:
        POP     DI
        CALL    KBR_WAIT
;
EXIT_TO_DOS:
; Reset mode 3
        MOV     AH,0            ; BIOS service request number
        MOV     AL,3            ; 16 color 80 x 25 alphanumeric
        INT     10H
;
        MOV     AH,4CH          ; DOS service request code
        MOV     AL,0            ; No error code returned
        INT     21H             ; TO DOS
;****************************************************************
;                           procedures
;****************************************************************
SET_PIXS     PROC    NEAR
; Display a vertical line of 7 pixel rows using the same color
; Each line is 4 pixels wide
        MOV     COUNT_4,4       ; Reset width counter
NEW_COL:
        MOV     CX,7            ; Counter for 7  rows
        PUSH    DI              ; Save buffer pointer
SET_THEM:
        MOV     ES:[DI],AH      ; Color to screen
        ADD     DI,320          ; Bump to next row
        LOOP    SET_THEM
; All columns displayed
        POP     DI              ; Restore pointer
        INC     DI              ; Bump to next column
; All 8 vertical pixels now set
        DEC     COUNT_4         ; Counter for 4 repetitions
        JNZ     NEW_COL
        RET
SET_PIXS     ENDP
;****************************************************************
;
SET_WRITE_256   PROC     NEAR
; Set the Graphics Controller's Graphic Mode Register to the
; desired write mode in 256 colors
; On entry:
;         AL = write mode requested
; Also set default bit mask
;
        PUSH    AX          ; Save mode
        MOV     DX,3CEH     ; Graphic Controller Address
                            ; Register
        MOV     AL,5        ; Offset of the Mode Register
        OUT     DX,AL       ; Select this register
        JMP     SHORT $+2
        INC     DX          ; Point to data register
        POP     AX          ; Recover mode in AL
; Set bit 6 to enable 256 colors
        OR      AL,01000000B    ; Mask for bit 6
        OUT     DX,AL       ; Selected
        JMP     SHORT $+2
; Set Bit Mask Register to default setting
        MOV     DX,3CEH     ; Graphic controller latch
        MOV     AL,8
        OUT     DX,AL       ; Select data register 8
        JMP     SHORT $+2
        INC     DX          ; To 3CFH
        MOV     AL,0FFH     ; Default mask
        OUT     DX,AL       ; Load bit mask
        JMP     SHORT $+2
        RET
SET_WRITE_256   ENDP
;
CODE    ENDS
        END     START
