;****************************************************************
;****************************************************************
;                           PALETTE.ASM
;****************************************************************
;****************************************************************
;
; Demonstration program for manipulating the VGA palette
; registers (see Chapter 3)
;
; Note:
;     This program will run on systems using the EGA video card
;     and on computers with VGA video
;     The values of the default palette are valid for the VGA
;     system only.
;
;*************************** stack ******************************
;
STACK   SEGMENT stack
;
        DB      0100H DUP ('?')
;
STACK   ENDS
;
;*************************** data *******************************
;
DATA    SEGMENT
;
PALETTE_1       DB      16 DUP (00H)
OVERSCAN_1      DB      0
;
; Alternative palette with the following colors:
; Palette bits:           rgbRGB
PALETTE_2       DB      00000000B       ; Black
                DB      00100000B       ; bB
                DB      00000100B       ; b
                DB      00100100B       ; B
                DB      00000001B       ; rR
                DB      00001000B       ; r
                DB      00001001B       ; R
                DB      00100111B       ; rRGB
                DB      00000010B       ; rgb
                DB      00010000B       ; gG
                DB      00010010B       ; g
                DB      00000010B       ; G
                DB      00111110B       ; rgbRG
                DB      00011101B       ; gbRB
                DB      00011011B       ; gbGB
                DB      00010111B       ; gRGB
;
OVERSCAN_2      DB      00111000B
;
; Number of active palette
PAL_NUM         DB      1
;
; Screen messages:
PAL1_MS         DB      2               ; Row
                DB      20              ; Column
                DB      7               ; White characters
                DB      'Palette No. 1: (Default palette)',0FFH
                DB      '   r g b R G B',0FFH
                DB      '=> 0 0 0 0 0 0  (text background)',0FFH
                DB      '=> 0 0 0 0 0 1',0FFH
                DB      '=> 0 0 0 0 1 0',0FFH
                DB      '=> 0 0 0 0 1 1',0FFH
                DB      '=> 0 0 0 1 0 0',0FFH
                DB      '=> 0 0 0 1 0 1',0FFH
                DB      '=> 0 1 0 1 0 0',0FFH
                DB      '=> 0 0 0 1 1 1  (text foreground)',0FFH
                DB      '=> 1 1 1 0 0 0  (overscan color)',0FFH
                DB      '=> 1 1 1 0 0 1',0FFH
                DB      '=> 1 1 1 0 1 0',0FFH
                DB      '=> 1 1 1 0 1 1',0FFH
                DB      '=> 1 1 1 1 0 0',0FFH
                DB      '=> 1 1 1 1 0 1',0FFH
                DB      '=> 1 1 1 1 1 0',0FFH
                DB      '=> 1 1 1 1 1 1',0FFH
                DB      00H
;
PAL2_MS         DB      2               ; Row
                DB      20              ; Column
                DB      7               ; White characters
                DB      'Palette No. 2:                  ',0FFH
                DB      '   r g b R G B',0FFH
                DB      '=> 0 0 0 0 0 0  (text background)',0FFH
                DB      '=> 1 0 0 0 0 0',0FFH
                DB      '=> 0 0 0 1 0 0',0FFH
                DB      '=> 1 0 0 1 0 0',0FFH
                DB      '=> 0 0 0 0 0 1',0FFH
                DB      '=> 0 0 1 0 0 0',0FFH
                DB      '=> 0 0 1 0 0 1',0FFH
                DB      '=> 0 0 0 1 1 1  (text foreground)',0FFH
                DB      '=> 0 0 0 0 1 0                   ',0FFH
                DB      '=> 0 1 0 0 0 0',0FFH
                DB      '=> 0 1 0 0 1 0',0FFH
                DB      '=> 1 1 1 0 1 1',0FFH
                DB      '=> 1 1 1 1 0 0',0FFH
                DB      '=> 1 1 1 1 0 1',0FFH
                DB      '=> 1 1 1 1 1 0',0FFH
                DB      '=> 1 1 1 1 1 1',0FFH
                DB      00H
;
TOP_MS          DB      0               ; Row
                DB      12              ; Column
                DB      7               ; White characters
                DB      'PALETTE DEMONSTRATION PROGRAM',00H
;
BOT_MS          DB      22              ; Row
                DB      10              ; Column
                DB      7               ; White characters
                DB      ' ** Press any key to toggle palettes **',0FFH
                DB      '       *** Press <F1> to exit ***',00H
;
;
DATA    ENDS
;
;*************************** code *******************************
;
CODE    SEGMENT
;
        ASSUME  CS:CODE
;
START:
;
; Establish data and extra segment addressability
        MOV     AX,DATA
        MOV     DS,AX
        ASSUME  DS:DATA
        MOV     ES,AX
        ASSUME  ES:DATA
;
; Read default palette registers into PALETTE_1 buffer
; Bytes No. 0 to 15 = palette registers
; Byte No. 16 = overscan register
        MOV     AH,16           ; BIOS service request number
        MOV     AL,9            ; Read all palette and overscan
        MOV     DX,OFFSET PALETTE_1
        INT     10H
;
; Set video mode number 3
        CALL    MODE_3
;
; Clear the screen
        CALL    SETES_VIDEO
        MOV     AL,20H          ; Blank
        MOV     AH,80H          ; Grey/black attribute
        MOV     CX,2000         ; Character count
        MOV     DI,0            ; Start of buffer
        CLD                     ; Forward
        REP     STOSW
;
;**********************|
;      color blocks    |
;**********************|
; Loop to display 16 solid, horizontal blocks
; Attribute is increased for each block
        MOV     AL,0DBH         ; A solid character
        MOV     DH,4            ; Row number 4
        MOV     DL,4            ; Column number 4
        CALL    SET_DI
        MOV     DX,16           ; Outer loop counter
        MOV     AH,0            ; Starting attribute
SHOW_5:
        CALL    COLOR_BLOK
        INC     AH              ; Bump attribute
        DEC     DX              ; Loop counter
        JNZ     SHOW_5
;**********************|
;       display        |
;    screen messages   |
;**********************|
        MOV     SI,OFFSET PAL1_MS
        CALL    SHOW_BLOCK
        MOV     SI,OFFSET TOP_MS
        CALL    SHOW_BLOCK
        MOV     SI,OFFSET BOT_MS
        CALL    SHOW_BLOCK
;
;**********************|
;   monitor keyboard   |
;**********************|
MONITOR:
        CALL    WAIT_4_KEY
        JC      EXIT            ; <F1> key was pressed, exit
; Toggle palette
        CMP     PAL_NUM,2
        JE      SET_PAL1
;
;**********************|
;    palette No. 2     |
;**********************|
        CALL    SETES_DATA      ; Required by BIOS service
        MOV     AH,10H          ; BIOS service request number
        MOV     AL,2            ; Change all palette registers
                                ; and the overscan register
        MOV     DX,OFFSET PALETTE_2
                                ; Address of new palette string
        INT     10H
; Set active palette number to this palette
        MOV     PAL_NUM,2
; Display palette codes message
        CALL    SETES_VIDEO
        MOV     SI,OFFSET PAL2_MS
        CALL    SHOW_BLOCK
        JMP     MONITOR
;**********************|
;    palette No. 1     |
;**********************|
SET_PAL1:
        CALL    SETES_DATA      ; Required by BIOS service
        MOV     AH,10H          ; BIOS service request number
        MOV     AL,2            ; Change all palette registers
                                ; and the overscan register
        MOV     DX,OFFSET PALETTE_1
                                ; Address of new palette string
        INT     10H
; Set active palette number to this palette
        MOV     PAL_NUM,1
; Display palette codes message
        CALL    SETES_VIDEO
        MOV     SI,OFFSET PAL1_MS
        CALL    SHOW_BLOCK
        JMP     MONITOR
;
;**********************|
;      exit to DOS     |
;**********************|
EXIT:
        CALL    MODE_3          ; Reset video mode and clear
        MOV     AH,4CH          ; DOS service request code
        MOV     AL,0            ; No error code returned
        INT     21H             ; TO DOS
;
;****************************************************************
;                           procedures
;****************************************************************
COLOR_BLOK      PROC    NEAR
; Display CX times the character/attribute in AX
;
        PUSH    DI              ; Save start column
        CLD                     ; Forward
        MOV     CX,15           ; 15 characters
        REP     STOSW
        POP     DI              ; Restore start column
        ADD     DI,160          ; Index to next column
        RET
;
COLOR_BLOK      ENDP
;
;
WAIT_4_KEY      PROC    NEAR
; Monitor keyboard until a key is pressed
; Set carry if <F1> key is pressed
        MOV     AH,0            ; BIOS service request number
        INT     16H
; Test for <F1> key
        CMP     AX,3B00H        ; <F1>
        JE      IS_F1
        CLC                     ; No carry on normal exit
        RET
; <F1> key pressed, set carry
IS_F1:
        STC
        RET
;
WAIT_4_KEY      ENDP
;
;
MODE_3          PROC    NEAR
; Set video mode number 3 for 80 by 25 color text
        MOV     AH,0            ; BIOS service request number
        MOV     AL,3            ; Mode
        INT     10H
        RET
;
MODE_3          ENDP
;
;
SHOW_BLOCK      PROC    NEAR
; Display a pre-formatted block message
; On entry: SI --> start of block message
; Message format:
;       Offset:         Function:
;              0 -----> Display row for start of message
;              1 -----> Display column for start of message
;              2 -----> Attribute for block
; Embedded codes:
;              00H ---> End of message
;              FFH ---> Index to next row
;
        MOV     DH,[SI]         ; Get row
        INC     SI              ; Point to start column byte
        MOV     DL,[SI]         ; Column
        MOV     CL,DL           ; Save start column in CL
        CALL    SET_DI          ; Set DI to offset in buffer
        INC     SI              ; Point to attribute byte
        MOV     AH,[SI]         ; To AH
        INC     SI
;
GET_AND_SHOW:
        MOV     AL,[SI]         ; Character
; Test for embedded controls
        CMP     AL,00H          ; Terminator 
        JE      SHOW_END
        CMP     AL,0FFH         ; End of line
        JE      LINE_END
; Display it by storing character in video buffer
        CALL    CHR_TO_VIDEO
        INC     DL              ; Next column
BUMP_PTRS:
        CALL    SET_DI
        INC     SI              ; Bump message pointer
        JMP     GET_AND_SHOW
;
LINE_END:
        INC     DH              ; Bump row pointer
        MOV     DL,CL           ; Reset column to start column
        JMP     BUMP_PTRS
;
SHOW_END:
        RET
;
SHOW_BLOCK      ENDP
;
;
SET_DI          PROC    NEAR
; Set DI to offset address in video buffer according to row and
; column positions
;             DI =  (row * 160) + (column * 2)
; On entry:
;             DH = screen row
;             DL = screen column
;
        PUSH    AX              ; Save accumulator
        MOV     AL,160          ; Bytes per row
        MUL     DH              ; AX = 160 * DH
        MOV     DI,AX           ; Save partial result in DI
        MOV     AL,2            ; Bytes per column
        MUL     DL              ; AX = DL * 2
        ADD     DI,AX           ; Add in previous result
        POP     AX              ; And accumulator
        RET
;
SET_DI          ENDP
;
;
CHR_TO_VIDEO    PROC    NEAR
; Display character in AL and attribute in AH by storing AX in
; the video buffer by ES:DI using a block move
        CLD
        STOSW
        RET
;
CHR_TO_VIDEO    ENDP
;
;
SETES_VIDEO     PROC    NEAR
; Set segment ES to video buffer address
        MOV     AX,0B800H
        MOV     ES,AX
        RET
;
SETES_VIDEO     ENDP
;
;
SETES_DATA      PROC    NEAR
; Set segment ES to data segment
        PUSH    DS
        POP     ES
        RET
;
SETES_DATA      ENDP
;
;
CODE    ENDS
        END     START
