;*****************************************************************
;*****************************************************************
;                        SVGADEMO.ASM
;*****************************************************************
;*****************************************************************
;
; Description:
; Demonstration of routines in the SVGA library and of the use
; of the VGA library services in a VESA SuperVGA mode
;
;
; In the SVGA module of GRAPHSOL.LIB
EXTRN   VESA_MODE:      FAR
EXTRN   VESA_105:       FAR
EXTRN   SVGA_PIX_105:   FAR
EXTRN   SVGA_CLS_105:   FAR
EXTRN   SVGA_READ_105:  FAR
;
; In other modules of GRAPHSOL.LIB
EXTRN   DO_4_QUADS:     FAR
EXTRN   INIT_X87:       FAR
EXTRN   CIRCLE:         FAR
EXTRN   ELLIPSE:        FAR
EXTRN   HYPERBOLA:      FAR
EXTRN   PARABOLA:       FAR
EXTRN   BRESENHAM:      FAR
EXTRN   SET_DEVICE:     FAR
EXTRN   FONT_TO_RAM:    FAR
EXTRN   FINE_TEXT:      FAR
EXTRN   MULTI_TEXT:     FAR
EXTRN   SET_DTA:        FAR
EXTRN   READ_HPFONT:    FAR
EXTRN   FINE_TEXTHP:    FAR
EXTRN   SET_MODE:       FAR
EXTRN   SUM_TO_GRAY:    FAR
EXTRN   TWO_BIT_IRGB:   FAR
;
; In SOLUTION.LIB
EXTRN   KBR_WAIT:       FAR
EXTRN   DOS_STRING_OUT: FAR
;
INCLUDELIB  GRAPHSOL
INCLUDELIB  SOLUTION
;
;****************************************************************
;                          stack segment
;****************************************************************
STACK   SEGMENT stack

                DB      0400H DUP ('?'); Default stack is 1K
STACK_TOP       EQU     THIS BYTE
;
STACK   ENDS
;
;*****************************************************************
;                        data segment
;*****************************************************************

DATA     SEGMENT  BYTE PUBLIC
;**********************|
;  WARNING messages    |
;**********************|
WARN_MSG        DB      '           ** W A R N I N G **'
        DB      0AH,0DH
        DB      'This program requires an 80x87 mathematical'
        DB      0AH,0DH
        DB      'coprocessor. If your system is not equipped'
        DB      0AH,0DH
        DB      'with this hardware, you can install the 80x87'
        DB      0AH,0DH
        DB      'Shareware emulator furnished in the microdisk'
        DB      0AH,0DH
        DB      '(see Appendix A). The program also requires a'
        DB      0AH,0DH
        DB      'SuperVGA card with VESA driver (see Chapter 8)'
        DB      0AH,0DH
        DB      'Press <Enter> to execute SVGADEMO.'
        DB      0AH,0DH
        DB      'Any other key to abort.'
        DB      0AH,0DH,'$'
;
NOVESA_MSG      DB      '** ERROR ** No VESA BIOS or required mode'
        DB      0AH,0DH
        DB      'Press any key to continue'
        DB      0AH,0DH,'$'
;**********************|
;   symetrical fonts   |
;**********************|
; RAM storage for symetrical font tables
; Each font table is preceeded by two bytes that determine its
; dimensions, as follows:
;   Byte at font table - 1 = number of pixel rows in each bit map
;   Byte at font table - 2 = number of horizontal bytes in each map
;
; 1 x 16 built in ROM font
                DB      1       ; Bit map x dimension, in bytes
                DB      16      ; Bit map y dimension, in bits
FONT_1X16       DB      4100 DUP (00H)
;
TEST_MSGA       DW      320     ; Start pixel column (0 to 639)
                DW      100     ; Start pixel row (0 to 479)
                DW      8       ; Horizontal pixel displacement
                DW      0       ; Vertical pixel displacement
                DB      00000000B       ; Color code
                DB      'Conic curves display demo',00H
                DW      0
;**********************|
;   HP printer fonts   |
;**********************|
; In these fonts the bit maps are of variable size, therefore no
; dimension are is implemented
HP_LG12         DB      32000 DUP ('!')
;
;**********************|
;  multi-line message  |
;**********************|
TEST_MSGD       DW      320     ; Start pixel column (0 to 639)
                DW      10      ; Start pixel row (0 to 479)
                DW      8       ; Horizontal pixel displacement
                DW      12      ; Line spacing
                DB      00001010B       ; Color code
                DB      'First line',0FFH
                DB      'Second line',0FFH
                DB      'Last line',0FFH
                DB      00H
;**********************|
;     HP font data     |
;**********************|
FONT_FILE       DB      'AK090UHP.SFP',00H
DTA_BUF         DB      128 DUP (00H)   ; Disk tranfer area
                DW      0
;**********************|
; fine grid messages   |
;**********************|
TEST_MSG4       DW      10      ; Start pixel column (0 to 1024)
                DW      600     ; Start pixel row (0 to 768)
                DW      2       ; Horizontal pixel displacement
                DW      0       ; Vertical pixel displacement
                DB      00001001B       ; Color code
                DB      'Text display demo using an HP soft font'
                DB      0H
;
DATA    ENDS
;*****************************************************************
;                        code segment
;*****************************************************************
;
CODE    SEGMENT
        ASSUME CS:CODE, DS:DATA


;*****************************************************************
;                         processing
;*****************************************************************
START:
.386
;*******************|
;  initialization   |
;*******************|
; Establish data and extra segment addressability
        MOV     AX,DATA         ; Address of DATA to AX
        MOV     DS,AX           ; and to DS
        ASSUME  DS:DATA         ; Assume from here on
;**********************|
;  display WARNING     |
;**********************|
        LEA     DX,WARN_MSG     ; Pointer to text message
        CALL    DOS_STRING_OUT  ; Procedure in SOLUTION.LIB
; wait for user keystroke and test for <Enter> key to continue
        CALL    KBR_WAIT        ; Procedure in SOLUTION.LIB
        CMP     AL,0DH          ; Test for <Enter> key pressed
        JE      OK_2START       ; Continue if <Enter> pressed
        JMP     DOS_EXIT        ; Abort execution if not <Enter>
;
;**********************|
;    init SuperVGA     |
;**********************|
OK_2START:
        MOV     CX,105H
        CALL    VESA_MODE
        JNC     OK_VESA         ; Continue if no error
;**********************|
;    VESA ERROR EXIT   |
;**********************|
BAD_VESA:
        LEA     DX,NOVESA_MSG   ; Pointer to text message
        CALL    DOS_STRING_OUT  ; Procedure in SOLUTION.LIB
        CALL    KBR_WAIT
        JMP     DOS_EXIT
OK_VESA:
; Set VESA mode 105H for 1024 by 768 pixels in 256 colors
        CALL    VESA_105
        JC      BAD_VESA        ; Go if not mode 105H
; Set display switch to SuperVGA device
        MOV     AL,'S'          ; Device = SVGA
        CALL    SET_DEVICE
;**********************|
;  load 8 x 16 font    |
;**********************|
        MOV     AL,16           ; Load 8 by 16 font
        LEA     DI,FONT_1X16    ; RAM storage for font
; DS:DI --> Start of RAM buffer for font
        CALL    FONT_TO_RAM     ; Local procedure
; Set video buffer base address
        MOV     AX,0A000H
        MOV     ES,AX
;
; Clear screen
        MOV     AL,00000111B    ; Color
        CALL    SVGA_CLS_105
;
; Test for setting a pixel in the center of the screen
        MOV     CX,512
        MOV     DX,388
        MOV     BL,00000100B    ; Red
        CALL    SVGA_PIX_105
;
; Test for reading a pixel in the center of the screen
        MOV     CX,512
        MOV     DX,388
        CALL    SVGA_READ_105
;
; Initialize mathematical coprocessor
        CALL    INIT_X87
;
;-----------------|
; x/y axis lines  |
;    in black     |
;-----------------|
; Leave 60 pixels clear on the horizontal axis
        MOV     BX,60
        MOV     CX,240
        MOV     SI,580
        MOV     DI,240
        MOV     AL,00000000B
        CALL    BRESENHAM
;
;------------------|
;   red circles    |
;------------------|
        MOV     BX,150
        CALL    CIRCLE
;
        MOV     BX,320
        MOV     CX,240
        MOV     AL,00000100B    ; Red
        CALL    DO_4_QUADS
;
;------------------|
;  blue ellipses   |
;------------------|
        MOV     BX,150
        MOV     CX,100
        CALL    ELLIPSE
        MOV     BX,320
        MOV     CX,240
        MOV     AL,00000001B    ; Blue
        CALL    DO_4_QUADS
;
;
        MOV     BX,100
        MOV     CX,150
        CALL    ELLIPSE
        MOV     BX,320
        MOV     CX,240
        MOV     AL,00000001B    ; Blue
        CALL    DO_4_QUADS
;
;------------------|
; green hyperbolas |
;------------------|
        MOV     BX,100
        MOV     CX,60
        MOV     DX,150
        CALL    HYPERBOLA
;
        MOV     BX,320
        MOV     CX,240
        MOV     AL,00000010B
        CALL    DO_4_QUADS
;
;------------------|
; brown parabola   |
;------------------|
        MOV     BX,20
        MOV     CX,140
        CALL    PARABOLA
;
        MOV     BX,320
        MOV     CX,240
        MOV     AL,00000110B
        CALL    DO_4_QUADS
;
;
;***********************|
; display messages at a |
;    pixel boundary     |
;***********************|
        LEA     SI,TEST_MSGA
        LEA     DI,FONT_1X16
        CALL    FINE_TEXT
;
        LEA     SI,TEST_MSGD
        LEA     DI,FONT_1X16
        CALL    MULTI_TEXT
;
;*****************************************************************
;                  load and display an HP soft font
;*****************************************************************
; Test font is Letter Gothic, portrait, 12 pitch
; Filename is : LG120R12.LPG (must be in current directory)
; Set Disk transfer Area at the buffer named DTA_BUF
        LEA     DX,DTA_BUF      ; DTA buffer in RAM
        CALL    SET_DTA
;
        LEA     DX,FONT_FILE    ; Buffer with ASCIIZ filename
        LEA     DI,HP_LG12      ; Storage buffer for HP font
        CALL    READ_HPFONT     ; Local procedure
;
        LEA     SI,TEST_MSG4    ; Pointer to message
        LEA     DI,HP_LG12      ; Offset of font table
        CALL    FINE_TEXTHP
;
;
        CALL    KBR_WAIT
;
;**********************|
;   change to gray     |
;**********************|
; Testing the SUM_TO_GRAY procedure in the VGA1 module
        CALL    SUM_TO_GRAY
        CALL    KBR_WAIT
;
        CALL    TWO_BIT_IRGB
        CALL    KBR_WAIT
;
;
;*******************|
;   exit to DOS     |
;*******************|
DOS_EXIT:
        MOV     AL,3
        CALL    SET_MODE
        MOV     AH,4CH          ; DOS service request code
        MOV     AL,0            ; No error code returned
        INT     21H             ; TO DOS
;
CODE    ENDS
        END     START

 AL,0            ; No error code returned
        INT     21H     