;****************************************************************
;****************************************************************
;                          TEXTDEMO.ASM
;****************************************************************
;****************************************************************
;
; Demonstration program for the text display procedures in the
; VGA2 module of the GRAPHSOL library. The procedures can also be
; used to display bitmaps (see Chapter 4)
;
;****************************************************************
;                           externals
;****************************************************************
; Procedures in SOLUTION.LIB
EXTRN   KBR_WAIT:       FAR
EXTRN   BIO_SET_MODE:   FAR
EXTRN   SET_WRITE_MODE: FAR
EXTRN   SET_DTA:        FAR
;
; Procedures in GRAPHSOL.LIB
EXTRN   TILE_FILL_18:   FAR
EXTRN   FINE_TEXT:      FAR
EXTRN   FINE_TEXTHP:    FAR
EXTRN   READ_HPFONT:    FAR
EXTRN   COARSE_TEXT:    FAR
EXTRN   FONT_TO_RAM:    FAR
EXTRN   MULTI_TEXT:     FAR
EXTRN   MONO_MAP_18:    FAR
EXTRN   COLOR_MAP_18:   FAR
;
INCLUDELIB SOLUTION
INCLUDELIB GRAPHSOL
;***************************** stack ****************************
;
STACK   SEGMENT stack
;
               DB      0400H DUP ('?'); Default stack is 1K
STACK_TOP      EQU     THIS BYTE
;
STACK   ENDS
;***************************** data *****************************
;
DATA    SEGMENT BYTE    PUBLIC
;
;**********************|
;   symmetrical fonts  |
;**********************|
; RAM storage for symmetrical font tables
; Each font table is preceded by two bytes that determine its
; dimensions, as follows:
;   Byte at font table - 1 = number of pixel rows in each bit map
;   Byte at font table - 2 = number of horizontal bytes in each map
;
; 1 x 8 built in ROM font
                DB      1       ; Bit map x dimension, in bytes
                DB      8       ; Bit map y dimension, in bits
FONT_1X8        DB      2050 DUP (00H)
;
; 1 x 16 built in ROM font
                DB      1       ; Bit map x dimension, in bytes
                DB      16      ; Bit map y dimension, in bits
FONT_1X16       DB      4100 DUP (00H)
;
;**********************|
;   HP printer fonts   |
;**********************|
; In these fonts the bit maps are of variable size, therefore no
; font dimension are provided
HP_LG12         DB      32000 DUP ('!')
;
;**********************|
; coarse grid messages |
;**********************|
TEST_MSG        DB      2       ; Start column
                DB      2       ; Start row
                DB      00000000B       ; Color code
                DB      'ABCDEFGHIJ',0FFH
                DB      'This is an 8 x 8 font',0FFH
                DB      0C9H,0CDH,0CDH,0CDH,0CDH,0BBH,0FFH
                DB      0BAH,020H,020H,020H,020H,0BAH,0FFH
                DB      0BAH,020H,020H,020H,020H,0BAH,0FFH
                DB      0BAH,020H,020H,020H,020H,0BAH,0FFH
                DB      0C8H,0CDH,0CDH,0CDH,0CDH,0BCH,0FFH
                DB      'End of test',0FFH
                DW      0
;
TEST_MSG1       DB      2       ; Start column
                DB      10      ; Start row
                DB      00000100B       ; Color code
                DB      'ABCDEFGHIJ abcdefghij',0FFH
                DB      'This is an 8 x 16 font',0FFH
                DB      0C9H,0CDH,0CDH,0CDH,0CDH,0BBH,0FFH
                DB      0BAH,020H,020H,020H,020H,0BAH,0FFH
                DB      0BAH,020H,020H,020H,020H,0BAH,0FFH
                DB      0BAH,020H,020H,020H,020H,0BAH,0FFH
                DB      0C8H,0CDH,0CDH,0CDH,0CDH,0BCH,0FFH
                DB      'End of test',0FFH
                DW      0
;
TEST_MSG2       DB      2       ; Start column
                DB      20      ; Start row
                DB      00000000B       ; Color code
                DB      'ABCDEJabcdij 123456789',0FFH
                DB      00H
;
TEST_MSG3       DB      2       ; Start column
                DB      20      ; Start row
                DB      00000000B       ; Color code
                DB      '             ABCDEFGHI',0FFH
                DB      00H
; Note: TEST_MSG3 overlays TEST_MSG2 on the same video area
;       the effect can be seen on screen row number 20, column
;       number 16
;
;**********************|
; fine grid messages   |
;**********************|
TEST_MSG4       DW      10      ; Start pixel column (0 to 639)
                DW      380     ; Start pixel row (0 to 479)
                DW      2       ; Horizontal pixel displacement
                DW      0       ; Vertical pixel displacement
                DB      00000000B       ; Color code
                DB      'Text message using HP soft font'
                DB      0H
;
TEST_MSGA       DW      320     ; Start pixel column (0 to 639)
                DW      100     ; Start pixel row (0 to 479)
                DW      8       ; Horizontal pixel displacement
                DW      0       ; Vertical pixel displacement
                DB      00000100B       ; Color code
                DB      'A A This is a test',00H
                DW      0
;
TEST_MSGB       DW      320     ; Start pixel column (0 to 639)
                DW      120     ; Start pixel row (0 to 479)
                DW      8       ; Horizontal pixel displacement
                DW      8       ; Vertical pixel displacement
                DB      00000010B       ; Color code
                DB      'A B This is a test',00H
                DW      0
;
TEST_MSGC       DW      320     ; Start pixel column (0 to 639)
                DW      300     ; Start pixel row (0 to 479)
                DW      8       ; Horizontal pixel displacement
                DW      -8       ; Vertical pixel displacement
                DB      00000010B       ; Color code
                DB      'A C This is a test',00H
                DW      0
;
;**********************|
;  multi-line message  |
;**********************|
TEST_MSGD       DW      320     ; Start pixel column (0 to 639)
                DW      10      ; Start pixel row (0 to 479)
                DW      8       ; Horizontal pixel displacement
                DW      12      ; Line spacing
                DB      00000000B       ; Color code
                DB      'First line',0FFH
                DB      'Second line',0FFH
                DB      'Last line',0FFH
                DB      00H
;
;**********************|
;     HP font data     |
;**********************|
FONT_FILE       DB      'AK090UHP.SFP',00H
DTA_BUF         DB      128 DUP (00H)   ; Disk transfer area
                DW      0
;
;**********************|
;   bit map of boar    |
;**********************|
; Block control area:                      Displacement -->
BOAR    DW      4           ; Present x coordinate         0
LPIG_Y  DW      440         ; y coordinate                 2
        DB      19          ; Horizontal rows in block     4
        DB      6           ; Number of bytes per row      5
;
; Symbol encoding for left-hand boar
        DB      01FH,080H,00FH,0FFH,0F0H,000H ; 1
        DB      000H,043H,0F0H,081H,00EH,000H ; 2
        DB      000H,03CH,001H,03CH,081H,000H ; 3
        DB      000H,040H,002H,042H,040H,0C0H ; 4
        DB      000H,040H,004H,099H,020H,030H ; 5
        DB      000H,080H,005H,024H,0A0H,00CH ; 6
        DB      000H,080H,005H,05AH,0A0H,003H ; 7
        DB      000H,080H,005H,05AH,0A0H,001H ; 8
        DB      007H,000H,005H,024H,0A0H,01EH ; 9
        DB      008H,000H,004H,099H,020H,060H ; 10
        DB      008H,000H,002H,042H,047H,080H ; 11
        DB      010H,000H,001H,03CH,088H,000H ; 12
        DB      028H,000H,000H,081H,007H,080H ; 13
        DB      05FH,0C1H,0F0H,03FH,000H,040H ; 14
        DB      0FCH,03EH,00FH,0FCH,000H,0B0H ; 15
        DB      014H,000H,000H,002H,061H,060H ; 16
        DB      024H,000H,000H,001H,099H,000H ; 17
        DB      078H,000H,000H,000H,006H,080H ; 18
        DB      000H,000H,000H,000H,001H,0C0H ; 19
        DW      0000H
;
BOAR_COLOR      DB      00000100B       ; R on
;
;**********************|
;   crosshair symbol   |
;**********************|
; Block control area:                      Displacement -->
CROSS_X DW      80          ; Present x coordinate         0
                            ; range = 0 to 630
CORSS_Y DW      440         ; Present y coordinate         2
                            ; range = 360 to 460
        DB      15          ; Horizontal rows in block     4
        DB      2           ; Number of bytes per row      5
;
; Symbol encoding of crosshair symbol
        DB      07H,0E0H,11H,10H,21H,08H,41H,04H,81H,02H
        DB      81H,02H,81H,02H,0FFH,0FFH,81H,02H,81H,02H
        DB      81H,02H,41H,04H,21H,08H,11H,10H,07H,0E0H
        DW      0000H       ; Safety padding
;
; Color encoding of crosshair symbol
CR_COL  DB      0,0,0,0,0,0FH,0FH,0FH,0FH,0FH,0FH,0,0,0,0,0
        DB      0,0,0,0FH,0,0,0,0FH,0,0,0,0FH,0,0,0,0
        DB      0,0,0FH,0,0,0,0,0FH,0,0,0,0,0FH,0,0,0
        DB      0,0FH,0,0,0,0,0,0FH,0,0,0,0,0,0FH,0,0
        DB      0FH,0,0,0,0,0,0,0CH,0,0,0,0,0,0,0FH,0
        DB      0FH,0,0,0,0,0,0,0CH,0,0,0,0,0,0,0FH,0
        DB      0FH,0,0,0,0,0,0,0CH,0,0,0,0,0,0,0FH,0
        DB      4 DUP(0FH),0CH,0CH,0CH,0,0CH,0CH,0CH,4 DUP(0FH),0
        DB      0FH,0,0,0,0,0,0,0CH,0,0,0,0,0,0,0FH,0
        DB      0FH,0,0,0,0,0,0,0CH,0,0,0,0,0,0,0FH,0
        DB      0FH,0,0,0,0,0,0,0CH,0,0,0,0,0,0,0FH,0
        DB      0,0FH,0,0,0,0,0,0FH,0,0,0,0,0,0FH,0,0
        DB      0,0,0FH,0,0,0,0,0FH,0,0,0,0,0FH,0,0,0
        DB      0,0,0,0FH,0,0,0,0FH,0,0,0,0FH,0,0,0,0
        DB      0,0,0,0,0,0FH,0FH,0FH,0FH,0FH,0FH,0,0,0,0,0
;
;**********************|
;     error messages   |
;**********************|
FILE_ERR        DB      0       ; Start column
                DB      29      ; Start row
                DB      00000000B       ; Color code
                DB      'ERROR - Open Failed',00H
;
DATA    ENDS
;****************************************************************
;
CODE    SEGMENT BYTE
        ASSUME  CS:CODE,SS:STACK
;
START_MAIN:
;**********************|
;    initialize DS     |
;**********************|
        MOV     AX,DATA         ; AX to module data
        MOV     DS,AX           ; and to DS
        ASSUME  DS:DATA
;**********************|
;  load 8 x 8 font     |
;**********************|
        MOV     AL,8            ; Load 8 by 8 font
        LEA     DI,FONT_1X8     ; RAM storage for font
; DS:DI --> Start of RAM buffer for font
        CALL    FONT_TO_RAM     ; Local procedure
;
;**********************|
;  load 8 x 16 font    |
;**********************|
        MOV     AL,16           ; Load 8 by 16 font
        LEA     DI,FONT_1X16    ; RAM storage for font
; DS:DI --> Start of RAM buffer for font
        CALL    FONT_TO_RAM     ; Local procedure
;**********************|
;   initialize video   |
;     buffer address   |
;**********************|
        MOV     AX,0A000H       ; AX to base address of video buffer
        MOV     ES,AX           ; To ES
;
        MOV     AH,0            ; BIOS service request code
        MOV     AL,18           ; VGA mode number 18. 640 by 480
                                ; pixels in 16 colors
        INT     10H             ; BIOS video interrupt
;**********************|
;   set write mode 2   |
;**********************|
; This program uses write mode 2 for graphics display. In this write
; mode the color code is contained in a CPU register
        MOV     AL,2            ; Mode number
        CALL    SET_WRITE_MODE
;**********************|
;  draw color blocks   |
;**********************|
; Fill entire screen
        MOV     CH,0            ; Start column
        MOV     CL,0            ; Start row
        MOV     DH,80           ; Horizontal bytes
        MOV     DL,60           ; Vertical bytes
        MOV     AL,00000111B    ; Color = gray
        CALL    TILE_FILL_18
; Fill screen area
        MOV     CH,40           ; Start column
        MOV     CL,50           ; Start row
        MOV     DH,10           ; Horizontal bytes
        MOV     DL,8            ; Vertical bytes
        MOV     AL,00001100B    ; Color = bright red
        CALL    TILE_FILL_18
;**********************|
; display boar bit map |
;**********************|
        LEA     SI,BOAR         ; Pointer to bit map
        LEA     BX,BOAR_COLOR   ; Pointer to color byte
        CALL    MONO_MAP_18     ; Procedure in VGA2
;**********************|
;   display crosshair  |
;    (color encoded)   |
;**********************|
        LEA     SI,CROSS_X      ; Pointer to bit map
        LEA     BX,CR_COL       ; Pointer to color map
        CALL    COLOR_MAP_18    ; Procedure in VGA2
;
;***********************|
;   display messages    |
;  at a course boundary |
;***********************|
        LEA     SI,TEST_MSG
        LEA     DI,FONT_1X8
        CALL    COARSE_TEXT
;
        LEA     SI,TEST_MSG1
        LEA     DI,FONT_1X16
        CALL    COARSE_TEXT
;
        LEA     SI,TEST_MSG2
        LEA     DI,FONT_1X16
        CALL    COARSE_TEXT
;
; The following message overlays the previous one
        LEA     SI,TEST_MSG3
        LEA     DI,FONT_1X16
        CALL    COARSE_TEXT
;
;***********************|
; display messages at a |
;    pixel boundary     |
;***********************|
        LEA     SI,TEST_MSGA
        LEA     DI,FONT_1X16
        CALL    FINE_TEXT
;
        LEA     SI,TEST_MSGB
        LEA     DI,FONT_1X16
        CALL    FINE_TEXT
;
        LEA     SI,TEST_MSGC
        LEA     DI,FONT_1X16
        CALL    FINE_TEXT
;
        LEA     SI,TEST_MSGD
        LEA     DI,FONT_1X8
        CALL    MULTI_TEXT
;
;*****************************************************************
;             load and display an HP-format soft font
;*****************************************************************
; Note: test font file must be in current directory
;
; Set Disk transfer Area at the buffer named DTA_BUF
        LEA     DX,DTA_BUF      ; DTA buffer in RAM
        CALL    SET_DTA
;
        LEA     DX,FONT_FILE    ; Buffer with ASCIIZ filename
        LEA     DI,HP_LG12      ; Storage buffer for HP font
        CALL    READ_HPFONT     ; Local procedure
;
        LEA     SI,TEST_MSG4    ; Pointer to message
        LEA     DI,HP_LG12      ; Offset of font table
        CALL    FINE_TEXTHP
;
DOS_EXIT:
        CALL    KBR_WAIT
        MOV     AL,0            ; Mode number
        CALL    SET_WRITE_MODE
        MOV     AL,3            ; Set alphanumeric mode 3
        CALL    BIO_SET_MODE    ; Procedure in SOLUTION.LIB
;
        MOV     AH,4CH          ; DOS service request code
        MOV     AL,0            ; No error code returned
        INT     21H             ; exit to MS DOS
;
CODE    ENDS
        END     START_MAIN
