;*****************************************************************
;*****************************************************************
;                        TIFFSHOW.ASM
;*****************************************************************
;*****************************************************************
;
; Image display program for Class B TIFF files using VGA mode 18
; (see Chapter 10)
;*****************************************************************
;
; Notes:
; 1. Class B TIFF files hold a single black-and-white pixel map
; 2. The following TIFF tags are handled by the software:
;    TIFF CODE   NAME             REQ VALUE/RANGE
;    254 (FEH)   NewSubfileType       0
;    255 (FFH)   OldSubfileType       1
;    256 (100H)  ImageWidth         * horizontal pixels (0 to 640)
;    257 (101H)  ImageLength        * vertical pixels (0 to 480)
;    258 (102H)  BitsPerSample        1
;    277 (115H)  SamplesPerPixel      1
;    259 (103H)  Compression          1 (no compression)
;                                     8005H (PackBits)
;    262 (106H)  PhotometricInterpet  1 (output 1-bits as black)
;    273 (111H)  StripOffsets       * offset of bitmap from start
;                                     of file. If length = 1 then
;                                     then data field holds offset
;                                     of the single bit map
;    278 (116H)  RowsPerStrip         Number of rows in strip
;                                     (not present if only 1 strip)
;    279 (117H)  StripByteCounts      Number of bytes in each strip
;                                     (not present if only 1 strip)
;    296 (128H)  ResolutionUnit       Units of image measurements
;                                     1 = no absolute unit
;                                     2 = inch
;                                     3 = centimeter
; 3. All other TIFF tags are ignored
;
;******************************************************************
;                      equates for TIFF tags
;******************************************************************
NEWSUBFILETYPE  EQU     00FEH
OLDSUBFILETYPE  EQU     00FFH
IMAGEWIDTH      EQU     0100H
IMAGELENGTH     EQU     0101H
BITSPERSMAPLE   EQU     0102H
COMPRESSION     EQU     0103H
PHOTOINTERP     EQU     0106H
STRIPOFFSETS    EQU     0111H
SAMPLESPERPIXEL EQU     0115H
RESOLUTIONUNIT  EQU     0128H
;
;*******************************************************************
;                   procedures in SOLUTION.LIB
;*******************************************************************
EXTRN   KBR_FLUSH:      FAR
EXTRN   KBR_STATUS:     FAR
EXTRN   KBR_WAIT:       FAR
EXTRN   DOS_STRING_OUT: FAR
EXTRN   BIN_TO_ASCHEX:  FAR
EXTRN   BIN_TO_ASC:     FAR
EXTRN   BIO_SET_MODE:   FAR
EXTRN   BIO_SET_CUR:    FAR
EXTRN   BIO_CLEAR_SCR:  FAR
EXTRN   BIO_SHOW_MESS:  FAR
EXTRN   PIXEL_ADD_18:   FAR
EXTRN   WRITE_PIX_18:   FAR
EXTRN   SET_WRITE_MODE: FAR
EXTRN   OPEN_FILE:      FAR
EXTRN   READ_128:       FAR
EXTRN   CLOSE_FILE:     FAR
EXTRN   SET_DTA:        FAR
EXTRN   OPEN_CREATE:    FAR
EXTRN   CLS_18:         FAR
;
INCLUDELIB SOLUTION
INCLUDELIB GRAPHSOL
;
;****************************************************************
;                          stack segment
;****************************************************************
STACK   SEGMENT stack

                DB      0400H DUP ('?'); Default stack is 1K
STACK_TOP       EQU     THIS BYTE
;
STACK   ENDS
;
;****************************************************************
;                          data segment
;****************************************************************
DATA    SEGMENT
;**********************|
; storage for TIFF file|
;**********************|
TIFF_FILE       DB      20000 DUP (00H)
;**********************|
; general use variables|
;     and buffers      |
;**********************|
ASCII_BUF       DB      5 DUP (20H)
IFD_OFFSET      DW      0               ; Offset of first IFD
                                        ; (from header data)
TAG_ADDRESS     DW      0       ; Start address of current tag
TAG1_ADDRESS    DW      0       ; Start of first tag in IFD
TAG_CODE        DW      0       ; Tag code storage
; Buffer with tag codes and associated names
; Note: Each entry consists of 2 code bytes and 22 ASCII characters
NAMES_BUFFER    DW      00FEH
                DB      'NewSubfileType        '
                DW      0100H
                DB      'ImageWidth            '
                DW      0101H
                DB      'ImageLength           '
                DW      011AH
                DB      'XResolution           '
                DW      011BH
                DB      'YResolution           '
                DW      0111H
                DB      'StripsOffset          '
                DW      0117H
                DB      'StripByteCounts       '
                DW      0116H
                DB      'RowsPerStrip          '
                DW      0115H
                DB      'SamplesPerPixel       '
                DW      0102H
                DB      'BitsPerSample         '
                DW      0107H
                DB      'Treshholding          '
                DW      0106H
                DB      'PhotometricInterpretat'
                DW      0103H
                DB      'Compression           '
                DW      00FFH
                DB      'OldSubfileType        '
                DW      011CH
                DB      'PlanarConfiguration   '
                DW      0128H
                DB      'ResolutionUnit        '
                DW      0000H
                DB      '                      '
;**********************|
;   input message      |
;**********************|
INPUT_MSG       DB      'GRAPHICS PROGRAMMING SOLUTIONS by'
                DB      ' Julio Sanchez and Maria P. Canton'
                DB      0AH,0DH
                DB      'Header, tag contents, and image'
                DB      ' display of a bi-level TIFF image file'
                DB      0AH,0DH
                DB      'Enter filename: '
                DB      00H
;**********************|
;    header message    |
;**********************|
HEADER_MSG      DB      0AH,0AH,0DH
                DB      'FILE HEADER FIELD             '
                DB      'Byte order: '
INTEL_MOTOROLA  DB      'II',0AH,0DH    ; Default is Intel order
                DB      'Version Number: '
VERSION_NO      DB      '2AH           '
                DB      'IFD Offset: '
OFF_DIGITS      DB      '           ',0AH,0DH
                DB      'Number of tags: '
TAG_NUM_MS      DB      '     ',0AH,0AH,0DH
                DB      00H
;**********************|
;    tag messages      |
;**********************|
TAGHEAD_MSG     DB      'Offset  Tag Code   Tag Name              '
                DB      'TYPE   LENGTH        VALUE/OFFSET'
                DB      0AH,0DH,00H
;
TAG_OFF_MS      DB      '        '
TAG_CODE_MS     DB      '           '
TAG_NAME_MS     DB      '                      '
TAG_TYPE_MS     DB      '       '
TAG_LENGTH_MS   DB      '              '
TAG_VALUE_MS    DB      '             '
                DB      0AH,0DH,00H
;**********************|
;   TIFF file storage  |
;**********************|
DOS_BUFFER      DB      14
                DB      16 DUP (00H)
;
HANDLE_1        DW      0       ; File handle
;
DTA_BUF         DB      128 DUP (00H)   ; Disk transfer area
                DW      0
DATA_BUF        DB      128 DUP (00H)   ; Disk data storage area
                DW      0
;**********************|
;  TIFF parameters and |
;      variables       |
;**********************|
; The following variables are used for storing the binary values
; contained in the TIFF tag fields
; Variables:
BYTE_WIDTH      DB      0       ; Horizontal image dimension
                                ; in bytes
PIX_LENGTH      DW      0       ; Vertical image dimension
                                ; in pixels
IMAGE_SIZE      DW      0       ; Number of bytes in a single
                                ; strip image bit map
PACKBITS        DB      0       ; 0 = file un-compressed
                                ; 1 = PackBits
STRIP_ROWS      DW      0       ; Number of pixel rows per strip
STRIP_COUNT     DW      0       ; Total number of strips
                                ; (from the length field of the
                                ; StripOffset tag)
STRIP_OFFSET    DW      0       ; Offset of bit map if STRIP_COUNT
                                ; = 1, else, offset of strip
                                ; pointers
;
; Current strip pointers:
OFFSET_PTR      DW      0       ; Pointer to current position in
                                ; OFFSETS_BUF
BCOUNT_PTR      DW      0       ; Pointer to current position in
                                ; BCOUNT_BUF
; Buffers:
OFFSETS_BUF     DB      80 DUP (00H)    ; Buffer for 20 strip
                                        ; offsets of 4-bytes each
BCOUNT_BUF      DB      80 DUP (00H)    ; Buffer for 20 strip byte
                                        ; counts of 4-bytes each
;**********************|
;   parameter message  |
;**********************|
PARAMETER_MSG   DB      0AH,0AH,0DH
                DB      'IMAGE PARAMETERS:',0AH,0DH
                DB      'Byte width (x coordinate): '
BYTE_WID_MS     DB      '     ',0AH,0DH
                DB      'Number of pixel rows (y coordinate): '
PIX_ROWS_MS     DB      '     ',0AH,0DH
                DB      'Bit map offset: '
MAP_OFFSET_MS   DB      '     ',0AH,0DH
                DB      'Image size (in bytes): '
IMAGE_SIZE_MS   DB      '     ',0AH,0DH
                DB      'Number of strips: '
STRIPS_MS       DB      '     ',0AH,0DH
                DB      'Compression: '
COMPRESS_MS     DB      'Pack Bits    '
                DB      0AH,0DH,00H
;
UNPACKED_MS     DB      'Un-compressed '
;
;**********************|
;   wait messages      |
;**********************|
WAIT1_MS        DB      0AH,0DH
                DB      'Press any key for image parameters',00H
WAIT2_MS        DB      0AH,0DH
                DB      'Press any key for bit map display ',00H
;**********************|
;    error messages    |
;**********************|
FILE_ERR        DB      0AH,0DH
                DB      'ERROR - Open Failed',00H
TIFF_ERR        DB      0AH,0DH
                DB      'ERROR - Required TIFF tag missing',00H
STRIP_ERR       DB      0AH,0DH
                DB      'ERROR - Cannot display multi-image',00H
COMP_ERR        DB      0AH,0DH
                DB      'ERROR - Unsupported compression',00H
;
;*****************************************************************
;                     image display area
;*****************************************************************
IMAGE_BLOCK     DW      0       ; x coordinate
                DW      0       ; y coordinate
BLOCK_ROWS      DW      0       ; Number of pixel rows
BLOCK_COLS      DB      0       ; Number of bytes per row
;
BIT_IMAGE       DB      38400 DUP (00H)
;
IMAGE_COLOR     DB      00000000B       ; I R G B color code
EXP_COUNT       DW      0       ; Byte counter for expansion
;
DATA    ENDS
;****************************************************************
;                          code segment
;****************************************************************
;
CODE    SEGMENT
        ASSUME  CS:CODE
;
START:
;*******************|
;  initialization   |
;*******************|
; Establish data and extra segment addressability
        MOV     AX,DATA         ; Address of DATA to AX
        MOV     DS,AX           ; and to DS
        ASSUME  DS:DATA         ; Assume from here on
        CALL    BIO_CLEAR_SCR   ; Clear screen procedure in
                                ; SOLUTION.LIB
        MOV     DX,0            ; Row 0, column 0
        CALL    BIO_SET_CUR     ; Cursor procedure in
                                ; SOLUTION.LIB
;*********************|
; program preparations|
;*********************|
        LEA     SI,INPUT_MSG
        CALL    BIO_SHOW_MESS
;*********************|
;  input filename     |
;  and open file      |
;*********************|
        LEA     DX,DOS_BUFFER
        MOV     BL,0            ; Code for open operation
        CALL    OPEN_CREATE     ; Procedure in SOLUTION.LIB
        JNC     OK_OPEN         ; Go if no error code
;**********************|
;    disk open error   |
;**********************|
; Open operation failed. Post error message and exit
        LEA     SI,FILE_ERR
        CALL    BIO_SHOW_MESS
        CALL    KBR_WAIT
        JMP     DOS_EXIT
;**********************|
;   read TIFF file     |
;      into RAM        |
;**********************|
OK_OPEN:
        LEA     DI,TIFF_FILE    ; Storage buffer for file
        MOV     HANDLE_1,AX     ; Store file handle
NEW_128:
        MOV     BX,HANDLE_1
        LEA     DX,DATA_BUF     ; Buffer for data storage
        PUSH    DI              ; Save buffer pointer
        CALL    READ_128        ; Read sector into buffer
        POP     DI              ; Restore buffer pointer
        CMP     AX,0            ; Test for end of file
        JNE     MOVE_128        ; Go if not at end of file
;**********************|
;    end of file       |
;**********************|
        MOV     BX,HANDLE_1     ; File handle
        CALL    CLOSE_FILE      ; Library routine
        JMP     END_OF_READ
;**********************|
;   move sector to     |
;    TIFF buffer       |
;**********************|
; At this point DATA_BUF holds 128 bytes from disk file
; DI --> storage position in the TIFF file RAM buffer
MOVE_128:
        MOV     CX,128          ; Byte counter
        LEA     SI,DATA_BUF     ; Pointer to data just read
PLACE_128:
        MOV     AL,[SI]         ; Byte from DATA_BUF
        MOV     [DI],AL         ; Into font's buffer
        INC     SI              ; Bump pointers
        INC     DI
        LOOP    PLACE_128       ; Continue until all sector read
; At this point the 128 bytes newly read from the disk file are
; stored in the font's buffer
        JMP     NEW_128
;******************************************************************
;                      display TIFF header
;******************************************************************
; Note: The TIFF header includes the number of tags encoded in the
;       entry count field of the IFD
;**********************|
;     byte order       |
;**********************|
END_OF_READ:
        LEA     SI,TIFF_FILE    ; Set source pointer
        MOV     AX,WORD PTR [SI]        ; Get byte order field
        CMP     AX,'II'         ; Test for default order
        JE      OK_INTEL        ; Go if Intel order
; Byte order is for Motorola processors
        LEA     DI,INTEL_MOTOROLA       ; Pointer to message area
        MOV     WORD PTR [DI],'MM'      ; Motorola order code
;**********************|
;    version number    |
;**********************|
OK_INTEL:
        MOV     AX,WORD PTR [SI+2]      ; Get version field
        CMP     AX,002AH        ; Normal code
        JE      VERSION_OK      ; Go if normal version code
; At this point there is an error in the version field
        LEA     DI,VERSION_NO           ; Pointer to message area
        MOV     WORD PTR [DI],'??'      ; Error in version code
;**********************|
;    offset of IFD     |
;**********************|
VERSION_OK:
; Get high_order word of offset and convert to ASCII hex
        MOV     DX,WORD PTR [SI+6]      ; High order word of offset
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASCHEX   ; Procedure in SOLUTION.LIB
        LEA     BX,OFF_DIGITS   ; Destination for 4 digits
        CALL    HEX_TO_BX       ; Local procedure
; Get low_order word of offset and convert to ASCII hex
        MOV     DX,WORD PTR [SI+4]      ; Low order word of offset
        MOV     IFD_OFFSET,DX   ; Store in variable
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASCHEX   ; Procedure in SOLUTION.LIB
        LEA     BX,OFF_DIGITS+6 ; Position for low order digits
        CALL    HEX_TO_BX       ; Local procedure
;**********************|
;     entry count      |
;**********************|
        LEA     SI,TIFF_FILE    ; Start of TIFF file
        ADD     SI,IFD_OFFSET   ; Add offset of first IFD
        MOV     DX,WORD PTR [SI]        ; Get entry count
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASC      ; Procedure in SOLUTION.LIB
; Move decimal ASCII digits to display area
        LEA     BX,TAG_NUM_MS   ; Display area
        CALL    DEC_TO_BX       ; Local procedure
;**********************|
;   display message    |
;**********************|
        LEA     SI,HEADER_MSG
        CALL    BIO_SHOW_MESS
;******************************************************************
;                        display TIFF tags
;******************************************************************
;**********************|
;  tag format message  |
;**********************|
        LEA     SI,TAGHEAD_MSG  ; Header for tag data
        CALL    BIO_SHOW_MESS
; Index to first tag
        LEA     SI,TIFF_FILE    ; Start of TIFF file
        ADD     SI,IFD_OFFSET   ; Add offset of first IFD
        ADD     SI,2            ; Skip entry count field
        MOV     TAG_ADDRESS,SI  ; Store start of tag field
;**********************|
;     tag offset       |
;**********************|
NEXT_TAG:
; Display tag offset in IFD
        MOV     DX,SI           ; SI holds offset
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASCHEX   ; Procedure in SOLUTION.LIB
; Set tag offset digits in message area
        LEA     BX,TAG_OFF_MS   ; Digit display area
        CALL    HEX_TO_BX       ; Local procedure
;**********************|
;     tag code         |
;**********************|
        MOV     DX,WORD PTR [SI]        ; Get tag code
        MOV     TAG_CODE,DX     ; Store code
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASCHEX   ; Procedure in SOLUTION.LIB
; Set tag code in display area
        LEA     BX,TAG_CODE_MS  ; Digit display area
        CALL    HEX_TO_BX       ; Local procedure
;**********************|
;      tag name        |
;**********************|
        LEA     SI,NAMES_BUFFER ; Buffer holding codes and names
        LEA     DI,TAG_NAME_MS  ; Destination for name string
FIND_NAME:
        MOV     AX,WORD PTR [SI]        ; Get code from table
        CMP     AX,0            ; Test for end of table
        JE      MOVE_NAME       ; Go if at end of table
        CMP     AX,TAG_CODE     ; Compare with tag code
        JE      MOVE_NAME       ; Go if tag code
; Tag code not matched. Index to next name
        ADD     SI,24           ; 2 code and 22 ASCII bytes
        JMP     FIND_NAME       ; Continue search
MOVE_NAME:
        MOV     CX,22           ; 22 ASCII characters in name
        ADD     SI,2            ; Skip code in buffer
MOVE_22:
        MOV     AL,[SI]         ; Get ASCII character
        MOV     [DI],AL         ; Place it in display buffer
        INC     SI              ; Bump pointers
        INC     DI
        LOOP    MOVE_22
;**********************|
;     tag type         |
;**********************|
        MOV     SI,TAG_ADDRESS  ; Re-load SI
        MOV     DX,WORD PTR [SI+2]        ; Get tag type
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASCHEX   ; Procedure in SOLUTION.LIB
; Set tag type in display area
        LEA     BX,TAG_TYPE_MS  ; Digit display area
        CALL    HEX_TO_BX       ; Local procedure
;**********************|
;     tag length       |
;**********************|
; Get and convert 4 tag length digits
        MOV     DX,WORD PTR [SI+6]        ; Get first 2 digits
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASCHEX   ; Procedure in SOLUTION.LIB
; Set low byte of tag length in display area
        LEA     BX,TAG_LENGTH_MS; Digit display area
        CALL    HEX_TO_BX       ; Local procedure
        MOV     DX,WORD PTR [SI+4]        ; Get next 2 digits
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASCHEX   ; Procedure in SOLUTION.LIB
; Set high byte of tag length in display area
        LEA     BX,TAG_LENGTH_MS+6      ; Digit display area
        CALL    HEX_TO_BX       ; Local procedure
;**********************|
;  tag value/offset    |
;**********************|
; Get and convert low order tag value/offset digits
        MOV     DX,WORD PTR [SI+10]       ; Get first 2 digits
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASCHEX   ; Procedure in SOLUTION.LIB
; Set low order digits in display area
        LEA     BX,TAG_VALUE_MS ; Digit display area
        CALL    HEX_TO_BX       ; Local procedure
        MOV     DX,WORD PTR [SI+8]         ; Get next 2 digits
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASCHEX   ; Procedure in SOLUTION.LIB
; Set high order digits in display area
        LEA     BX,TAG_VALUE_MS+6       ; Digit display area
        CALL    HEX_TO_BX       ; Local procedure
;**********************|
; display tag contents |
;**********************|
        LEA     SI,TAG_OFF_MS   ; Message with tag digits
        CALL    BIO_SHOW_MESS
;**********************|
;      next tag        |
;**********************|
        MOV     SI,TAG_ADDRESS  ; Offset of previous tag
        ADD     SI,12           ; Index to new tag
        MOV     TAG_ADDRESS,SI  ; Store new tag address
; Test for last tag
        MOV     AX,WORD PTR [SI]        ; Get data
        CMP     AX,0            ; Test for last IFD
        JE      LAST_IFD        ; Go if last
        JMP     NEXT_TAG        ; Continue
;*****************************************************************
;               obtain and display image parameters
;*****************************************************************
LAST_IFD:
; Wait for user to press any key
; Note: the following code re-obtains the display parameters
;       from the corresponding tags. This makes it independent
;       from the header and tag display routines
        LEA     SI,TIFF_FILE    ; Start of TIFF file
        MOV     DX,WORD PTR [SI+4]      ; Low order word of offset
        ADD     SI,DX           ; Add offset of first IFD
        ADD     SI,2            ; Skip entry count field
        MOV     TAG1_ADDRESS,SI ; Store this address
; SI ==> first tag (tag code field)
;**********************|
;     image width      |
;**********************|
        MOV     SI,TAG1_ADDRESS ; Address of first tag in IFD
        MOV     AX,IMAGEWIDTH   ; Get tag for image width
        CALL    FIND_TAG
        JNC     OK_WIDTH        ; Go if tag found
;**********************|
; TIFF file error exit |
;**********************|
BAD_TIFF_FILE:
        LEA     SI,TIFF_ERR
        CALL    BIO_SHOW_MESS
        CALL    KBR_WAIT
        JMP     DOS_EXIT
OK_WIDTH:
; Image width processing in the IBM VGA environment assumes
; type 3, length 1, and value in the low order word of the
; value/offset field
        MOV     AX,WORD PTR [SI+8]         ; Get low-order word
; Convert pixel width to byte width for display routine
        MOV     CL,8            ; Bit divisor
        DIV     CL              ; AX/8 = byte width
; AL = quotient and AH = remainder
; If remainder, increment byte count
        CMP     AH,0            ; Is there a remainder ?
        JE      NO_REM          ; Go if no remainder
        INC     AL              ; Bump quotient
NO_REM:
        MOV     BYTE_WIDTH,AL   ; Store width in variable
;**********************|
;      image length    |
;**********************|
        MOV     SI,TAG1_ADDRESS ; Address of first tag in IFD
        MOV     AX,IMAGELENGTH  ; Get tag for image width
        CALL    FIND_TAG
        JNC     OK_LENGTH       ; Go if tag found
        JMP     BAD_TIFF_FILE   ; Exit if tag missing
OK_LENGTH:
; Image length processing in the IBM VGA environment assumes
; type 3, length 1, and value in the low order word of the
; value/offset field
        MOV     AX,WORD PTR [SI+8]         ; Get low-order word
        MOV     PIX_LENGTH,AX           ; Store value in variable
;**********************|
;    image offset      |
;**********************|
; Note:
;     The number of strips in the image is obtained from the length
;     field of the StripOffsets tag
        MOV     SI,TAG1_ADDRESS ; Address of first tag in IFD
        MOV     AX,STRIPOFFSETS ; Get tag for strip offsets
        CALL    FIND_TAG
        JNC     OK_OFFSETS      ; Go if tag found
        JNC     OK_LENGTH       ; Go if tag found
OK_OFFSETS:
        MOV     AX,WORD PTR [SI+4]        ; Get number of strips
                                ; from length field
        MOV     STRIP_COUNT,AX  ; Store value in variable
        MOV     AX,WORD PTR [SI+8]         ; Get strip offset
        MOV     STRIP_OFFSET,AX ; Store in variable
;**********************|
;  strip parameters    |
;**********************|
; Image can be encoded in one or more strips. The number of strips
; has been is obtained from the length field of the StripOffsets
; tag and stored in the variable STRIP_COUNT
; Unpacking and display of multi-strip images requires the number
; of number of rows per strip and the number of bytes in each strip
; row. These parameters are not necessary if the image is encoded
; in a single strip
        CMP     STRIP_COUNT,1   ; Test for single strip
        JNE     MULTI_STRIP     ; Go if not a single strip
        JMP     ONE_STRIP
MULTI_STRIP:
; Multi-strip image processing is not implemented
        LEA     SI,STRIP_ERR
        CALL    BIO_SHOW_MESS
        CALL    KBR_WAIT
        JMP     DOS_EXIT
;**********************|
;  single strip image  |
;**********************|
ONE_STRIP:
; If the image is encoded in a single strip, the STRIP_OFFSET
; variable holds the address of the bit map. The number of bytes
; in the bit map can be calculated by multiplying the variables
; the variables BYTE_WIDTH and PIX_LENGTH
        MOV     AX,PIX_LENGTH   ; Word variable to AX
        MOV     BL,BYTE_WIDTH   ; Byte variable to BL
        MOV     BH,0            ; Clear high byte of multiplier
        MUL     BX
; Doubleword result in AX and DX
; Code assumes that the image byte length is < FFFFH
        MOV     IMAGE_SIZE,AX   ; Store product in variable
;**********************|
;     compression      |
;**********************|
; Determine compression code. Un-compressed and PackBits is
; supported
        MOV     SI,TAG1_ADDRESS ; Address of first tag in IFD
        MOV     AX,COMPRESSION  ; Get tag for strip offsets
        CALL    FIND_TAG
        JNC     OK_COMPRESS     ; Go if tag found
        JMP     BAD_TIFF_FILE   ; Compression tag is required
OK_COMPRESS:
        MOV     AX,WORD PTR [SI+8]         ; Get compression code
        CMP     AX,8005H        ; Code for PackBits compression
        JNE     NOT_PB          ; Go if not PackBits
; Store code for PackBits compression
        MOV     PACKBITS,1
        JMP     SHOW_PARAMS
NOT_PB:
        CMP     AX,1            ; Test for no compression code
        JE      SHOW_PARAMS     ; Go if not compressed
;**********************|
;  compression error   |
;**********************|
; Compression scheme not supported by this program
        LEA     SI,COMP_ERR
        CALL    BIO_SHOW_MESS
        CALL    KBR_WAIT
        JMP     DOS_EXIT
;**********************|
;  display parameters  |
;**********************|
SHOW_PARAMS:
        LEA     SI,WAIT1_MS
        CALL    BIO_SHOW_MESS
        CALL    KBR_WAIT
; Convert data to ASCII and move into message area
        MOV     DL,BYTE_WIDTH   ; Byte parameter
        MOV     DH,0            ; Clear high order byte
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASC      ; Procedure in SOLUTION.LIB
; Set digits in display area
        LEA     BX,BYTE_WID_MS  ; Digit display area
        CALL    DEC_TO_BX       ; Local procedure
; Obtain and convert number of pixel rows
        MOV     DX,PIX_LENGTH   ; Word parameter
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASC      ; Procedure in SOLUTION.LIB
; Set digits in display area
        LEA     BX,PIX_ROWS_MS  ; Digit display area
        CALL    DEC_TO_BX       ; Local procedure
; Obtain and convert offset of bit map from start of TIFF file
        MOV     DX,STRIP_OFFSET ; Word parameter
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASC      ; Procedure in SOLUTION.LIB
; Set digits in display area
        LEA     BX,MAP_OFFSET_MS        ; Digit display area
        CALL    DEC_TO_BX       ; Local procedure
; Obtain and convert number of strips
        MOV     DX,STRIP_COUNT  ; Word parameter
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASC      ; Procedure in SOLUTION.LIB
; Set digits in display area
        LEA     BX,STRIPS_MS    ; Digit display area
        CALL    DEC_TO_BX       ; Local procedure
; Obtain and convert image size
        MOV     DX,IMAGE_SIZE   ; Word parameter
        LEA     DI,ASCII_BUF    ; Conversion buffer
        CALL    BIN_TO_ASC      ; Procedure in SOLUTION.LIB
; Set digits in display area
        LEA     BX,IMAGE_SIZE_MS; Digit display area
        CALL    DEC_TO_BX       ; Local procedure
; Set compression mode message according to variable
        CMP     PACKBITS,1      ; Is it PackBits compression ?
        JE      OK_PACK_MS      ; Message is OK
; Change compression message in display area
        LEA     SI,UNPACKED_MS  ; Pointer to source text
        LEA     DI,COMPRESS_MS  ; Pointer to display area
        MOV     CX,12           ; Characters in message
MOVE_12:
        MOV     AL,[SI]         ; Get source character
        MOV     [DI],AL         ; Place character in buffer
        INC     SI              ; Bump buffer pointers
        INC     DI
        LOOP    MOVE_12         ; Continue
OK_PACK_MS:
        LEA     SI,PARAMETER_MSG
        CALL    BIO_SHOW_MESS
; Set display variables in block
        MOV     AX,PIX_LENGTH   ; Number of pixel rows
        MOV     BLOCK_ROWS,AX   ; See note below
        MOV     AL,BYTE_WIDTH   ; Get image width
        MOV     BLOCK_COLS,AL   ; Place in display block
; Test for packed bit map, unpack if necessary
        CMP     PACKBITS,1      ; Code for packed
        JE      UNPACK_MAP      ; Go if packed
;**********************|
;  move unpacked map   |
;**********************|
        LEA     SI,TIFF_FILE    ; Pointer to start of file
        ADD     SI,STRIP_OFFSET ; Add offset to bit map
        LEA     DI,BIT_IMAGE    ; Location in display block
        MOV     CX,IMAGE_SIZE   ; Number of bytes in un-compressed
                                ; bit map
MOVE_IMAGE:
        MOV     AL,[SI]         ; Get bit map byte
        NOT     AL              ; Invert white and black bits
        MOV     [DI],AL         ; Place it in display block area
        INC     SI              ; Bump buffer pointers
        INC     DI
        LOOP    MOVE_IMAGE
        JMP     DISPLAY_IMAGE
;******************************************************************
;                        unpack bit map
;******************************************************************
; Unpacking logic for TIFF PackBits scheme
; PackBits packages consist of 2 bytes. The first byte (n) encodes
; the following options:
;       1. if n is in the range 0 to 127 then next n+1 bytes are
;          to be interpreted as literal values
;       2. if n is in the range -127 to -1 then the following byte
;          is to be repeated -n+1 times
;       3. if n = 128 then no operation is executed
UNPACK_MAP:
        int     03h
        LEA     DI,BIT_IMAGE    ; Destination in display block
        LEA     SI,TIFF_FILE    ; Pointer to start of file
        ADD     SI,STRIP_OFFSET ; Add offset to bit map
; SI ==> start of image if there is a single strip
TEST_N_BYTE:
        MOV     AL,[SI]         ; Get n byte
        CMP     AL,128          ; Code for NOP
        JB      LITERAL_CODE    ; Go if in the literal range
        JA      REPEAT_CODE     ; Go if in repeat range
; Code is 128 (NOP)
        INC     SI              ; Skip NOP code
        JMP     NEXT_PACK_CODE  ; Continue
;**********************|
;   literal expansion  |
;**********************|
LITERAL_CODE:
        MOV     CL,AL           ; Counter to CL
        MOV     CH,0            ; Clear high byte of counter
        INC     CX              ; Add 1
        INC     SI              ; Skip n byte
        ADD     EXP_COUNT,CX    ; Add bytes to counter
LIT_MOVE:
        MOV     AL,[SI]         ; Get literal byte
        NOT     AL              ; Invert white and black bits
        MOV     [DI],AL         ; Place in bit map
        INC     DI              ; Bump pointers
        INC     SI
        LOOP    LIT_MOVE
        JMP     NEXT_PACK_CODE
;**********************|
;  repeated expansion  |
;**********************|
REPEAT_CODE:
        NEG     AL              ; Negate to convert 2s complement
                                ; representation
        MOV     CL,AL           ; Counter to CL
        MOV     CH,0            ; Clear high byte of counter
        INC     CX              ; Add 1
        INC     SI              ; Skip n byte
        ADD     EXP_COUNT,CX    ; Add bytes to counter
        MOV     AL,[SI]         ; Get byte to repeat
        NOT     AL              ; Invert black and white bits
        INC     SI              ; Skip to next n byte
EXP_MOVE:
        MOV     [DI],AL         ; Place byte in buffer
        INC     DI              ; Bump bit map pointer
        LOOP    EXP_MOVE
;**********************|
;  get next pack code  |
;**********************|
; EXP_COUNT holds the total bytes in bit map at this point
; IMAGE_SIZE holds the total bytes in the expanded bit map
NEXT_PACK_CODE:
        MOV     AX,EXP_COUNT    ; Bytes now in bit map
        CMP     AX,IMAGE_SIZE   ; Compare with map size
        JAE     DISPLAY_IMAGE   ; Go if at end of image
        JMP     TEST_N_BYTE
;*****************************************************************
;                      display raster image
;*****************************************************************
DISPLAY_IMAGE:
        LEA     SI,WAIT2_MS
        CALL    BIO_SHOW_MESS
        CALL    KBR_WAIT
;**********************|
;   initialize video   |
;     buffer address   |
;**********************|
        MOV     AX,0A000H       ; AX to base address of video buffer
        MOV     ES,AX           ; To ES
;
        MOV     AH,0            ; BIOS service request code
        MOV     AL,18           ; VGA mode number 18. 640 by 480
                                ; pixels in 16 colors
        INT     10H             ; BIOS video interrupt
;**********************|
;   set write mode 2   |
;**********************|
; This program uses write mode 2 for graphics display. In this write
; mode the color code is contained in a CPU register
        MOV     AL,2            ; Mode number
        CALL    SET_WRITE_MODE
;**********************|
;     clear screen     |
;**********************|
        MOV     AL,00000111B    ; Color = gray
        CALL    CLS_18          ; Library procedure for VGA mode 18
;**********************|
;  display image block |
;**********************|
        LEA     SI,IMAGE_BLOCK  ; Pointer to formatted block
        LEA     BX,IMAGE_COLOR  ; Pointer to color variable\
        CALL    SHOW_TIFF_MAP
;*****************************************************************
;                         exit to DOS
;*****************************************************************
DOS_EXIT:
        CALL    KBR_WAIT
;
        MOV     AL,3            ; Set alphanumeric mode 3
        CALL    BIO_SET_MODE    ; Procedure in SOLUTION.LIB
;
        MOV     AH,4CH          ; DOS service request code
        MOV     AL,0            ; No error code returned
        INT     21H             ; TO DOS
;*****************************************************************
;                         local procedures
;*****************************************************************
HEX_TO_BX       PROC    NEAR
; Procedure to move and expand 4 hexadecimal digits
; On entry:
;         DI ==> buffer containing 4 hex digits
;         BX ==> destination for 2-pairs of hex digits, in the
;                form XX-YY
        MOV     AX,WORD PTR [DI]        ; Get first 2 digits
        MOV     WORD PTR [BX],AX        ; Place in buffer
        ADD     BX,3            ; Bump buffer pointers
        ADD     DI,2
        MOV     AX,WORD PTR [DI]        ; Get first 2 digits
        MOV     WORD PTR [BX],AX        ; Place in buffer
        RET
HEX_TO_BX       ENDP
;*****************************************************************
DEC_TO_BX       PROC    NEAR
; Procedure to move 5 decimal digits to another buffer
; On entry:
;         DI ==> buffer containing 5 decimal digits
;         BX ==> destination buffer for 5 digits
        PUSH    CX
        MOV     CX,5            ; Counter for 5 digits
FIVE_DIGITS:
        MOV     AL,[DI]         ; Get source digit
        MOV     [BX],AL         ; Place in buffer
        INC     DI              ; Bump pointers
        INC     BX
        LOOP    FIVE_DIGITS
        POP     CX              ; Restore register
        RET
DEC_TO_BX       ENDP
;*****************************************************************
;
FIND_TAG        PROC    NEAR
; Find a specific tag code in the Image File Directory
; On entry:
;          AX = desired tag code
;          SI ==> start of Image File Directory (IFD)
; On exit:
;          Carry clear if tag code found
;          SI ==> first tag field (code)
;
;          Carry set code not present in IFD
;
TEST_TAG_CODE:
        MOV     BX,WORD PTR [SI]        ; Get tag code
        CMP     BX,0            ; Test for last IFD
        JE      END_OF_IFD      ; Go if last
        CMP     AX,BX           ; Compare with one desired
        JNE     NEXT_TAG_CODE   ; Index if not
; At this point desired tag code has been found
        CLC                     ; Tag found return code
        RET
NEXT_TAG_CODE:
        ADD     SI,12           ; Index to next tag
; Test for last tag
        JMP     TEST_TAG_CODE   ; Continue
END_OF_IFD:
        STC                     ; Tag not found
        RET
FIND_TAG        ENDP
;*****************************************************************
;
; Variables used by SHOW_TIFF_MAP procedure
X_COORD         DW      0000H           ; Storage for x coordinate
BYTES           DB      0H              ; Number of bytes per row
COUNT_8         DB      8               ; Bit counter
PIX_ROWS        DW      0               ; Number of pixel rows
;
;
SHOW_TIFF_MAP   PROC    NEAR
; Display a graphics block in the data segment
; On entry:
;      SI --> Start of control area of graphic block to be
;             displayed
;      BX --> color code for bit map
;
; Register set-up after routine initialization:
;               CX = x coordinate of block start
;               DX = y coordinate of block start
;               CS:PIX_COUNT = number of rows in block
;               BH = number of bytes per block row
;               AL = color code
;               DI --> start of encoded graphic block
;
; Initialize registers
        PUSH    BX              ; Save pointer in stack
        MOV     CX,WORD PTR [SI]        ; x coordinate
        MOV     CS:X_COORD,CX              ; Store in variable
        ADD     SI,2                    ; Bump pointer
        MOV     DX,WORD PTR [SI]        ; y coordinate
        ADD     SI,2                    ; Bump pointer
        MOV     AX,WORD PTR [SI]        ; Get pixel length
        MOV     CS:PIX_ROWS,AX          ; Store in variable
        ADD     SI,2                    ; Bump to next item
        MOV     BH,BYTE PTR [SI]        ; Bytes per block
        MOV     CS:BYTES,BH                ; Store in variable
        INC     SI                      ; Bump pointer
        XCHG    SI,DI           ; Buffer start to DI
        POP     SI              ; Color code block pointer
        MOV     AL,[SI]         ; Color code to AL
        MOV     CS:COUNT_8,8       ; Prime bit counter
; Color code in the AL register
DISPLAY_BYTE:
        MOV     AH,[DI]         ; High nibble to AH
; Test for al zero display pattern
        CMP     AH,0            ; Nothing to display?
        JNE     TEST_BIT        ; Continue if not zero
        ADD     CX,8            ; Skip this byte
        JMP     NEXT_BYTE       ; Skip byte
TEST_BIT:
        TEST    AH,10000000B    ; Is high bit set?
        JZ      NEXT_BIT        ; Bit not set
; Set the pixel
        PUSH    AX              ; Save entry registers
        PUSH    BX
        CALL    PIXEL_ADD_18
        CALL    WRITE_PIX_18
        POP     BX              ; Restore registers
        POP     AX
NEXT_BIT:
        SAL     AH,1            ; Shift AH to test next bit
        INC     CX              ; Bump x coordinate counter
        DEC     CS:COUNT_8         ; Bit counter
        JZ      NEXT_BYTE       ; Exit if counter rewound
        JMP     TEST_BIT        ; Continue
; Index to next byte in row, if not at end of row
NEXT_BYTE:
        DEC     BH              ; Bytes per row counter
        JZ      NEXT_ROW        ; End of graphic row
BYTE_ENTRY:
        INC     DI              ; Bump graphic code pointer
        MOV     CS:COUNT_8,8       ; Reset bits counter
        JMP     DISPLAY_BYTE
; Index to next row
NEXT_ROW:
; Test for last graphic row
        DEC     CS:PIX_ROWS     ; Pixel row counter
        JZ      GRAPH_END       ; Done, exit
        MOV     BH,CS:BYTES        ; Reset bytes counter
        INC     DX              ; Bump y coordinate control
        MOV     CX,CS:X_COORD      ; Reset x coordinate control
        JMP     BYTE_ENTRY
GRAPH_END:
        RET
SHOW_TIFF_MAP   ENDP
;
CODE    ENDS
        END     START           ; Reference to label at which
                                ; execution begins
