;*****************************************************************
;*****************************************************************
;                         XGALUT.ASM
;*****************************************************************
;*****************************************************************
; Description:
; Demo program for experimentation with XGA LUT registers in the
; IIRRGGBB format
;
; Instructions:
; After executing the program you may input the digits 0 or 1 in
; the II, RR, GG, and BB positions. Program will display a
; rectangle in the selected palette color.
; A low pitch beep indicates an invalid input digit (not 0 or 1)
; A high pitch beep indicates the end of the 8-digit input value
;
;*****************************************************************
;                            EXTERNALS
;*****************************************************************
EXTRN   KBR_WAIT:       FAR
EXTRN   KBR_FLUSH:      FAR
EXTRN   OPEN_FILE:      FAR
EXTRN   READ_128:       FAR
EXTRN   CLOSE_FILE:     FAR
EXTRN   SET_DTA:        FAR
EXTRN   DOS_STRING_OUT: FAR
EXTRN   SOUND:          FAR
;
INCLUDELIB      SOLUTION
;****************************************************************
;                          stack segment
;****************************************************************
STACK   SEGMENT stack

                DB      0400H DUP ('?'); Default stack is 1K
STACK_TOP       EQU     THIS BYTE
;
STACK   ENDS
;
;****************************************************************
;                segment for task state data
;****************************************************************
TASK    SEGMENT
;**********************|
;  AI state buffer     |
;**********************|
STATE_BUF       DB      360 DUP (00H)
;
TASK    ENDS
;
;****************************************************************
;                          data segment
;****************************************************************
DATA    SEGMENT
;**********************|
;     ERROR message    |
;**********************|
BADXGA_MSG      DB      0AH,0DH
        DB      '** ERROR ** No XGA Graphics hardware in system'
        DB      0AH,0DH
        DB      'Press any key to continue'
        DB      0AH,0DH,'$'
;**********************|
;   AI list address    |
;**********************|
AI_ADD          DD      0       ; Doubleword storage for address
                                ; of Adapter Interface services
;
;
HOPEN_DATA      DW      3       ; Length of data block
INIT_FLAGS      DB      01000000B
                                ; 7 6 5 4 3 2 1 0 <= flags
                                ; | | ___________
                                ; | |      |_______ Reserved
                                ; | |_ Dont't load defualt palette
                                ; |___ Don't clear bit planes
AF_MODE         DB      0       ; Advanced funtion mode
                                ; No.   Pixels     Text
                                ; 00    1024x768   85x38
                                ; 01    640x480    80x34
                                ; 02    1024x768   128x54
                                ; 03    1024x768   146x51
RET_FLAGS       DB      0       ; Status
                                ; 0 if initialization successful
                                ; Not 0 if initialization failed
;
HINIT_DATA      DW      2       ; Length of data block
BUF_SEG         DW      0       ; Segment of AI buffer
;
HCLOSE_DATA     DW      0       ; Length field is zero for HCLOSE
HEGS_DATA       DW      0       ; Length fiels is zero for HEGS
DUMMY           DW      0       ; Dummy data area
;
;**********************|
;       colors         |
;**********************|
HSCOL_DATA      DW      4       ; Length of data block
FORE_COL        DB      0F0H    ; 8-bit color code
                DB      0       ; Padding for double word
                DW      0
;
HSBCOL_DATA     DW      4       ; Length of data block
BACK_COL        DB      11110000B       ; Bright red in 2-bit
                                        ; IRGB format
                DB      0       ; Padding for double word
                DW      0
;
THIS_COLOR      DB      0       ; Storage for present color
;**********************|
;    drawing data      |
;**********************|
; Data block for rectangle draw
HRECT_DATA      DW      8       ; Length of data block
RECT_X          DW      0       ; x coordinate of top-left corner
RECT_Y          DW      0       ; y coordinate of top-left corner
RECT_WIDTH      DW      0       ; Width (1 to 1024)
RECT_HIGH       DW      0       ; Height (1 to 768)
;
; Data block to set coordinates of current position
HSCP_DATA       DW      4       ; Length of data block
NEW_X           DW      0       ; New x for currend point
NEW_Y           DW      0       ; New y for current point
;
;**********************|
; text operations data |
;**********************|
HSCS_DATA       DW      4       ; Length of data block
FONT_OFF        DW      0       ; Offset of loaded font
FONT_SEG        DW      0       ; Segment of loaded font
;
;**********************|
;     text messages    |
;**********************|
HCCHST_DATA_2   DW      51      ; Length of data block
                DB      'XGA/8514/A Adapter Interface'
                DB      ' Double-bit IRGB Pallete'
                DW      0
;
HCCHST_DATA_3   DW      38      ; Length of data block
                DB      'Enter IIRRGGBB value (<Esc> to end):   '
                DW      0
;
HCCHST_DATA_4   DW      27      ; Length of data block
                DB      'IIRRGGBB value => '
BIN_DIGITS      DB      '00000000'
                DW      0
;
HCCHST_DATA_5   DW      27      ; Length of data block
                DB      'IIRRGGBB value => '
                DB      '        '
                DW      0
; ASFONT (select chracter set)
ASFONT_DATA     DW      6       ; Length of data block
                DB      0       ; Font number
                DB      0       ; Reserved
AFONT_OFF       DW      0       ; Offset of loaded font
AFONT_SEG       DW      0       ; Segment of loaded font
;**********************|
;      fonts           |
;**********************|
; ASCIIZ filename for XGA 85-by-38 font
F1220_NAME      DB      'STAN1220.FNT',00H
FONT_HANDLE     DW      0       ; Handle for font file
;
;**********************|
;   storage for font   |
;**********************|
; Font header area
FONT_BUF        DW      0       ; Number of code pages
                DW      0       ; Default code page (0 to 4)
                DW      0       ; Alternate default (0 to 4)
                DD      0       ; 4-byte ID string ('437',0)
PAGE_1_OFFSET   DW      0       ; Offset of CSD within file
                DD      0       ; 4-byte ID string ('850',0)
                DW      0       ; Offset of CSD within file
                DD      0       ; 4-byte ID string ('860',0)
                DW      0       ; Offset of CSD within file
                DD      0       ; 4-byte ID string ('863',0)
                DW      0       ; Offset of CSD within file
                DD      0       ; 4-byte ID string ('865',0)
                DW      0       ; Offset of CSD within file
;
; Character set definition block for for first code page
                DB      0       ; Reserved
                DB      0       ; Font type:
                                ; 0 = multiplane image
                                ; 3 = short vector font
                DB      0       ; Reserved
                DD      0       ; Reserved
CELL_WIDTH      DB      0       ; Pixel width of character cell
CELL_HEIGHT     DB      0       ; Pixel height of cell
                DB      0       ; Reserved
                DW      0       ; Cell size
CSD_FLAGS       DW      0       ; Flag bits:
                                ; Bit 14 ... 0 = single plane
                                ;            1 = multiplane
                                ;     13 ... 0 = not prop. space
                                ;            1 = prop. space
IDX_TABLE_O     DW      0       ; Offset of index table
IDX_TABLE_S     DW      0       ; Segment of index table
                DW      0       ; Offset of envelope table
                DW      0       ; Segment of envelope table
                DB      0       ; Initial code point
                DB      0       ; Final code point
CSD_TABLE_O     DW      0       ; Offset of character definition
CSD_TABLE_S     DW      0       ; Segment of character definition
                DB      14250 DUP (00H)
;
DATA_BUF        DB      129 DUP (00H)   ; Storage for disk data
;
;**********************|
;    palette data      |
;**********************|
; Function to load pallete
HLDPAL_DATA     DW      10      ; Length of data block
LOAD_CODE       DB      0       ; Code to load user pallete
                                ; 1 = load default pallete
                DB      0       ; Reserved
                DW      0       ; Number of first entry
                DW      256     ; Number of entries to load
PAL_OFF         DW      0       ; Offset of user palette
PAL_SEG         DW      0       ; Segment of user pallete
;
; Functions HSPAL (save palette data) and HRPAL (restore palette)
HSPAL_DATA      DW      769     ; Length of palette
                DW      769 DUP (00H)   ; Storage for pallete
;
;
; Double-bit IRGB pallete in the following format
;            7 6 5 4 3 2 1 0  <= Bits
;            I I R R G G B B  <= Color codes
;
;                      | R   B   G       R   B   G     |
IRGB_SHADES     DB      000,000,000,000,036,072,036,000 ; 1
                DB      036,108,036,000,036,144,036,000 ; 3
                DB      036,036,072,000,036,072,072,000 ; 5
                DB      036,108,072,000,036,144,072,000 ; 7
                DB      036,036,108,000,036,072,108,000 ; 9
                DB      036,108,108,000,036,144,108,000 ; 11
                DB      036,036,144,000,036,072,144,000 ; 13
                DB      036,108,144,000,036,144,144,000 ; 15
                DB      072,036,036,000,072,072,036,000 ; 17
                DB      072,108,036,000,072,144,036,000 ; 19
                DB      072,036,072,000,072,072,072,000 ; 21
                DB      072,108,072,000,072,144,072,000 ; 23
                DB      072,036,108,000,072,072,108,000 ; 25
                DB      072,108,108,000,072,144,108,000 ; 27
                DB      072,036,144,000,072,072,144,000 ; 29
                DB      072,108,144,000,072,144,144,000 ; 31
                DB      108,036,036,000,108,071,036,000 ; 33
                DB      108,108,036,000,108,144,036,000 ; 35
                DB      108,036,072,000,108,072,072,000 ; 37
                DB      108,108,072,000,108,144,072,000 ; 39
                DB      108,036,108,000,108,072,108,000 ; 41
                DB      108,108,108,000,108,144,108,000 ; 43
                DB      036,036,144,000,108,072,144,000 ; 45
                DB      108,108,144,000,108,144,144,000 ; 47
                DB      144,036,036,000,144,072,036,000 ; 49
                DB      144,108,036,000,144,144,036,000 ; 51
                DB      144,036,072,000,144,072,072,000 ; 53
                DB      144,108,072,000,144,144,072,000 ; 55
                DB      144,036,108,000,144,072,108,000 ; 57
                DB      144,108,108,000,144,144,108,000 ; 59
                DB      144,036,144,000,144,072,144,000 ; 61
                DB      144,108,144,000,144,144,144,000 ; 63
;
                DB      072,072,072,000,072,108,072,000 ; 1
                DB      072,144,072,000,072,180,072,000 ; 3
                DB      072,072,108,000,072,108,108,000 ; 5
                DB      072,144,108,000,072,180,108,000 ; 7
                DB      072,072,144,000,072,108,144,000 ; 9
                DB      072,144,144,000,072,180,144,000 ; 11
                DB      072,072,180,000,072,108,180,000 ; 13
                DB      072,144,180,000,072,180,180,000 ; 15
                DB      108,072,072,000,108,108,072,000 ; 17
                DB      108,144,072,000,108,180,072,000 ; 19
                DB      108,072,108,000,108,108,108,000 ; 21
                DB      108,144,108,000,108,180,108,000 ; 23
                DB      108,072,144,000,108,108,144,000 ; 25
                DB      108,144,144,000,108,180,144,000 ; 27
                DB      108,072,180,000,108,108,180,000 ; 29
                DB      108,144,180,000,108,180,180,000 ; 31
                DB      144,072,072,000,144,108,072,000 ; 33
                DB      144,144,072,000,144,180,072,000 ; 35
                DB      144,072,108,000,144,108,108,000 ; 37
                DB      144,144,108,000,144,180,108,000 ; 39
                DB      144,072,144,000,144,108,144,000 ; 41
                DB      144,144,144,000,144,180,144,000 ; 43
                DB      072,072,180,000,144,108,180,000 ; 45
                DB      144,144,180,000,144,180,180,000 ; 47
                DB      180,072,072,000,180,108,072,000 ; 49
                DB      180,144,072,000,180,180,072,000 ; 51
                DB      180,072,108,000,180,108,108,000 ; 53
                DB      180,144,108,000,180,180,108,000 ; 55
                DB      180,072,144,000,180,108,144,000 ; 57
                DB      180,144,144,000,180,180,144,000 ; 59
                DB      180,072,180,000,180,108,180,000 ; 61
                DB      180,144,180,000,180,180,180,000 ; 63
;
                DB      108,108,108,000,108,144,108,000 ; 1
                DB      108,180,108,000,108,216,108,000 ; 3
                DB      108,108,144,000,108,144,144,000 ; 5
                DB      108,180,144,000,108,216,144,000 ; 7
                DB      108,108,180,000,108,144,180,000 ; 9
                DB      108,180,180,000,108,216,180,000 ; 11
                DB      108,108,216,000,108,144,216,000 ; 13
                DB      108,180,216,000,108,216,216,000 ; 15
                DB      144,108,108,000,144,144,108,000 ; 17
                DB      144,180,108,000,144,216,108,000 ; 19
                DB      144,108,144,000,144,144,144,000 ; 21
                DB      144,180,144,000,144,216,144,000 ; 23
                DB      144,108,180,000,144,144,180,000 ; 25
                DB      144,180,180,000,144,216,180,000 ; 27
                DB      144,108,216,000,144,144,216,000 ; 29
                DB      144,180,216,000,144,216,216,000 ; 31
                DB      180,108,108,000,180,144,108,000 ; 33
                DB      180,180,108,000,180,216,108,000 ; 35
                DB      180,108,144,000,180,144,144,000 ; 37
                DB      180,180,144,000,180,216,144,000 ; 39
                DB      180,108,180,000,180,144,180,000 ; 41
                DB      180,180,180,000,180,216,180,000 ; 43
                DB      108,108,216,000,180,144,216,000 ; 45
                DB      180,180,216,000,180,216,216,000 ; 47
                DB      216,108,108,000,216,144,108,000 ; 49
                DB      216,180,108,000,216,216,108,000 ; 51
                DB      216,108,144,000,216,144,144,000 ; 53
                DB      216,180,144,000,216,216,144,000 ; 55
                DB      216,108,180,000,216,144,180,000 ; 57
                DB      216,180,180,000,216,216,180,000 ; 59
                DB      216,108,216,000,216,144,216,000 ; 61
                DB      216,180,216,000,216,216,216,000 ; 63
;
                DB      144,144,144,000,144,180,144,000 ; 1
                DB      144,216,144,000,144,252,144,000 ; 3
                DB      144,144,180,000,144,180,180,000 ; 5
                DB      144,216,180,000,144,252,180,000 ; 7
                DB      144,144,216,000,144,180,216,000 ; 9
                DB      144,216,216,000,144,252,216,000 ; 11
                DB      144,144,252,000,144,180,252,000 ; 13
                DB      144,216,252,000,144,252,252,000 ; 15
                DB      180,144,144,000,180,180,144,000 ; 17
                DB      180,216,144,000,180,252,144,000 ; 19
                DB      180,144,180,000,180,180,180,000 ; 21
                DB      180,216,180,000,180,252,180,000 ; 23
                DB      180,144,216,000,180,180,216,000 ; 25
                DB      180,216,216,000,180,252,216,000 ; 27
                DB      180,144,252,000,180,180,252,000 ; 29
                DB      180,216,252,000,180,252,252,000 ; 31
                DB      216,144,144,000,216,180,144,000 ; 33
                DB      216,215,144,000,216,252,144,000 ; 35
                DB      216,144,180,000,216,180,180,000 ; 37
                DB      216,216,180,000,216,252,180,000 ; 39
                DB      216,144,216,000,216,180,216,000 ; 41
                DB      216,216,216,000,216,252,216,000 ; 43
                DB      144,144,252,000,216,180,252,000 ; 45
                DB      216,216,252,000,216,252,252,000 ; 47
                DB      252,144,144,000,252,180,144,000 ; 49
                DB      252,216,144,000,252,252,144,000 ; 51
                DB      252,144,180,000,252,180,180,000 ; 53
                DB      252,216,180,000,252,252,180,000 ; 55
                DB      252,144,216,000,252,180,216,000 ; 57
                DB      252,216,216,000,252,252,216,000 ; 59
                DB      252,144,252,000,252,180,252,000 ; 61
                DB      252,216,252,000,252,252,252,000 ; 63
;
DATA    ENDS
;
;****************************************************************
;                          code segment
;****************************************************************
;
CODE    SEGMENT
        ASSUME  CS:CODE
;
START:
;*******************|
;  initialization   |
;*******************|
; Establish data and extra segment addressability
        MOV     AX,DATA         ; Address of DATA to AX
        MOV     DS,AX           ; and to DS
        ASSUME  DS:DATA         ; Assume from here on
;**********************|
;  get adapter address |
;**********************|
; The local procedure AI_VECTOR obtains the segment:offset
; address of the adapter handler
        CALL    AI_VECTOR       ; Local procedure
        JNC     OK_VECTOR       ; Go if no carry
; Error exit
AI_ERROR:
        LEA     DX,BADXGA_MSG   ; Pointer to error message
        CALL    DOS_STRING_OUT  ; Library procedure
        CALL    KBR_WAIT
        JMP     DOS_EXIT
OK_VECTOR:
; Store segment and offset of AI handler
        MOV     WORD PTR AI_ADD,DX      ; Store offset of address
        MOV     WORD PTR AI_ADD+2,CX    ; and segment
; Entry point is stored in a DS variable
;**********************|
;    initialize AI     |
;**********************|
; Call HOPEN sevice (enable adapter)
        MOV     INIT_FLAGS,0    ; Set initialization flags
                                ; to clear memory and load default
                                ; palette
        MOV     AF_MODE,0       ; Set 1024x768 mode number 0
        MOV     AX,8            ; Code number for this service
        LEA     BX,HOPEN_DATA   ; Pointer to data block
        CALL    AI_SERVICE
;
        MOV     AL,RET_FLAGS    ; Not zero if open error
        CMP     AL,0            ; Test for no error
        JZ      OK_OPEN         ; Go if no error
        JMP     AI_ERROR        ; Error exit
;
; Call HINIT (Initialize adapter state)
OK_OPEN:
        MOV     AX,TASK         ; Segment for task state
        MOV     BUF_SEG,AX      ; Store segment of adapter state
                                ; buffer
        MOV     AX,48           ; Code number for this service
        LEA     BX,HINIT_DATA   ; Pointer to data block
        CALL    AI_SERVICE
;
; Call HSCOL (set color)
; HSCOL
        MOV     FORE_COL,0      ; Black
        MOV     AX,7            ; Code number for this service
        LEA     BX,HSCOL_DATA   ; Pointer to data block
        CALL    AI_SERVICE
;
; Call HSPAL to save current Pallete
        MOV     AX,57           ; Code number for this service
        LEA     BX,HSPAL_DATA   ; Pointer to data block
        CALL    AI_SERVICE
;
        LEA     BX,HSPAL_DATA
;
; Fill screen using rectangle draw function HRECT
        MOV     RECT_X,0        ; x origin
        MOV     RECT_Y,0        ; y origin
        MOV     RECT_WIDTH,1024 ; width
        MOV     RECT_HIGH,768   ; high
        MOV     AX,32           ; Code number for this service
        LEA     BX,HRECT_DATA   ; Pointer to data block
        CALL    AI_SERVICE
;**********************|
;    load font file    |
;**********************|
; Before using text display operations one of the four font files
; provided with the adapter must be loaded into RAM
        LEA     DX,F1220_NAME   ; Filename for XGA 12x20 font
        LEA     DI,FONT_BUF     ; Buffer for storing font
        CALL    XGA_FONT        ; Local procedure to load font
; Carry set if error during font load
        JNC     OK_XGA_FONT     ; Go if no error
        JMP     AI_ERROR        ; Error exit
;**********************|
; select character set |
;**********************|
; The AI is informed of the address of the loaded font by means
; of the HSCS (set character set) function
OK_XGA_FONT:
        PUSH    DS              ; DS to stack
        POP     AX              ; and to AX
        MOV     FONT_SEG,AX     ; Store segment address
        MOV     AFONT_SEG,AX    ; For alphanumeric operations
; The offset of the font's character set definition block is
; located at byte 10 of the font header
        LEA     SI,FONT_BUF     ; Offset of font buffer
        MOV     BX,[SI+10]      ; Get offset of first code page
        ADD     BX,SI           ; Add offset to pointer
        MOV     FONT_OFF,BX     ; Store pointer in block
        MOV     AFONT_OFF,BX    ; For alphanumeric operations
;**********************|
; update font pointers |
;**********************|
; Update pointers in character set defintion area by adding
; the load address of the font (in SI)
        ADD     IDX_TABLE_O,SI  ; Add to index table offset
        ADD     CSD_TABLE_O,SI  ; and to CSD table offset
; AX still holds the segment address. Store segment portion of
; address
        MOV     IDX_TABLE_S,AX  ; In index table
        MOV     CSD_TABLE_S,AX  ; In character set table
;
; Call HSCS (set character set) function
        MOV     AX,37           ; Code number for this service
        LEA     BX,HSCS_DATA    ; Pointer to data block
        CALL    AI_SERVICE
;**********************|
;    text commands     |
;**********************|
; AI text command that start with the prefix letter H are used to
; perform text display operations at the pixel level
; First set current color to bright red
        MOV     FORE_COL,00001100B      ; Bright red
        MOV     AX,7            ; Code number for this service
        LEA     BX,HSCOL_DATA   ; Pointer to data block
        CALL    AI_SERVICE
;
; Change to double-bit IRGB pallete
        CALL    COLOR_LUT       ; Local procedure
;**********************|
;   display pallete    |
;**********************|
; Draw 256 colored rectangles on the 100th. screen line to show
; the default palette settings
        MOV     DX,100          ; Start at pixel row 100
        CALL    DO_256          ; Local procedure
;
;**********************|
;    display message   |
;    for 2-bit IRGB    |
;**********************|
; Display a text message using the AI text commands
; First set current position to top-center of the screen using
; the HSCP (set current position) function
        MOV     NEW_X,220       ; Screen column
; y coordinate already set during previous message
; Call HSCP
        MOV     AX,4            ; Code number for this service
        LEA     BX,HSCP_DATA    ; Pointer to data block
        CALL    AI_SERVICE
; Call HCCHST (display character string at current position)
        MOV     AX,39           ; Code number for this service
        LEA     BX,HCCHST_DATA_2        ; Pointer to data block
        CALL    AI_SERVICE
;
;**********************|
;  prompt for input    |
;**********************|
; Prompt the user for binary input in IIRRGGBB format
; Display a text message using the AI text commands
; First set current position to top-center of the screen using
; the HSCP (set current position) function
        MOV     NEW_X,0         ; Screen column
        MOV     NEW_Y,300       ; Screen row
; Call HSCP
        MOV     AX,4            ; Code number for this service
        LEA     BX,HSCP_DATA    ; Pointer to data block
        CALL    AI_SERVICE
; Call HCCHST (display character string at current position)
        MOV     AX,39           ; Code number for this service
        LEA     BX,HCCHST_DATA_3        ; Pointer to data block
        CALL    AI_SERVICE
;
NEW_COLOR:
        CALL    KBR_FLUSH
;
; Loop for 8 input characters
        MOV     CX,8            ; Counter for 8 binary digits
        MOV     DL,0            ; DL hold binary
        LEA     SI,BIN_DIGITS
GET_8:
        SHL     DL,1            ; Shift counter
SKIP_DIGIT:
        PUSH    CX
        PUSH    DX
        PUSH    SI
        CALL    KBR_WAIT        ; Procedure in SOLUTION.LIB
        POP     SI
        POP     DX
        POP     CX
        CMP     AX,011BH        ; Test for <Esc>
        JNE     NOT_ESC         ; Go if not <Esc> key
        JMP     PROG_END
NOT_ESC:
; Test for valid digit (0 or 1)
        CMP     AL,'0'          ; '0' is valid
        JE      DIGIT_OK        ; Go if '0'
        CMP     AL,'1'          ; '1' is also valid
        JE      DIGIT_OK        ; Go if '1'
; At this point keyboard input is invalid
        CALL    BEEP_LOW        ; Signal error in input
        JMP     SKIP_DIGIT
DIGIT_OK:
        MOV     [SI],AL         ; Store digit
        INC     SI
; Update binary counter
        CMP     AL,'0'          ; Test for a 1 digit
        JE      DIGIT_0         ; Digit is a 0
        OR      DL,00000001B    ; Enter a 1 digit
DIGIT_0:
        LOOP    GET_8
;
; Produce high pitch beep to indicate end of input
        CALL    BEEP_HIGH       ; Local procedure
;
; Call HSCOL (set color)
; At this point DL has binary color code
        MOV     THIS_COLOR,DL
;
; Clear old binary text message
        MOV     FORE_COL,0      ; Current color
        MOV     AX,7            ; Code number for this service
        LEA     BX,HSCOL_DATA   ; Pointer to data block
        CALL    AI_SERVICE
;
        MOV     RECT_X,0        ; x origin
        MOV     RECT_Y,325      ; y origin
        MOV     RECT_WIDTH,600  ; width
        MOV     RECT_HIGH,100   ; high
        MOV     AX,32           ; Code number for this service
        LEA     BX,HRECT_DATA   ; Pointer to data block
        CALL    AI_SERVICE
;
; Set current color
        MOV     AL,THIS_COLOR
        MOV     FORE_COL,AL     ; Current color
        MOV     AX,7            ; Code number for this service
        LEA     BX,HSCOL_DATA   ; Pointer to data block
        CALL    AI_SERVICE
;
        MOV     NEW_X,0         ; Screen column
        MOV     NEW_Y,350       ; Screen row
; Call HSCP
        MOV     AX,4            ; Code number for this service
        LEA     BX,HSCP_DATA    ; Pointer to data block
        CALL    AI_SERVICE
; Call HCCHST (display character string at current position)
        MOV     AX,39           ; Code number for this service
        LEA     BX,HCCHST_DATA_4        ; Pointer to data block
        CALL    AI_SERVICE
;
; Fill screen area using rectangle draw function HRECT
        MOV     RECT_X,300      ; x origin
        MOV     RECT_Y,400      ; y origin
        MOV     RECT_WIDTH,100  ; width
        MOV     RECT_HIGH,100   ; high
        MOV     AX,32           ; Code number for this service
        LEA     BX,HRECT_DATA   ; Pointer to data block
        CALL    AI_SERVICE
;**********************|
;   repeat input       |
;**********************|
        JMP     NEW_COLOR
;
;**********************|
;  erase XGA/8514-a    |
;**********************|
PROG_END:
; HEGS (erase graphics screen)
        MOV     AX,13           ; Code number for this service
        LEA     BX,HEGS_DATA    ; Pointer to dummy data block
        CALL    AI_SERVICE
;*******************|
;   exit to DOS     |
;*******************|
DOS_EXIT:
        MOV     AH,4CH          ; DOS service request code
        MOV     AL,0            ; No error code returned
        INT     21H             ; TO DOS
;*****************************************************************
;                           procedures
;*****************************************************************
BEEP_LOW        PROC    NEAR
; Produce low pitch beep to indicated invalid input
; First save context
        PUSH    BX
        PUSH    CX
        PUSH    DX
        PUSH    SI
        MOV     BX,400          ; Frequency for low beep
        MOV     CX,500          ; Timer for 1/2 second
        CALL    SOUND           ; In SOLUTION.LIB
        POP     SI
        POP     DX
        POP     CX
        POP     BX
        RET
BEEP_LOW        ENDP
;
BEEP_HIGH       PROC    NEAR
; Produce high pitched beep to indicated end of input
; First save context
        PUSH    BX
        PUSH    CX
        PUSH    DX
        PUSH    SI
        MOV     BX,800          ; Frequency for low beep
        MOV     CX,1000         ; Timer for 1 second
        CALL    SOUND           ; In SOLUTION.LIB
        POP     SI
        POP     DX
        POP     CX
        POP     BX
        RET
BEEP_HIGH       ENDP
;
AI_VECTOR       PROC    NEAR
; Procedure to obtain the address vector to the XGA/8514-A Adapter
; Interface. This procedure must be called before calls are made
; to the Adapter Interface services (by means of the AI_SERVICE
; procedure)
;
; On entry:
;          nothing
; On exit:
;          carry set if no AI installed
;          carry clear if AI present
;             CX => segment of AI link table
;             DX => offset of AI link table
;
;**********************|
;    get vector 7FH    |
;**********************|
; Use MS DOS service number 53, interrupt 21H, to obtain the
; vector for the XGA/8514-A AI interrupt (7FH)
        MOV     AH,53           ; MS DOS service number
        MOV     AL,7FH          ; AI interrupt
        INT     21H             ; MS DOS interrupt
; ES => segment of interrupt handler
; BX => offset of handler
        MOV     AX,ES           ; Segment to AX
        OR      AX,BX           ; OR segment and offset
        JNZ     OK_AI           ; Go if address not 0000:0000
;**********************|
;  ERROR - no AI       |
;**********************|
NO_AI:
        STC                     ; Error return
        RET
;**********************|
;  get AI address      |
;**********************|
; Service number 0105H, interrupt 7FH, returns the address of the
; XGA/8514-A entry point
OK_AI:
        MOV     AX,0105H        ; Service request number
        INT     7FH             ; in XGA AI interrupt
        JC      NO_AI           ; Go if error code returned
; At this point CX:DX holds the address of the XGA/8514-A entry
; point (in segment:offset form)
        CLC                     ; No error code
        RET
AI_VECTOR       ENDP
;*****************************************************************
;
AI_SERVICE      PROC    NEAR
; Procedure to access the services in the XGA/8514-A Adapter
; Interface
;
; On entry:
;         AX = service number
;         DS:BX = address of parameter block
;
        PUSH    BP              ; Save base pointer
        MOV     BP,SP           ; Set BP to stack
; Push address of caller's parameter block
        PUSH    DS
        PUSH    BX              ; the offset
; Multiply by 4 to form offset as required by AI
        SHL     AX,1            ; AX time 2
        SHL     AX,1            ; again
        MOV     SI,AX           ; Offset to SI
        LES     BX,AI_ADD       ; Entry block address (ES:BX)
        CALL    DWORD PTR ES:[BX][SI]    ; Call AI service
        POP     BP              ; Restore caller's BP
        RET
AI_SERVICE      ENDP
;*****************************************************************
;
DO_256          PROC    NEAR
; Display 256 rectangles of 4 by 80 pixels each
; On entry:
;         DX = y coordinate of start point
;
;
; Store entry parameter
        MOV     RECT_Y,DX       ; Save as HRECT function data
        MOV     RECT_X,0        ; Initalize to first column
        MOV     AH,0            ; AH will holds the color value
        MOV     CX,256          ; Counter for 256 blocks
DO_ONE_RECT:
        PUSH    AX              ; Save color
        PUSH    CX              ; and counter
; Call HSCOL (set color) and HSBCOL (set background color)
        MOV     FORE_COL,AH             ; Color
        MOV     AX,7            ; Code number for this service
        LEA     BX,HSCOL_DATA   ; Pointer to data block
        CALL    AI_SERVICE
; Display a filled rectangle using HRECT function
        MOV     RECT_WIDTH,4    ; width
        MOV     RECT_HIGH,80    ; high
        MOV     AX,32           ; Code number for this service
        LEA     BX,HRECT_DATA   ; Pointer to data block
        CALL    AI_SERVICE
        POP     CX              ; Restore counter
        POP     AX              ; and color
        INC     AH              ; Bump to next color value
        ADD     RECT_X,4        ; move rectangle right
        LOOP    DO_ONE_RECT
 ; All 256 rectangles have been drawn
        RET
DO_256          ENDP
;
;*****************************************************************
;
COLOR_LUT       PROC    NEAR
; Initialize DAC registers for 256-color mode in the following
; format:
;               7 6 5 4 3 2 1 0  <= bits
;               |_| |_| |_| |_|
;                I   R   G   B
;
;**********************|
;  set LUT registers   |
;**********************|
; Set address of color table in HLDPAL data area
        PUSH    DS              ; DS to stack
        POP     AX              ; and to AX
        MOV     PAL_SEG,AX      ; Store segment address
        LEA     SI,IRGB_SHADES  ; Pointer to offset of address
        MOV     PAL_OFF,SI      ; Store offset
; Call HLDPAL to set palette registers
        MOV     AX,19           ; Code number for this service
        LEA     BX,HLDPAL_DATA  ; Pointer to data block
        CALL    AI_SERVICE
        RET
COLOR_LUT       ENDP
;*****************************************************************
;
XGA_FONT        PROC    NEAR
; Read an XGA or 8514-a font file into RAM
; On entry:
;         DS:DX --> ASCIIZ filename for font file
;                   (must be in the current path)
;         DS:DI --> RAM buffer for font storage
;
; On exit:
;       Carry clear if font read and stored in buffer
;       Carry set if file not found or disk error
;
; Open font file
        PUSH    DI              ; Save entry pointer
        CALL    OPEN_FILE
        POP     DI
        JNC     OK_XOPEN        ; Go if no error code
;**********************|
;    disk open error   |
;**********************|
; Open operation failed. Set carry flag and return to caller
        STC                     ; Signal error
        RET
;**********************|
;   read font into RAM |
;**********************|
OK_XOPEN:
        MOV     FONT_HANDLE,AX          ; Store file handle
NEW_128:
        MOV     BX,FONT_HANDLE
        LEA     DX,DATA_BUF             ; Buffer for data storage
        PUSH    DI              ; Save buffer pointer
        CALL    READ_128        ; Read sector into buffer
        POP     DI              ; Restore buffer pointer
        CMP     AX,0            ; Test for end of file
        JNE     MOVE_128        ; Go if not at end of file
;**********************|
;    end of file       |
;**********************|
        MOV     BX,FONT_HANDLE  ; Handle for font file
        CALL    CLOSE_FILE      ; Library routine
        JMP     END_OF_READ
;**********************|
;   move sector to     |
;    font buffer       |
;**********************|
; At this point DATA_BUF holds 128 bytes from disk file
; DI --> storage position in the font's buffer
MOVE_128:
        MOV     CX,128          ; Byte counter
        LEA     SI,DATA_BUF     ; Pointer to data just read
PLACE_128:
        MOV     AL,[SI]         ; Byte from DATA_BUF
        MOV     [DI],AL         ; Into font's buffer
        INC     SI              ; Bump pointers
        INC     DI
        LOOP    PLACE_128       ; Continue until all sector read
; At this point the 128 bytes newly read from the disk file are
; stored in the font's buffer
        JMP     NEW_128
END_OF_READ:
        CLC
        RET
XGA_FONT        ENDP
;
;
CODE    ENDS
        END     START           ; Reference to label at which
                                ; execution begins

    ENDS
        END     START         