; Module name: BITIO.ASM                      Library:   GRAPHSOL.LIB
; Created: April 22, 1992
;********************************************************************
;********************************************************************
;                             BITIO.ASM
;********************************************************************
;********************************************************************
; Description:
; Library module for the following functions:
;     1. TIFF file operations
;        a. load a bi-level TIFF file
;        b. display a TIFF image
;     2. ColorPro plotter support routines
;        a. initialize serial port to ColorPro default values
;        b. send character to ColorPro plotter (using serial line)
;        c. receive one character from ColorPro
;        d. receive and store ColorPro data string
;        e. get ColorPro status
;        f. VGA display emulation using ColorPro plotter
;        g. flush old characters from RS-232C receiver register
;     3. PCL printer support
;        a. send control string to PCL printer
;
; Note: The library routines that are contained in this module can
;       be used from the high level languages C, Pascal, and BASIC
;       as well as in programs coded directly in Assembly Language.
;       Appendix B explains interfacing the library routines with
;       these languages.
;*******************************************************************
;                          ROUTINES
;*******************************************************************
; Public procedures in this module:
; NAME             DESCRIPTION
; SHOW_TIFF        Display a graphics file in TIFF format
; LOAD_TIFF        Load a graphics file in TIFF format
; CP_COMMAND       Send command string to ColorPro plotter
; CP_OUTPUT        Receive and store ColorPro data string
; CP_STATUS        Get status of ColorPro plotter hardware
; CP_PLOT          ColorPro output using PIXEL_WRITE (in VGA3)
; GET_CP_CHAR      Receive ColorPro character through serial line
; FLUSH_232        Remove garbage in RS-232C receiver register 
; SERIAL_2_CP      Initialize serial port to ColorPro default
; PCL_STRING       Send control string to PCL printer
;
;*******************|
;      PUBLICS      |
;*******************|
PUBLIC  SHOW_TIFF
PUBLIC  LOAD_TIFF
PUBLIC  CP_COMMAND
PUBLIC  CP_OUTPUT
PUBLIC  CP_STATUS
PUBLIC  CP_PLOT
PUBLIC  GET_CP_CHAR
PUBLIC  FLUSH_232
PUBLIC  SERIAL_2_CP
PUBLIC  PCL_STRING
;
;**********************|
;     externals        |
;**********************|
; In SOLUTION.LIB
EXTRN   OPEN_FILE:      FAR
EXTRN   CLOSE_FILE:     FAR
EXTRN   READ_128:       FAR
EXTRN   KBR_FLUSH:      FAR
EXTRN   BIN_TO_ASC:     FAR
EXTRN   ASC_TO_BIN:     FAR
;
; In other modules of GRAPHSOL.LIB
EXTRN   PIXEL_WRITE:    FAR
;
;****************************************************************
;                      TIFF file data segment
;****************************************************************
TIFF_DATA       SEGMENT
;**********************|
; storage for TIFF file|
;**********************|
; Maximum size of TIFF file is 20K
TIFF_FILE       DB      20480 DUP (00H)
;**********************|
;   disk file buffer   |
;**********************|
DATA_BUF        DB      128 DUP (00H)   ; Disk data storage area
                DW      0
;**********************|
;    copy of caller's  |
;     display block    |
;**********************|
USER_BLOCK      DW      0       ; x coordinate
                DW      0       ; y coordinate
                DW      0       ; Offset of bit map from start
                                ; of file
                DW      0       ; Number of pixel rows
                DB      0       ; Number of bytes per row
                DB      0       ; I R G B color code
                DB      15 DUP (00H)
                DW      0
;**********************|
;       bit image      |
;**********************|
BIT_IMAGE       DB      38400 DUP (00H)
;
TIFF_DATA       ENDS
;
;****************************************************************
;                       ColorPro data segment
;****************************************************************
CP_DATA         SEGMENT
;**********************|
;   ColorPro status    |
;**********************|
; Plotter status command and storage area for ASCII status string
;
STAT_$  DB      'OS ;'          ; Output status command
        DB      0
ST_BUF  DB      '   ',00H       ; Storage for plotter status string
                                ; (up to 3 ASCII digits)
;**********************|
;    CP_PLOT command   |
;**********************|
CPP_$   DB      'PD '
CPP_CX  DB      '     '           ; Absolute x axis coordinate
        DB      ','               ; variable
CPP_DX  DB      '     '           ; Absolute y axis coordinate
        DB      ';'           ; variable
        DB      00H
CP_DATA         ENDS
;
;********************************************************************
;                            code segment
;********************************************************************
;
P_CODE  SEGMENT PUBLIC
        ASSUME  CS:P_CODE
;
;*****************************************************************
;*****************************************************************
;                          procedures
;*****************************************************************
;*****************************************************************
;
; Code segment variables used by library procedures
X_COORD         DW      0000H   ; Storage for x coordinate
BYTES           DB      0H      ; Number of bytes per block row
COUNT_8         DB      8       ; Bit counter for the VARI_PATTERN
PIX_ROWS        DW      0       ; Number of pixel rows in map
;
; Data variables exclusive for the SHOW_TIFF procedure
STRIP_OFFSET    DW      0       ; Offset of bit map from start of
                                ; TIFF file
TIFF_HANDLE     DW      0       ; File handle for TIFF disk file
IMAGE_SIZE      DW      0       ; Image dimension
EXP_COUNT       DW      0       ; Byte counter for expansion
USERS_DS        DW      0       ; Caller's DS segment
;
;*****************************************************************
;                      display raster image
;*****************************************************************
;
SHOW_TIFF       PROC    FAR
; Procedure to display a bit mapped graphics file in TIFF format
; using VGA, XGA, or SVGA system
; Display system selection requires a call to the SET_DEVICE
; procedure in the VGA3 module of this library
;
; On entry:
;        SI ==> caller's display block formatted as follows:
;
;   OFFSET    STORAGE    CONTENTS
;     0        WORD      x screen coordinate for image
;     2        WORD      y screen coordinate for image
;     4        WORD      Offset of bit map from start of file
;                        (from the StripOffset tag)
;     6        WORD      number of vertical rows in bit map
;                        (from the ImageLength tag)
;     8        BYTE      number of horizontal bytes in bit map
;     9        BYTE      I R G B color code for bit display
;     10      STRING     ASCIIZ string containing the filename
;                        for the TIFF file
;
;
        CALL    FAR PTR LOAD_TIFF       ; Local procedure to load
                                ; TIFF file into RAM and decompress
                                ; image
        LEA     SI,USER_BLOCK   ; Reset entry pointer
; Initialize registers
        MOV     CX,WORD PTR [SI]        ; x coordinate
        MOV     CS:X_COORD,CX              ; Store in variable
        ADD     SI,2                    ; Bump pointer
        MOV     DX,WORD PTR [SI]        ; y coordinate
        ADD     SI,2                    ; Bump pointer
        MOV     BP,WORD PTR [SI]        ; Bit map offset
        ADD     SI,2
        MOV     AX,WORD PTR [SI]        ; Number of pix rows
        MOV     CS:PIX_ROWS,AX          ; Store in variable
        ADD     SI,2                    ; Bump pointer
        MOV     BH,BYTE PTR [SI]        ; Bytes per block
        MOV     CS:BYTES,BH                ; Store in variable
        INC     SI                      ; Bump pointer
        MOV     AL,[SI]         ; Color code to AL
        MOV     CS:COUNT_8,8       ; Prime bit counter
        LEA     DI,BIT_IMAGE    ; Pointer to unpacked bit map
; Register and variables after routine initialization:
;               CX = x coordinate of block start
;               DX = y coordinate of block start
;               BP = offset from start of TIFF file to bit map
;               BH = number of bytes per block row
;               AL = color code
;               DI ==> image bit map
;               CS:X_COORD = current x coordinate
;               CS:BYTES = number of bytes per block row
;               CS:PIX_ROWS = number of pixel rows in block
;               CS:COUNT_8 = 8
;**********************|
;   reset ES segment   |
;**********************|
        PUSH    AX              ; Save accumulator
        MOV     AX,0A000H       ; Video buffer segment base
        MOV     ES,AX           ; To extra segment
        POP     AX              ; Restore AX
;**********************|
;  display image block |
;**********************|
DISPLAY_BYTE_T:
        MOV     AH,[DI]         ; High nibble to AH
; Test for al zero display pattern
        CMP     AH,0            ; Nothing to display?
        JNE     TEST_BIT_T      ; Continue if not zero
        ADD     CX,8            ; Skip this byte
        JMP     NEXT_BYTE_T     ; Skip byte
TEST_BIT_T:
        TEST    AH,10000000B    ; Is high bit set?
        JZ      NEXT_BIT_T      ; Bit not set
; Set the pixel
        PUSH    AX              ; Save entry registers
        PUSH    BX
        CALL    PIXEL_WRITE     ; Procedure in VGA3 to write
                                ; pixel in VGA, XGA, or SVGA
        POP     BX              ; Restore registers
        POP     AX
NEXT_BIT_T:
        SAL     AH,1            ; Shift AH to test next bit
        INC     CX              ; Bump x coordinate counter
        DEC     CS:COUNT_8         ; Bit counter
        JZ      NEXT_BYTE_T     ; Exit if counter rewound
        JMP     TEST_BIT_T      ; Continue
; Index to next byte in row, if not at end of row
NEXT_BYTE_T:
        DEC     BH              ; Bytes per row counter
        JZ      NEXT_ROW_T      ; End of graphic row
BYTE_ENTRY_T:
        INC     DI              ; Bump graphic code pointer
        MOV     CS:COUNT_8,8       ; Reset bits counter
        JMP     DISPLAY_BYTE_T
; Index to next row
NEXT_ROW_T:
; Test for last graphic row
        DEC     CS:PIX_ROWS     ; Row counter
        JZ      GRAPH_END_T     ; Done, exit
        MOV     BH,CS:BYTES        ; Reset bytes counter
        INC     DX              ; Bump y coordinate control
        MOV     CX,CS:X_COORD      ; Reset x coordinate control
        JMP     BYTE_ENTRY_T
GRAPH_END_T:
        MOV     AX,CS:USERS_DS  ; Restore caller's DS
        MOV     DS,AX
        CLC                     ; No error reported
        RET
SHOW_TIFF       ENDP
;******************************************************************
;
LOAD_TIFF       PROC    FAR
; procedure for loading a bi-level TIFF file
; On entry:
;        SI ==> caller's display block formatted as follows:
;
;   OFFSET    STORAGE    CONTENTS
;     0        WORD      x screen coordinate for image
;     2        WORD      y screen coordinate for image
;     4        WORD      Offset of bit map from start of file
;                        (from the StripOffset tag)
;     6        WORD      number of vertical rows in bit map
;     8        BYTE      number of horizontal bytes in bit map
;     9        BYTE      I R G B color code for bit display
;     10      STRING     ASCIIZ string containing the filename
;                        for the TIFF file
;
; Assumptions and limitations:
;    1. Bit map un-compressed or in PackBits compression mode
;    2. Unpacked bit map not to exceed 255 pixel rows
;    3. TIFF bi-level image
;    4. Target display system resolution not to exceed 64K
;
;**********************|
;     ES to local data |
;**********************|
        PUSH    ES              ; Save video buffer base
        MOV     AX,TIFF_DATA
        MOV     ES,AX
; Move user's display block to TIFF_DATA segment
        MOV     CX,25           ; 25 bytes in block
        LEA     DI,ES:USER_BLOCK
MOVE_TO_USER:
        MOV     AL,[SI]         ; Get caller's byte
        MOV     ES:[DI],AL      ; Move to local segment
        INC     SI              ; Bump pointers
        INC     DI
        LOOP    MOVE_TO_USER
;**********************|
;   DS to local data   |
;**********************|
        MOV     AX,DS           ; Caller's DS to AX
        MOV     CS:USERS_DS,AX     ; Store in variable
; Change DS to local segment
        MOV     AX,TIFF_DATA    ; Local segment
        MOV     DS,AX
        ASSUME  DS:TIFF_DATA    ; New ASSUME statement
        POP     ES              ; Restore video buffer base to ES
        LEA     SI,USER_BLOCK   ; Reset entry pointer
;**********************|
; store strip offset   |
;**********************|
        MOV     AX,WORD PTR [SI+4]      ; From display block
        MOV     CS:STRIP_OFFSET,AX      ; Store in CS variable
;**********************|
; calculate image size |
;**********************|
; Image dimensions are stored in the display block
; The image size is required by during bit map decompression
        MOV     AX,WORD PTR [SI+6]      ; Number of pixel rows
        MOV     BL,BYTE PTR [SI+8]      ; Number of bytes
        MOV     BH,0            ; Clear high of multiplier
        MUL     BX              ; AX:DX = AX * BX
        MOV     CS:IMAGE_SIZE,AX        ; Store in CS variable
; Note that the high order byte of the product can be discarded
; since code assumes that the display resolution is less than
; 64K
;*********************|
;    open TIFF file   |
;*********************|
        ADD     SI,10           ; Index to ASCIIZ string area
        MOV     DX,SI           ; For OPEN_FILE procedure
        CALL    OPEN_FILE       ; Procedure in SOLUTION.LIB
        JNC     OK_TIFF_OPEN    ; Continue if no carry
;**********************|
;  open operation fail |
;**********************|
        POP     SI              ; Restore context
        MOV     AX,CS:USERS_DS  ; Restore caller's DS
        MOV     DS,AX
        STC                     ; Code for error return
        RET
;**********************|
;    set ES segment    |
;    to local data     |
;**********************|
; Note: The SHOW_TIFF procedure uses the segment TIFF_DATA for
;       storing TIFF image data
        MOV     AX,TIFF_DATA    ; Set to local segment
        MOV     ES,AX
OK_TIFF_OPEN:
;**********************|
;   read TIFF file     |
;      into RAM        |
;**********************|
        LEA     DI,TIFF_FILE    ; Storage buffer for file
        MOV     CS:TIFF_HANDLE,AX       ; Store file handle
NEW_128:
        MOV     BX,CS:TIFF_HANDLE
        LEA     DX,DATA_BUF     ; Buffer for data storage
        PUSH    DI              ; Save buffer pointer
        CALL    READ_128        ; Read sector into buffer
        POP     DI              ; Restore buffer pointer
        CMP     AX,0            ; Test for end of file
        JNE     MOVE_128        ; Go if not at end of file
;**********************|
;    end of file       |
;**********************|
        MOV     BX,CS:TIFF_HANDLE       ; File handle
        CALL    CLOSE_FILE      ; Library routine
        JMP     END_OF_READ
;**********************|
;   move sector to     |
;    TIFF buffer       |
;**********************|
; At this point DATA_BUF holds 128 bytes from disk file
; DI --> storage position in the TIFF file RAM buffer
MOVE_128:
        MOV     CX,128          ; Byte counter
        LEA     SI,DATA_BUF     ; Pointer to data just read
PLACE_128:
        MOV     AL,[SI]         ; Byte from DATA_BUF
        MOV     [DI],AL         ; Into font's buffer
        INC     SI              ; Bump pointers
        INC     DI
        LOOP    PLACE_128       ; Continue until all sector read
; At this point the 128 bytes newly read from the disk file are
; stored in the font's buffer
        JMP     NEW_128
;*****************************|
;        unpack bit map       |
;*****************************|
; Unpacking logic for TIFF PackBits scheme
; PackBits packages consist of 2 bytes. The first byte (n) encodes
; the following options:
;       1. if n is in the range 0 to 127 then next n+1 bytes are
;          to be interpreted as literal values
;       2. if n is in the range -127 to -1 then the following byte
;          is to be repeated -n+1 times
;       3. if n = 128 then no operation is executed
END_OF_READ:
        MOV     CS:EXP_COUNT,0  ; Clear counter
        LEA     DI,BIT_IMAGE    ; Destination in display block
        LEA     SI,TIFF_FILE    ; Pointer to start of file
        ADD     SI,CS:STRIP_OFFSET ; Add offset to bit map
; SI ==> start of image if there is a single strip
TEST_N_BYTE:
        MOV     AL,[SI]         ; Get n byte
        CMP     AL,128          ; Code for NOP
        JB      LITERAL_CODE    ; Go if in the literal range
        JA      REPEAT_CODE     ; Go if in repeat range
; Code is 128 (NOP)
        INC     SI              ; Skip NOP code
        JMP     NEXT_PACK_CODE  ; Continue
;**********************|
;   literal expansion  |
;**********************|
LITERAL_CODE:
        MOV     CL,AL           ; Counter to CL
        MOV     CH,0            ; Clear high byte of counter
        INC     CX              ; Add 1
        INC     SI              ; Skip n byte
        ADD     CS:EXP_COUNT,CX    ; Add bytes to counter
LIT_MOVE:
        MOV     AL,[SI]         ; Get literal byte
        NOT     AL              ; Invert white and black bits
        MOV     [DI],AL         ; Place in bit map
        INC     DI              ; Bump pointers
        INC     SI
        LOOP    LIT_MOVE
        JMP     NEXT_PACK_CODE
;**********************|
;  repeated expansion  |
;**********************|
REPEAT_CODE:
        NEG     AL              ; Negate to convert 2s complement
                                ; representation
        MOV     CL,AL           ; Counter to CL
        MOV     CH,0            ; Clear high byte of counter
        INC     CX              ; Add 1
        INC     SI              ; Skip n byte
        ADD     CS:EXP_COUNT,CX    ; Add bytes to counter
        MOV     AL,[SI]         ; Get byte to repeat
        NOT     AL              ; Invert black and white bits
        INC     SI              ; Skip to next n byte
EXP_MOVE:
        MOV     [DI],AL         ; Place byte in buffer
        INC     DI              ; Bump bit map pointer
        LOOP    EXP_MOVE
;**********************|
;  get next pack code  |
;**********************|
; CS:EXP_COUNT holds the total bytes in bit map at this point
; CS:IMAGE_SIZE holds the total bytes in the expanded bit map
NEXT_PACK_CODE:
        MOV     AX,CS:EXP_COUNT    ; Bytes now in bit map
        CMP     AX,CS:IMAGE_SIZE   ; Compare with map size
        JAE     DISPLAY_IMAGE   ; Go if at end of image
        JMP     TEST_N_BYTE
DISPLAY_IMAGE:
        RET
LOAD_TIFF       ENDP
;*****************************************************************
;                   ColorPro plotter routines
;*****************************************************************
;
SERIAL_2_CP     PROC    FAR
; Initialize serial port to ColorPro default setting of its rear
; panel switches, as follows:
;                             I     O
;                          B1   ==>   |
;                          B2   <==   |  Baud rate = 9600
;                          B3   ==>   |
;                          B4   <==   |
;
;                          US   <==   |  Paper type = 8 1/2 x 11
;
;                          SI   ==>   |  No parity
;                          S2   ==>   |
;
; Bit mask: 111xxxxx ...... Baud = 9600
;           xxx00xxx ...... Parity = none
;           xxxxx0xx ...... Stop bits = 1
;           xxxxxx11 ...... Word length = 8
;           11100011
;
        MOV     AL,11100011B    ; Bit mask for ColorPro default
        MOV     AH,0            ; BIOS service request number
        MOV     DX,0            ; COM1 in all hardware types
        INT     14H             ; BIOS service request
        RET
SERIAL_2_CP     ENDP
;*****************************************************************
;
CARD_BASE       DW      0       ; Address of serial port
;
;******************************************************************
CP_COMMAND      PROC    FAR
; Send command string to ColorPro plotter
; On entry:
;         DS:SI ==> command string, terminated in NULL byte
;
        PUSH    AX
        PUSH    BX
        PUSH    CX
        PUSH    DX
        PUSH    DI
;**********************|
; get serial port base |
;       address        |
;**********************|
; Get address of RS-232C card from BIOS data area
        PUSH    DS              ; Save operation segment
        MOV     DX,0            ; BIOS data area segment
        MOV     DS,DX           ; Data segment to BIOS area
        MOV     CX,DS:0400H     ; Offset of card 1
        POP     DS                      ; Restore program DS
        MOV     CS:CARD_BASE,CX    ; Store in code segment
COMMAND_CHAR:
        MOV     AL,[SI]         ; Get command character
        CMP     AL,0            ; Test for NULL
        JE      COMMAND_END     ; Go if terminator
        CALL    SEND_ONE_CP     ; Send character top ColorPro
        INC     SI              ; Bump message point
        JMP     COMMAND_CHAR    ; Continue
COMMAND_END:
        POP     DI
        POP     DX
        POP     CX
        POP     BX
        POP     AX
        RET
CP_COMMAND      ENDP
;******************************************************************
;
SEND_ONE_CP     PROC    NEAR
; Send AL character through the RS 232C line using hardwired
; handshake
; Note:  DTR signal on plotter's line # 20 is wired to the 
;        serial port's DSR line on computer's line # 6
;
        PUSH    AX              ; Save character to send
;**********************|
;  check status line   |
;**********************|
WAIT_FOR_DTR:
        MOV     DX,CS:CARD_BASE    ; RS-232C base port address
        ADD     DX,6            ; Modem status register
        IN      AL,DX           ; Read byte at status register
        JMP     SHORT $+2       ; I/O delay
        MOV     AH,AL           ; Store in AH
        DEC     DX              ; Line status register
        IN      AL,DX           ; Status
        JMP     SHORT $ + 2     ; I/O delay
; At this point AH holds the modem status register contents and
; AL holds the line status register
; Bit 5 of the modem status register holds the state of the
; plotter's DTR signal, as received in the computers DSR line
; Bit 5 of the line status register holds state of the transmitter
; holding register (1 = THR is empty). The THR must be empty before
; data can be sent
        AND     AX,2020H        ; Mask off all other bits
        XOR     AX,2020H        ; Test for both high
; XORing with the original mask sets the zero flag if both bits
; were set
        JNZ     WAIT_FOR_DTR    ; Repeat if not high
; At this point the plotter is ready to receive
;**********************|
;    send character    |
;**********************|
        POP     AX              ; Restore character to send
        MOV     DX,CS:CARD_BASE    ; Transmitter holding register
        OUT     DX,AL           ; Send character
        JMP     SHORT $ + 2     ; I/O delay
        RET
SEND_ONE_CP     ENDP
;
;*****************************************************************
GET_CP_CHAR     PROC    FAR
; Procedure to receive one character through the serial line
; Processing assumes that the communication parameters have
; been previously selected to match the computer and the
; ColorPro plotter
;
; On exit:
;        Carry clear if character received
;        AL = A8H if transmission error detected
;        Carry set if user aborted by pressing a key
;
; First flush keyboard buffer of old characters
        CALL    KBR_FLUSH         ; Routine in SOLUTION.LIB
;**********************|
; get serial port base |
;       address        |
;**********************|
; Get address of RS-232C card from BIOS data area
        PUSH    DS              ; Save operation segment
        MOV     DX,0            ; BIOS data area segment
        MOV     DS,DX           ; Data segment to BIOS area
        MOV     CX,DS:0400H     ; Offset of card 1
        POP     DS                      ; Restore program DS
; Card base address is not in CX
        MOV     DX,CX           ; Serial card base address
        ADD     DX,5            ; Status register
CHK_STAT:
        IN      AL,DX           ; Character ready?
        TEST    AL,1            ; Data ready bit on status
                                ; register
        JNZ     D_READY         ; Get data
        TEST    AL,1EH          ; Bits 1, 2, 3 and 4 are error
                                ; bits
        JNZ     DATA_ERROR      ; Go if error bit set
; Test for key pressed
        PUSH    DX              ; Save card base address
        MOV     AH,1            ; BIOS service request number
        INT     16H             ; for keyboard status
        POP     DX              ; Restore card address
        JZ      CHK_STAT        ; No key pressed, continue
; Key was pressed during line monitoring
        STC                     ; Carry flag is return code
        RET
D_READY:
        SUB     DX,5            ; Return DX to port base address
        IN      AL,DX           ; Read character into AL
        CLC                     ; Normal exit
        RET
DATA_ERROR:
        MOV     AL,0A8H         ; A8H = error received
        CLC                     ; Normal exit
        RET
GET_CP_CHAR     ENDP
;
;*****************************************************************
;
CP_OUTPUT       PROC    FAR
; Receive ColorPro data through the serial port and store string
; in buffer passed by the caller. Reception ends if the terminator
; code 0DH is received or if the user presses any key
; On entry:
;       DS:DI --> used buffer for plotter data
; On exit:
;       carry cleared if no user abort
;       carry set if user pressed key during reception
;
GET_IDS:
        PUSH    DI              ; and pointer
        CALL    FAR PTR GET_CP_CHAR     ; RS-232C receive routine
        POP     DI              ; Restore registers
        JC      USER_ABORT      ; User pressed a key
; At this point AL holds the character received
        CMP     AL,0DH          ; End of message
        JE      MESSAGE_END     ; Received
; Character received is not CR
        MOV     [DI],AL         ; Store character in buffer
        INC     DI              ; Bump buffer pointer
        JMP     GET_IDS         ; Continue reception
;**********************|
;     user aborted     |
;**********************|
USER_ABORT:
        STC
        RET
;**********************|
;  display ID message  |
;**********************|
MESSAGE_END:
        CLC
        RET
CP_OUTPUT       ENDP
;*****************************************************************
;
CP_STATUS       PROC    FAR
; Get status of ColorPro plotter and return in the DX register
; On exit:
;        carry clear if valid status data
;        carry set if aborted by caller
;
        PUSH    DS              ; Save caller's data segment
        MOV     AX,CP_DATA      ; Local data segment
        MOV     DS,AX           ; To DS
        ASSUME  DS:CP_DATA
;
        LEA     SI,STAT_$       ; Output plotter status command
        CALL    FAR PTR CP_COMMAND      ; Send command to ColorPro
        LEA     DI,ST_BUF       ; Buffer for plotter string
        CALL    FAR PTR CP_OUTPUT       ; Library procedure in BITIO
                                        ; module
        JNC     OK_STRING1      ; Go if not aborted by caller
        POP     DS              ; Restore caller's DS
        STC                     ; Aborted by caller
        RET
OK_STRING1:
        LEA     BX,ST_BUF       ; Pointer to ASCII character buffer
        CALL    ASC_TO_BIN      ; Procedure in SOLUTION.LIB
; At this point DX holds binary value of plotter status
        POP     DS              ; Restore caller's DS
        CLC                     ; Not aborted by caller
        RET
CP_STATUS       ENDP
;******************************************************************
;
CP_PLOT         PROC    FAR
; This procedure is called by PIXEL_WRITE in the VGA3 module
; when the DEVICE switch is set to the ColorPro plotter ("P")
; On entry:
;          CX holds x coordinate (range 0 to 639)
;          DX holds y coordinate (range 0 to 479)
; On exit:
;         Plotter pen moved to new coordinates (draw mode)
;
; Procedure assumptions:
;      The plotter's coordinate system has must have been
;      previously reset to emulate the VGA screen in 640 by 480
;      resolution (mode number 18) and that the absolute mode has
;      been set
;
; Set DS to local data
        PUSH    DS              ; Save caller's data segment
        MOV     AX,CP_DATA      ; Local data segment
        MOV     DS,AX           ; To DS
        ASSUME  DS:CP_DATA
;
        PUSH    SI
        PUSH    DI              ; Save registers
        PUSH    AX
        PUSH    BX
        PUSH    CX
        PUSH    DX
        PUSH    DX              ; DX is pushed twice
; Convert x screen coordinate to ASCII and store in command string
        MOV     DX,CX           ; For conversion
        LEA     DI,CPP_CX       ; Pointer to x coordinate area
                                ; in plotter command string
        CALL    BIN_TO_ASC      ; Procedure in SOLUTION.LIB
; Convert y screen coordinate to ASCII and store in command string
        POP     DX              ; y coordinate from stack
        LEA     DI,CPP_DX       ; Pointer to y coordinate area
        CALL    BIN_TO_ASC      ; Local conversion routine
;**********************|
;   draw to new point  |
;**********************|
        LEA     SI,CPP_$        ; Command string
        CALL    FAR PTR CP_COMMAND
;
        POP     DX              ; Restore machine context
        POP     CX
        POP     BX
        POP     AX
        POP     DI
        POP     SI
        POP     DS              ; Restore caller's DS
        RET
CP_PLOT         ENDP
;*****************************************************************
;
PCL_STRING      PROC    FAR
; Send control string (terminated in 00H) to PCL printer
; On entry:
;       DS:SI --> PCL command string
;
        PUSH    BX              ; Save operational registers
        PUSH    CX
        PUSH    DX
        PUSH    DI
;
SEND_CHAR:
        MOV     AL,[SI]         ; Get character
        CMP     AL,0            ; Test for message terminator
        JNE     NOT_END         ; Continue if not terminator
        JMP     END_PCL         ; End of message
NOT_END:
        MOV     DX,0            ; Printer port
; Test printer status and send character when printer ready
; At this point:
;         AL = character to send
;         DX = printer port (valid range is 0, 1, and 2)
;**********************|
;    initialize port   |
;**********************|
INIT_PORT:
        MOV      AH,1            ; Service request number
        INT      17H             ; Transfer to BIOS
        TEST     AH,80H          ; Test printer not busy bit 
        JNZ      NOT_BUSY
        TEST     AH,1            ; Time out bit
        JNZ      STATUS_ERROR
        JMP      INIT_PORT       ; Repeat initialization and
                                ; status check
;*******************|
;    error exit     |
;*******************|
; Code notifies the caller of a status error by setting the
; carry flag
STATUS_ERROR:
        POP     DI              ; Restore registers
        POP     DX
        POP     CX
        POP     BX
        STC                     ; Error return code  
        RET
;******************|
;  send character  |
;******************|
NOT_BUSY:
        MOV      AH,0            ; Function request code
        INT      17H             ; Transfer to BIOS
;******************|
; next character   |
;******************|
        INC     SI              ; Bump message pointer
        JMP     SEND_CHAR
;******************|
;   end of message |
;******************|
END_PCL:
        POP     DI              ; Restore registers
        POP     DX
        POP     CX
        POP     BX
        CLC                     ; No error return code  
        RET
PCL_STRING      ENDP
;*****************************************************************
;
FLUSH_232       PROC    FAR
; Procedure to remove any old characters from the RS-232C 
; receiver register
        MOV     DX,CS:CARD_BASE  ; Serial card base address
FLUSH_ONE:
        ADD     DX,5            ; Status register
        IN      AL,DX           ; Character ready?
        TEST    AL,00000001B    ; Data ready bit on status reg.
        JNZ     REMOVE_DATA     ; Get data
; No data in receiver register
        RET                     ; Line is flushed
REMOVE_DATA:
        MOV     DX,CARD_BASE    ; Receiver buffer register
        IN      AL,DX           ; Read character into AL
        JMP     FLUSH_ONE       ; Continue
FLUSH_232       ENDP
;******************************************************************
;
;
P_CODE  ENDS
        END
