; Module name: DAVIDEO.ASM                    Library:           .LIB
; Created:
; Update:
;
;********************************************************************
;********************************************************************
;                           DAVIDEO.ASM
;********************************************************************
;********************************************************************
;
; Library module for video services using direct buffer access
;
;*******************|
;      PUBLICS      |
;*******************|
PUBLIC	ES_TO_VIDEO
PUBLIC	CLEAR_SCREEN
PUBLIC	SET_DI_PTR
PUBLIC	SHOW_BLOCK
PUBLIC	SHOW_CX_CHARS
PUBLIC	SHOW_CX_VERT
;
;--------------------------------------------------------------------
;
;
P_CODE	SEGMENT PUBLIC
	ASSUME	CS:P_CODE
;
;*******************************************************************
ES_TO_VIDEO	PROC	FAR
; Set ES register to the base address of the video buffer according
; to the video mode stored in BIOS address 0040:0049
; On entry:
;         nothing
; On exit:
;         ES --> segment address of video buffer for alphanumeric
;                display
;         carry clear
	PUSH	AX		; Save register
	PUSH	DS		; Save segment
	MOV	AX,0040H	; Segment base of data area
	MOV	DS,AX		; To data segment
	MOV	AL,[0049H]	; Current mode
	POP	DS		; Restore segment
	CMP	AL,7		; Code for monochrome card
	JE	SET_MONO_BASE	; Set base for mono card
;********************|
;     color card     |
;********************|
	MOV	AX,0B800H	; Color system base
	MOV	ES,AX		; to ES
	POP	AX		; Restore entry register
	CLC			; No error detection
	RET
;*********************|
;  monochrome card    |
;*********************|
SET_MONO_BASE:
	MOV	AX,0B000H	; Video base in monochrome systems
	MOV	ES,AX		; To ES
	POP	AX		; Restore entry register
	CLC			; No error detection
	RET
ES_TO_VIDEO	ENDP
;*******************************************************************
;
CLEAR_SCREEN	PROC	FAR
; Clear the video display with a block move
; On entry:
;         ES --> video buffer base address (ES_TO_VIDEO)
; On exit:
;         carry clear
	PUSH	AX		; Save entry registers
	PUSH	CX
	PUSH	DI
	MOV	DI,0		; Start at offset 0 in buffer
	MOV	AL,20H		; Blank byte
	MOV	AH,7		; Normal display attribute
	MOV	CX,2000		; Repeat 2000 times
	CLD			; Forward direction in memory
	REP	STOSW		; Store 2000 words in buffer
	POP	DI
	POP	CX
	POP	AX		; Restore entry registers
	CLC			; No error detection
	RET
CLEAR_SCREEN	ENDP
;*******************************************************************
;
SET_DI_PTR	PROC	FAR
; Set DI to the offset address in the video buffer that corresponds
; with a certain screen point, expressed as a row and column position
; On entry:
;         DH = screen row (0 to 24)
;         DL = screen column (0 to 79)
; On exit:
;  No error:
;         DI --> buffer offset (row * 160) + (column * 2)
;         carry clear
;  Error:
;        carry set. AX destroyed.
;
	CALL	TEST_RANGE	; Local procedure
	PUSH	AX		; Save accumulator
	PUSH	DX		; and DX
	MOV	AL,160		; Bytes per row
	MUL	DH		; AX = 160 * DH
	MOV	DI,AX		; Save partial result in DI
	MOV	AL,2		; Bytes per column
	MUL	DL		; AX = DL * 2
	ADD	DI,AX		; Add in previous result
	POP	DX		; Restore DX
	POP	AX		; And accumulator
	CLC			; No error detection
	RET
SET_DI_PTR	ENDP
;*******************************************************************
;
SHOW_BLOCK	PROC	FAR
; Display a preformatted block message
; On entry:
;          SI --> start of block message
; Message format:
;       Offset:         Function:
;              0 ------ Display row for start of message
;              1 ------ Display column for start of message
;              2 ------ Attribute for block
; Embedded codes:
;              00H = End of message
;              FFH = End of row
; On exit:
;        carry clear
;
	PUSH	AX		; Save entry registers
	PUSH	CX
	PUSH	DX
	MOV	DH,[SI]		; Get row
	INC	SI		; Point to start column byte
	MOV	DL,[SI]		; Column
	MOV	CL,DL		; Save start column in CL
	CALL	SET_DI_PTR	; Set DI to offset in buffer
	INC	SI		; Point to attribute byte
	MOV	AH,[SI]		; To AH
	INC	SI
GET_AND_SHOW:
	MOV	AL,[SI]		; Character
;*********************|
;   test for control  |
;      characters     |
;*********************|
	CMP	AL,00H		; Terminator 
	JE	SHOW_END
	CMP	AL,0FFH		; End of line
	JE	LINE_END
; Display it by storing character in video buffer
	CLD			; Forward direction
	STOSW			; Store word
	INC	DL		; Next column
;*********************|
;   bump pointer      |
;*********************|
BUMP_PTRS:
	CALL	SET_DI_PTR
	INC	SI		; Bump message pointer
	JMP	GET_AND_SHOW
;********************|
;    end of line     |
;********************|
LINE_END:
	INC	DH		; Bump row pointer
	MOV	DL,CL		; Reset column to start column
	JMP	BUMP_PTRS
;********************|
;   end of routine   |
;********************|
SHOW_END:
	POP	DX		; Restore registers
	POP	CX
	POP	AX
	CLC			; No error detection
	RET
SHOW_BLOCK	ENDP
;*******************************************************************
;
SHOW_CX_CHARS	PROC	FAR
; Display CX number of characters
; On entry:
;         AL = Character to display
;         AH = Attribute (if AH = 0 then attribute is not changed)
;         DH = screen row
;         DL = screen column
;         CX = number of characters to repeat
; On exit:
;        carry clear if no error
;        carry set if error. AX destroyed
;
	CALL	SET_DI_PTR
	JNC	OK_SHOW_CX	; No error reported
	RET			; Error return
OK_SHOW_CX:
	CMP	AH,0		; Test for attribute code
	JE	USE_BYTE_MOVE	; Use installed attributes
;*********************|
;   attribute in AH   |
;*********************|
	CLD			; Forward direction
	REP	STOSW		; Store AX by [DI], bump twice
				; and repeat
	CLC			; No error detection
	RET
;********************|
;  current attribute |
;********************|
USE_BYTE_MOVE:
	CLD			; Forward direction
	STOSB			; Store AL by [DI] and bump once
	INC	DI		; Skip attribute byte
	LOOP	USE_BYTE_MOVE
	CLC			; No error detection
	RET
SHOW_CX_CHARS	ENDP
;*******************************************************************
;
SHOW_CX_VERT	PROC	FAR
; Display CX number of characters vertically using current attribute
; On entry:
;         AL = Character to display
;         DH = screen row
;         DL = screen column
;         CX = number of characters to repeat
; On exit:
;        carry clear
	CALL	SET_DI_PTR
	JNC	SET_VERTS	; No error reported
	RET			; Error return
;*********************|
;   attribute in AH   |
;*********************|
SET_VERTS:
	CLD			; Forward
	STOSB			; Store AL by [DI] and bump once
	INC	DI		; Skip attribute
	ADD	DI,158		; Bump to next row
	LOOP	SET_VERTS
	CLC			; No error detection
	RET
SHOW_CX_VERT	ENDP
;*******************************************************************
;                         internal procedures
;*******************************************************************
TEST_RANGE	PROC	NEAR
; Procedure to test that the screen address passed in the DH and DL
; registers is a valid one
; On entry:
;         DH = screen row (legal range is 0 to 24)
;         DL = screen column (legal range is 0 to 79)
; On exit:
;        carry set if error. AX destroyed. Returns to original caller
;        nothing if no error
	CMP	DH,24		; Test for valid input
	JG	BAD_CR		; Invalid value
	CMP	DL,79		; Maximum column value
	JG	BAD_CR		; Invalid value
; Input is in legal range
	RET			; Return
BAD_CR:
	POP	AX		; Adjust stack
	STC			; Error flag on
	RET			; Return to original caller
TEST_RANGE	ENDP
;
P_CODE	ENDS
	END
:
	POP	AX		; Adjust stack
	STC			; Error flag on
	RET			; Return to ori