; Module name: DISKIO.ASM                    Library:   SOLUTION.LIB
; Created:
; Update:
;
;*******************************************************************
;*******************************************************************
;                             DISKIO.ASM
;*******************************************************************
;*******************************************************************
;
; Library module for disk operations
;
;*******************|
;      PUBLICS      |
;*******************|
PUBLIC	SET_DTA
PUBLIC	GET_FIRST
PUBLIC	GET_NEXT
PUBLIC	OPEN_CREATE
PUBLIC	CLOSE_FILE
PUBLIC	OPEN_FILE
PUBLIC	READ_128
PUBLIC	WRITE_128
;
;*******************|
;     EXTERNALS     |
;*******************|
;
;*******************************************************************
;                              code
;*******************************************************************
;
P_CODE	SEGMENT PUBLIC
	ASSUME	CS:P_CODE
;
;*******************************************************************
SET_DTA		PROC	FAR
; Set memory area to be used by DOS as disk transfer area
; On entry:
;          DX --> 128-bytes buffer to be used as DTA
;
; On exit:
;         carry clear
;
	MOV	AH,26		; DOS service request
	INT	21H
	RET
SET_DTA		ENDP
;*******************************************************************
;
GET_FIRST	PROC	FAR
; Define file or path name and find first matching filename in
; the current or the specified directory, if there is one
;
; Note: this procedure should be used in conjunction with the
; procedure GET_NEXT in processing filenames that can contain
; the DOS global characters "*" and "?"
;
; On entry:
;         DX --> Buffer formatted for DOS service number 10
;             Offset 0 = maximum characters to input
;             Offset 1 = characters actually input, excluding CR
; On exit:
;         Current DTA filled as follows:
;               Byte 0 to 20 = Reserved for DOS
;               Byte 21      = attribute of matched file
;               Byte 22-23   = file time
;               Byte 24-25   = file date
;               Byte 26-27   = LSW of file size
;               Byte 28-29   = MSW of file size
;               Byte 30-42   = FILENAME AND EXTENSION in the form
;                              of an ASCIIZ string
;         carry clear if function was successful
;         carry set if function failed
;         AX = error code as follows:
;               02H = invalid path
;               12H = no matching directory entry found
;
;*****************|
; buffered input  |
; from keyboard   |
;*****************|
	MOV	AH,10		; DOS service request number
	PUSH	DX		; Save buffer start address
	PUSH	DX		; twice
	INT	21H		; To DOS
; Filename now in buffer. Set a null byte in buffer to create an
; ASCIIZ string for the PATH or FILENAME
	POP	DI		; Recover buffer start
	INC	DI		; To input count byte
	MOV	AL,[DI]		; No. of characters in buffer
	MOV	AH,0		; Prepare for addition
	ADD	DI,AX		; DX --> CR byte
	INC	DI		; One more
	MOV	BYTE PTR [DI],0H
				; Set null byte terminator
;*****************|
; search for first|
;     match       |
;*****************|
	POP	DX		; Recover buffer start
	ADD	DX,2		; Index to start of file or path
				; name
	MOV	AH,78		; DOS service request number
				; to find first match
	MOV	CX,0		; Normal attribute for file
	INT	21H
; Carry flag is set if no matched filename found
; If no carry, matched filename can be found as an ASCIIZ string
; in the current DTA + 30. This filename can be used to open the
; file or to obtain the file handle
	RET
GET_FIRST	ENDP
;******************************************************************
;
GET_NEXT	PROC	FAR
; This procedure assumes a previous, successful call to the procedure
; GET_FIRST. It finds the next matching filename that matches the
; filespec, if it exists
;
; On entry:
;         The DTA must contain information from a successful
;         GET_FIRST call
; On exit:
;         carry clear if operation successful
;         Current DTA filled as follows:
;               Byte 0 to 20 = Reserved for DOS
;               Byte 21      = attribute of matched file
;               Byte 22-23   = file time
;               Byte 24-25   = file date
;               Byte 26-27   = LSW of file size
;               Byte 28-29   = MSW of file size
;               Byte 30-42   = FILENAME AND EXTENSION in the form
;                              of an ASCIIZ string
;         carry set if operation failed - no match found
;
	MOV	AH,79		; DOS service request number
				; for next match search
	INT	21H
; Carry flag is set if no next matched filename found
; If no carry, matched filename can be found as an ASCIIZ string
; in the current DTA + 30. This filename can be used to open the
; file or to obtain the file handle
	RET
GET_NEXT	ENDP
;
;*******************************************************************
;
OPEN_CREATE	PROC	FAR
;
; Use DOS buffered keyboard input service to input a filename,
; to open file, and to return a 16-bit file handle that can be used
; for further file access
; The routine's action depends on the setting of the BL register
; On entry:
;         DX --> Buffer formatted as for DOS service number 10
;                  Offset 0 = maximum characters to input
;                  Offset 1 = characters actually input,
;                             excluding CR
;         BL = 0 - Open file
;         BL = 1 - Create new file
; On exit:
;         carry clear if file open - operation successful
;           AX = 16-bit file handle
;         carry set if operation failed
;           AX = error code as follows:
;               1 = invalid function
;               2 = file not found
;
; Note: This procedure assumes that DOS service number 26
;       has been previously used to set the DTA address
;
;*****************|
; buffered input  |
; from keyboard   |
;*****************|
	PUSH	BX		; Save exit/create switch
	MOV	AH,10		; DOS service request number
	PUSH	DX		; Save buffer start address
	PUSH	DX		; twice
	INT	21H		; To DOS
; Filename now in buffer. Set a null byte in buffer to
; create an ASCIIZ string for the PATH or FILENAME
	POP	DI		; Recover buffer start
	INC	DI		; To input count byte
	MOV	AL,[DI]		; No. of characters in buffer
	MOV	AH,0		; Prepare for addition
	ADD	DI,AX		; DX --> CR byte
	INC	DI		; One more
	MOV	BYTE PTR [DI],0H
				; Set null byte terminator
; Recover buffer address and exit/create switch
	POP	DX		; Buffer start
	ADD	DX,2		; Index to path or filename
	POP	BX		; Open/create switch
	CMP	BL,0		; Test for open switch
	JE	OPEN_F
	JMP	CREATE_F
;******************|
;      open        |
;******************|
OPEN_F:
; This routine opens a file for read/write access if the filename
; is contained in the form of an ASCIIZ string pointed by DX
	MOV	AH,61		; DOS service request number
				; to open file (handle mode)
	MOV	AL,2		; Read/write access
	INT	21H
; Carry set if open failed
; If carry clear, file is open and AX = file handle
; BL (in stack) holds exit/create switch
	RET
;******************|
;     create       |
;******************|
CREATE_F:
	MOV	CX,0		; Normal access 
	MOV	AH,60		; DOS service request
	INT	21H
; If carry clear, AX = handle for new file
; If carry set, create function failed
	RET
OPEN_CREATE	ENDP
;*******************************************************************
;
CLOSE_FILE	PROC	FAR
; Close file using file handle
; On entry:
;         BX = file handle
; On exit:
;         carry clear if operation successful - file closed
;         carry set if operation failed - invalid handle or file not
;         open
;
	MOV	AH,62		; DOS service request
	INT	21H
	RET
CLOSE_FILE	ENDP
;*******************************************************************
;
READ_128	PROC	FAR
; Read 128 bytes from an open file into buffer using the file
; handle. This procedure assumes that the file has been previously
; opened or created using the procedure OPEN_CREATE
;
; On entry:
;         BX =   file handle
;         DX --> 128 bytes user buffer
; On exit:
;         carry clear if operation successful
;           AX = number of bytes read into buffer
;           AX = 0 if end of file
;         carry set if operation failed
;           AX = error code
;                5 = access denied
;                6 = invalid handle or file not open
;
	PUSH	CX		; Save entry CX
	MOV	AH,63		; DOS service request
	MOV	CX,128		; No. of bytes to read
	INT	21H
	POP	CX		; Restore
	RET
READ_128	ENDP
;*******************************************************************
;
WRITE_128	PROC	FAR
; Transfer 128 bytes from buffer into disk file using file handle
; This procedure assumes that the file has been previously open or
; created
; On entry:
;         BX =   file handle
;         DX --> 128 bytes buffer area
;
; On exit:
;        carry clear if operation successful
;          AX = number of bytes written into buffer
;          AX = 0 if disk full
;        carry set if operation failed
;          AX = error code
;               5 = access denied
;               6 = invalid handle or file not open
;
	PUSH	CX		; Save entry CX
	MOV	AH,64		; DOS service request
	MOV	CX,128		; No. of bytes to write
	INT	21H
	POP	CX		; Restore CX
	RET
WRITE_128	ENDP
;*******************************************************************
;
OPEN_FILE	PROC	FAR
; Open file using an ASCIIZ string for the filename
; On entry:
;         DX --> buffer containing ASCIIZ string for filename
; On exit:
;         if carry clear file was opened successfully
;         AX = file handle
;         if carry set open operation failed
;         AX = error code
;              1 = invalid function
;              2 = file not found
;              3 = path not found
;              4 = no available handle
;              5 = access denied
;             12 = invalid access code
;
	MOV	AH,61		; DOS service request number
				; to open file (handle mode)
	MOV	AL,2		; Read/write access
	INT	21H
	RET
OPEN_FILE	ENDP
;
P_CODE	ENDS
	END
umber
				;