; Module name: XGA1.ASM                       Library:   GRAPHSOL.LIB
; Created: June, 1992
;********************************************************************
;********************************************************************
;                              XGA1.ASM
;********************************************************************
;********************************************************************
; Description:
; Library module for XGA video functions using the Adapter Interface
;
;    1. Open AI for use by software
;    2. Close AI and video system
;    3. Issue an AI command
;    4. Read an XGA or 8514/A font file into RAM
;    5. Set foreground and background colors for AI services
;    6. Clear screen using an AI service
;    7. Display text using an AI service
;    8. Initialize 256 DAC registers from a caller's color table
;
; Note: The library routines that are contained in this module can
;       be used from the high level languages C, Pascal, and BASIC
;       as well as in programs coded directly in Assembly Language.
;       Appendix B explains interfacing the library routines with
;       these languages.
;*******************************************************************
;                          ROUTINES
;*******************************************************************
; Public procedures in this module:
; NAME             DESCRIPTION
; OPEN_AI          Initialize AI software
; CLOSE_AI         Close AI and disable video system
; AI_COMMAND       Issue an AI command 
; AI_FONT          Read and XGA or 8514/A font file into RAM
; AI_COLOR         Set foreground and background colors
; AI_CLS           Clear screen
; AI_TEXT          Display a text message
; AI_PALETTE       Initialize 256 DAC registers
;*******************************************************************
;*******************|
;      PUBLICS      |
;*******************|
PUBLIC  OPEN_AI
PUBLIC  CLOSE_AI
PUBLIC  AI_COMMAND
PUBLIC  AI_FONT
PUBLIC  AI_COLOR
PUBLIC  AI_CLS
PUBLIC  AI_TEXT
PUBLIC  AI_PALETTE
;
;**********************|
;     EXTERNALS        |
;**********************|
EXTRN   OPEN_FILE:      FAR
EXTRN   READ_128:       FAR
EXTRN   CLOSE_FILE:     FAR
EXTRN   SET_DTA:        FAR
;
INCLUDELIB  SOLUTION
;
;*****************************************************************
;            externals in other libraries or modules
;*****************************************************************
; In SOLUTION.LIB
;
INCLUDELIB SOLUTION
;*****************************************************************
;*****************************************************************
;                segment for task state data
;****************************************************************
;*****************************************************************
TASK    SEGMENT
;**********************|
;  AI state buffer     |
;**********************|
STATE_BUF       DB      360 DUP (00H)
;
TASK    ENDS
;
;*****************************************************************
;*****************************************************************
;                      segment for XGA data
;*****************************************************************
;*****************************************************************
XGA_DATA        SEGMENT
;**********************|
;   AI list address    |
;**********************|
AI_ADD          DD      0       ; Doubleword storage for address
                                ; of Adapter Interface services
;
;
HOPEN_DATA      DW      3       ; Length of data block
INIT_FLAGS      DB      01000000B
                                ; 7 6 5 4 3 2 1 0 <= flags
                                ; | | ___________
                                ; | |      |_______ Reserved
                                ; | |_ Don't load default palette
                                ; |___ Don't clear bit planes
AF_MODE         DB      0       ; Advanced function mode
                                ; No.   Pixels     Text
                                ; 00    1024x768   85x38
                                ; 01    640x480    80x34
                                ; 02    1024x768   128x54
                                ; 03    1024x768   146x51
RET_FLAGS       DB      0       ; Status
                                ; 0 if initialization successful
                                ; Not 0 if initialization failed
;
HINIT_DATA      DW      2       ; Length of data block
BUF_SEG         DW      0       ; Segment of AI buffer
;
HEGS_DATA       DW      0       ; Length field is zero for HEGS
HCLOSE_DATA     DW      0       ; Also for HCLOSE
;
; Function to load palette
HLDPAL_DATA     DW      10      ; Length of data block
LOAD_CODE       DB      0       ; Code to load user palette
                                ; 1 = load default palette
                DB      0       ; Reserved
                DW      0       ; Number of first entry
                DW      256     ; Number of entries to load
PAL_OFF         DW      0       ; Offset of user palette
PAL_SEG         DW      0       ; Segment of user palette
                DW      0       ; Padding
;
;**********************|
;      fonts           |
;**********************|
; ASCIIZ filename for XGA 85-by-38 font
FONT_NAME       DB      '            ',00H
FONT_HANDLE     DW      0       ; Handle for font file
;
;**********************|
;   storage for font   |
;**********************|
; Font header area
FONT_BUF        DW      0       ; Number of code pages
                DW      0       ; Default code page (0 to 4)
                DW      0       ; Alternate default (0 to 4)
                DD      0       ; 4-byte ID string ('437',0)
PAGE_1_OFFSET   DW      0       ; Offset of CSD within file
                DD      0       ; 4-byte ID string ('850',0)
                DW      0       ; Offset of CSD within file
                DD      0       ; 4-byte ID string ('860',0)
                DW      0       ; Offset of CSD within file
                DD      0       ; 4-byte ID string ('863',0)
                DW      0       ; Offset of CSD within file
                DD      0       ; 4-byte ID string ('865',0)
                DW      0       ; Offset of CSD within file
;
; Character set definition block for first code page
                DB      0       ; Reserved
                DB      0       ; Font type:
                                ; 0 = multi-plane image
                                ; 3 = short vector font
                DB      0       ; Reserved
                DD      0       ; Reserved
CELL_WIDTH      DB      0       ; Pixel width of character cell
CELL_HEIGHT     DB      0       ; Pixel height of cell
                DB      0       ; Reserved
                DW      0       ; Cell size
CSD_FLAGS       DW      0       ; Flag bits:
                                ; Bit 14 ... 0 = single plane
                                ;            1 = multi-plane
                                ;     13 ... 0 = not prop. space
                                ;            1 = prop. space
IDX_TABLE_O     DW      0       ; Offset of index table
IDX_TABLE_S     DW      0       ; Segment of index table
                DW      0       ; Offset of envelope table
                DW      0       ; Segment of envelope table
                DB      0       ; Initial code point
                DB      0       ; Final code point
CSD_TABLE_O     DW      0       ; Offset of character definition
CSD_TABLE_S     DW      0       ; Segment of character definition
                DB      14250 DUP (00H)
DATA_BUF        DB      129 DUP (00H)   ; Storage for disk data
;**********************|
; text operations data |
;**********************|
; Select character set
HSCS_DATA       DW      4       ; Length of data block
FONT_OFF        DW      0       ; Offset of loaded font
FONT_SEG        DW      0       ; Segment of loaded font
; String at give position
HCHST_DATA      DW      0       ; Variable holding length of
                                ; string + 4
TEXT_X          DW      0       ; x coordinate for display
TEXT_Y          DW      0       ; y coordinate
TEXT_STRING     DB      120 DUP (20H)   ; Buffer for text string
;
;**********************|
;       colors         |
;**********************|
; Foreground color
HSCOL_DATA      DW      4       ; Length of data block
FORE_COL        DB      0F0H    ; 8-bit color code
                DB      0       ; Padding for double word
                DW      0
; Background color
HSBCOL_DATA     DW      4       ; Length of data block
BACK_COL        DB      11110000B       ; Bright red in 2-bit
                                        ; IRGB format
                DB      0       ; Padding for double word
                DW      0
;**********************|
;    drawing data      |
;**********************|
; Data block for rectangle draw
HRECT_DATA      DW      8       ; Length of data block
RECT_X          DW      0       ; x coordinate of top-left corner
RECT_Y          DW      0       ; y coordinate of top-left corner
RECT_WIDTH      DW      0       ; Width (1 to 1024)
RECT_HIGH       DW      0       ; Height (1 to 768)
;
XGA_DATA        ENDS
;
;*****************************************************************
;*****************************************************************
;                          procedures
;*****************************************************************
;*****************************************************************
P_CODE  SEGMENT PUBLIC
        ASSUME  CS:P_CODE
.386
;*****************************************************************
OPEN_AI         PROC    FAR
; Initialize Adapter Interface software
; On exit:
;        carry clear if AI initialized
;        carry set if error
;**********************|
;   save caller's DS   |
;  set DS to XGA_DATA  |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DATA             ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DATA
;**********************|
;     get AI address   |
;**********************|
; The local procedure AI_VECTOR obtains the segment:offset
; address of the Adapter Interface handler
        CALL    AI_VECTOR       ; Local procedure
        JNC     OK_VECTOR       ; Go if no carry
; Error exit
AI_ERROR:
        POP     DS              ; Restore caller's DS
        STC                     ; Error flag
        RET
OK_VECTOR:
; Store segment and offset of AI handler in memory variables
        MOV     WORD PTR AI_ADD,DX      ; Store offset of address
        MOV     WORD PTR AI_ADD+2,CX    ; and segment
; Make sure previous functions colors the adapter
; HCLOSE (close adapter)
        MOV     AX,34           ; Code number for this service
        LEA     BX,HCLOSE_DATA  ; Pointer to dummy data block
        CALL    AI_SERVICE
;**********************|
; Open and initialize  |
;         AI           |
;**********************|
; Call HOPEN service (enable adapter)
        MOV     INIT_FLAGS,0    ; Set initialization flags
                                ; to clear memory and load default
                                ; palette
        MOV     AF_MODE,0       ; Set 1024x768 mode number 0
        MOV     AX,8            ; Code number for this service
        LEA     BX,HOPEN_DATA   ; Pointer to data block
        CALL    AI_SERVICE
;
        MOV     AL,RET_FLAGS    ; Not zero if open error
        CMP     AL,0            ; Test for no error
        JZ      OK_OPEN         ; Go if no error
        JMP     AI_ERROR        ; Error exit
; Call HINIT (Initialize adapter state)
OK_OPEN:
        MOV     AX,TASK         ; Segment for task state
        MOV     BUF_SEG,AX      ; Store segment of adapter state
                                ; buffer
        MOV     AX,48           ; Code number for this service
        LEA     BX,HINIT_DATA   ; Pointer to data block
        CALL    AI_SERVICE
        POP     DS              ; Restore caller's DS
        CLC                     ; No error flag
        RET
OPEN_AI         ENDP
;
;*****************************************************************
CLOSE_AI        PROC    FAR
; Erase video and close Adapter Interface
;**********************|
;   save caller's DS   |
;  set DS to XGA_DATA  |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DATA             ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DATA
; HCLOSE (close adapter)
        MOV     AX,34           ; Code number for this service
        LEA     BX,HCLOSE_DATA  ; Pointer to dummy data block
        CALL    AI_SERVICE
        POP     DS              ; Restore caller's DS
        RET
CLOSE_AI        ENDP
;
;*****************************************************************
AI_FONT         PROC    FAR
; Read an XGA or 8514-a font file into RAM to enable text display
; AI functions
; On entry:
;         DS:SI --> ASCIIZ filename for font file
;                   (must be in the current path)
;
; On exit:
;       Carry clear if font read and stored in buffer
;       Carry set if file not found or disk error
;
;**********************|
;  filename to local   |
;        data          |
;**********************|
        PUSH    DS              ; Save caller's DS
        PUSH    ES              ; Save segment
        MOV     AX,XGA_DATA             ; Local data segment
        MOV     ES,AX                   ; to DS
        MOV     BX,OFFSET ES:FONT_NAME  ; Pointer to local buffer
; DS:DX --> 12-character filename passed by caller
; ES:BX --> local buffer
        MOV     CX,12           ; Characters to copy
COPY_12:
        MOV     AL,[SI]         ; Get character
        MOV     ES:[BX],AL      ; Place in local buffer
        INC     SI              ; Bump buffer pointers
        INC     BX
        LOOP    COPY_12
        POP     ES              ; Restore entry ES
; Set DS to local data
        MOV     AX,XGA_DATA             ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DATA
;**********************|
;    disk operations   |
;**********************|
        LEA     DX,FONT_NAME    ; Pointer to filename buffer
        LEA     DI,FONT_BUF     ; Pointer to storage
; Open font file
        PUSH    DI              ; Save entry pointer
        CALL    OPEN_FILE
        POP     DI
        JNC     OK_XOPEN        ; Go if no error code
;**********************|
;    disk open error   |
;**********************|
; Open operation failed. Set carry flag and return to caller
        POP     DS              ; Restore caller's DS
        STC                     ; Signal error
        RET
;**********************|
;   read font into RAM |
;**********************|
OK_XOPEN:
        MOV     FONT_HANDLE,AX          ; Store file handle
NEW_128:
        MOV     BX,FONT_HANDLE
        LEA     DX,DATA_BUF             ; Buffer for data storage
        PUSH    DI              ; Save buffer pointer
        CALL    READ_128        ; Read sector into buffer
        POP     DI              ; Restore buffer pointer
        CMP     AX,0            ; Test for end of file
        JNE     MOVE_128        ; Go if not at end of file
;**********************|
;    end of file       |
;**********************|
        MOV     BX,FONT_HANDLE  ; Handle for font file
        CALL    CLOSE_FILE      ; Library routine
        JMP     END_OF_READ
;**********************|
;   move sector to     |
;    font buffer       |
;**********************|
; At this point DATA_BUF holds 128 bytes from disk file
; DI --> storage position in the font's buffer
MOVE_128:
        MOV     CX,128          ; Byte counter
        LEA     SI,DATA_BUF     ; Pointer to data just read
PLACE_128:
        MOV     AL,[SI]         ; Byte from DATA_BUF
        MOV     [DI],AL         ; Into font's buffer
        INC     SI              ; Bump pointers
        INC     DI
        LOOP    PLACE_128       ; Continue until all sector read
; At this point the 128 bytes newly read from the disk file are
; stored in the font's buffer
        JMP     NEW_128
;**********************|
; select character set |
;**********************|
END_OF_READ:
; The AI is informed of the address of the loaded font by means
; of the HSCS (set character set) function
        PUSH    DS              ; DS to stack
        POP     AX              ; and to AX
        MOV     FONT_SEG,AX     ; Store segment address
; The offset of the font's character set definition block is
; located at byte 10 of the font header
        LEA     SI,FONT_BUF     ; Offset of font buffer
        MOV     BX,[SI+10]      ; Get offset of first code page
        ADD     BX,SI           ; Add offset to pointer
        MOV     FONT_OFF,BX     ; Store pointer in block
;**********************|
; update font pointers |
;**********************|
; Update pointers in character set definition area by adding
; the load address of the font (in SI)
        ADD     IDX_TABLE_O,SI  ; Add to index table offset
        ADD     CSD_TABLE_O,SI  ; and to CSD table offset
; AX still holds the segment address. Store segment portion of
; address
        MOV     IDX_TABLE_S,AX  ; In index table
        MOV     CSD_TABLE_S,AX  ; In character set table
;
; Call HSCS (set character set) function
        MOV     AX,37           ; Code number for this service
        LEA     BX,HSCS_DATA    ; Pointer to data block
        CALL    AI_SERVICE
;**********************|
;     exit routine     |
;**********************|
        POP     DS              ; Restore caller's DS
        CLC                     ; No error flag
        RET
AI_FONT         ENDP
;
;*****************************************************************
AI_COLOR        PROC    FAR
; Set foreground and background colors using AI services
; On entry
;         AL = foreground color
;         AH = background color
;**********************|
;   save caller's DS   |
;  set DS to XGA_DATA  |
;**********************|
        PUSH    DS              ; DS to stack
        MOV     BX,XGA_DATA             ; Local data segment
        MOV     DS,BX                   ; to DS
        ASSUME  DS:XGA_DATA
; Set foreground and background colors in parameter blocks
        MOV     FORE_COL,AL     ; Set foreground
        MOV     BACK_COL,AH     ; Set background
; Call HSCOL (set foreground color)
        MOV     AX,7            ; Code number for this service
        LEA     BX,HSCOL_DATA   ; Pointer to data block
        CALL    AI_SERVICE
; Call HSBCOL (set background color)
        MOV     AX,10           ; Code number for this service
        LEA     BX,HSBCOL_DATA   ; Pointer to data block
        CALL    AI_SERVICE
        POP     DS              ; Restore caller's DS
        RET
AI_COLOR        ENDP
;*****************************************************************
;
AI_CLS          PROC    FAR
; Clear screen using AI service
;**********************|
;   save caller's DS   |
;  set DS to XGA_DATA  |
;**********************|
        PUSH    DS              ; DS to stack
        MOV     AX,XGA_DATA             ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DATA
; Use HEGS (erase graphics screen)
        MOV     AX,13           ; Code number for this service
        LEA     BX,HEGS_DATA    ; Pointer to dummy data block
        CALL    AI_SERVICE
        POP     DS              ; Restore caller's DS
        RET
AI_CLS          ENDP
;
;*****************************************************************
AI_TEXT         PROC    FAR
; Display a text message on XGA screen using an AI service
; On entry:
;         CX = x pixel coordinate for start of message
;         DX = y pixel coordinate for start of message
;         AL = foreground color
;         AH = background color
;         DS:SI --> to text message, terminated in a NULL byte
;**********************|
;   save caller's DS   |
;  set DS to XGA_DATA  |
;**********************|
        PUSH    ES              ; Save caller's ES segment
        PUSH    DS              ; and DS
        MOV     BX,XGA_DATA     ; Local data segment
        MOV     DS,BX           ; to DS
        ASSUME  DS:XGA_DATA
;**********************|
;     set colors       |
;**********************|
        PUSH    CX              ; Save entry registers
        PUSH    DX
        PUSH    SI
        CALL    FAR PTR AI_COLOR        ; Local procedure
        POP     SI              ; Restore registers
        POP     DX
        POP     CX
; Set ES to caller's DS
        POP     ES              ; Caller's DS
        PUSH    ES              ; Back to stack
; Set string coordinate in parameter block
; AI service uses the bottom left corner of the character cell
; Code adds the cell height so that y coordinate corresponds with
; the top left corner of the cell
        MOV     AL,CELL_HEIGHT  ; From CSD block area
        MOV     AH,0            ; Clear high byte
        ADD     DX,AX           ; Add cell height to y
        MOV     TEXT_X,CX       ; Store x coordinate
        MOV     TEXT_Y,DX       ; and y coordinate
; Move text message and count bytes
        LEA     DI,TEXT_STRING  ; Pointer to string area
        MOV     CX,0            ; Clear counter
TEXT_TRANSFER:
        MOV     AL,ES:[SI]      ; Get caller's character
        CMP     AL,0            ; Test for terminator
        JE      END_OF_TEXT     ; Go if terminator found
        MOV     [DI],AL         ; Place in parameter area
        INC     CX              ; Bump character counter
        INC     DI              ; Bump text pointers
        INC     SI
        JMP     TEXT_TRANSFER   ; Continue
END_OF_TEXT:
        ADD     CX,4            ; Add constant
        MOV     HCHST_DATA,CX   ; Place count in variable
;**********************|
;   display message    |
;**********************|
; Call HCHST (display character string at given position)
        MOV     AX,38           ; Code number for this service
        LEA     BX,HCHST_DATA   ; Pointer to data block
        CALL    AI_SERVICE
        POP     DS              ; Restore caller's segments
        POP     ES
        RET
AI_TEXT         ENDP
;****************************************************************
;
AI_PALETTE      PROC    FAR
; Initialize 256 DAC color registers from a 4-byte per color
; table using an AI service
; On entry:
;         DS:SI --> 1024 byte table of palette colors
;                   (4 bytes per color encoding)
;**********************|
;  set LUT registers   |
;**********************|
; Set address of caller's color table in HLDPAL data area
        PUSH    DS              ; caller's DS to stack
        POP     BX              ; and to BX
;**********************|
;   save caller's DS   |
;  set DS to XGA_DATA  |
;**********************|
        PUSH    DS              ; DS to stack
        MOV     AX,XGA_DATA             ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DATA
        MOV     PAL_SEG,BX      ; Store segment address
        MOV     PAL_OFF,SI      ; Store offset
        MOV     LOAD_CODE,0     ; Code for user palette
; Call HLDPAL to set palette registers
        MOV     AX,19           ; Code number for this service
        LEA     BX,HLDPAL_DATA  ; Pointer to data block
        CALL    AI_SERVICE
        POP     DS              ; Restore caller's DS
        RET
AI_PALETTE      ENDP
;
;*****************************************************************
AI_COMMAND      PROC    FAR
; Procedure to access the services in the XGA/8514-A Adapter
; Interface - far call version of AI_SERVICE
;
; On entry:
;         AX = service number
;         DS:BX = address of parameter block
;
        PUSH    DS              ; Save caller's DS
        PUSH    BP              ; Save base pointer
        MOV     BP,SP           ; Set BP to stack
; Push address of caller's parameter block
        PUSH    DS
        PUSH    BX              ; the offset
; Multiply by 4 to form offset as required by AI
        SHL     AX,1            ; AX time 2
        SHL     AX,1            ; again
        MOV     SI,AX           ; Offset to SI
;**********************|
;   save caller's DS   |
;  set DS to XGA_DATA  |
;**********************|
        MOV     AX,XGA_DATA             ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DATA
        LES     BX,AI_ADD       ; Entry block address (ES:BX)
        CALL    DWORD PTR ES:[BX][SI]    ; Call AI service
        POP     BP              ; Restore caller's BP
        POP     DS              ; Restore caller's data
        RET
AI_COMMAND      ENDP
;
;*****************************************************************
;                support procedures (NEAR type)
;*****************************************************************
AI_VECTOR       PROC    NEAR
; Procedure to obtain the address vector to the XGA/8514-A Adapter
; Interface. This procedure must be called before calls are made
; to the Adapter Interface services (by means of the AI_SERVICE
; procedure)
;
; On entry:
;          nothing
; On exit:
;          carry set if no AI installed
;          carry clear if AI present
;             CX => segment of AI link table
;             DX => offset of AI link table
;
;**********************|
;    get vector 7FH    |
;**********************|
; Use MS DOS service number 53, interrupt 21H, to obtain the
; vector for the XGA/8514-A AI interrupt (7FH)
        MOV     AH,53           ; MS DOS service number
        MOV     AL,7FH          ; AI interrupt
        INT     21H             ; MS DOS interrupt
; ES => segment of interrupt handler
; BX => offset of handler
        MOV     AX,ES           ; Segment to AX
        OR      AX,BX           ; OR segment and offset
        JNZ     OK_AI           ; Go if address not 0000:0000
;**********************|
;  ERROR - no AI       |
;**********************|
NO_AI:
        STC                     ; Error return
        RET
;**********************|
;  get AI address      |
;**********************|
; Service number 0105H, interrupt 7FH, returns the address of the
; XGA/8514-A entry point
OK_AI:
        MOV     AX,0105H        ; Service request number
        INT     7FH             ; in XGA AI interrupt
        JC      NO_AI           ; Go if error code returned
; At this point CX:DX holds the address of the XGA/8514-A entry
; point (in segment:offset form)
        CLC                     ; No error code
        RET
AI_VECTOR       ENDP
;*****************************************************************
;
AI_SERVICE      PROC    NEAR
; Procedure to access the services in the XGA/8514-A Adapter
; Interface
;
; On entry:
;         AX = service number
;         DS:BX = address of parameter block
;
        PUSH    BP              ; Save base pointer
        MOV     BP,SP           ; Set BP to stack
; Push address of caller's parameter block
        PUSH    DS
        PUSH    BX              ; the offset
; Multiply by 4 to form offset as required by AI
        SHL     AX,1            ; AX time 2
        SHL     AX,1            ; again
        MOV     SI,AX           ; Offset to SI
        LES     BX,AI_ADD       ; Entry block address (ES:BX)
        CALL    DWORD PTR ES:[BX][SI]    ; Call AI service
        POP     BP              ; Restore caller's BP
        RET
AI_SERVICE      ENDP
;
;
P_CODE  ENDS
        END
