; Module name: XGA2.ASM                       Library:   GRAPHSOL.LIB
; Created: June 30, 1992
;********************************************************************
;********************************************************************
;                              XGA2.ASM
;********************************************************************
;********************************************************************
; Description:
; Library module for XGA video operations using direct access to
; video memory , graphics coprocessor operations, and sprite
; manipulations
;
;    1. Initialization and setup
;       a. Initialize XGA and report system configuration
;       b. Set XGA mode
;       c. Initialize coprocessor
;    2. Display operations while in 1024 by 768 resolution,
;       256 colors mode (number 2)
;       a. Display pixel 
;       b. Display rectangle
;       c. RAM to video pixBlt in 1 bit-per-pixel format
;       d. RAM to video pixBlt in 8 bit-per-pixel format
;       e. Draw Bresenham line
;       f. Clear screen
;     3. Sprite manipulations
;       a. Load sprite image into buffer
;       b. Display sprite
;       c. Erase sprite
;
; Note: The library routines that are contained in this module can
;       be used from the high level languages C, Pascal, and BASIC
;       as well as in programs coded directly in Assembly Language.
;       Appendix B explains interfacing the library routines with
;       these languages.
;*******************************************************************
;                          ROUTINES
;*******************************************************************
; Public procedures in this module:
; NAME             DESCRIPTION
; INIT_XGA         Initialize XGA for direct access
; XGA_MODE         Set XGA mode
; INIT_COP         Initialize coprocessor
; XGA_PIXEL_2      Display pixel in mode number 2
; COP_RECT_2       Display rectangle in mode number 2
; COP_SYSVID_1     RAM to video pixBlt in 1 bpp format
; COP_SYSVID_8     RAM to video pixBlt in 8 bpp format
; COP_LINE_2       Draw a line (Bresenham's algorithm)
; XGA_CLS_2        Clear screen in mode number 2
; SPRITE_IMAGE     Load sprite image into buffer
; SPRITE_AT        Display sprite at coordinates
; SPRITE_OFF       Erase sprite
;*******************************************************************
;*******************|
;      PUBLICS      |
;*******************|
PUBLIC  INIT_XGA
PUBLIC  XGA_MODE
PUBLIC  XGA_PIXEL_2
PUBLIC  INIT_COP
PUBLIC  COP_RECT_2
PUBLIC  COP_SYSVID_1
PUBLIC  COP_SYSVID_8
PUBLIC  COP_LINE_2
PUBLIC  XGA_CLS_2
PUBLIC  SPRITE_IMAGE
PUBLIC  SPRITE_AT
PUBLIC  SPRITE_OFF
;
;*****************************************************************
;*****************************************************************
;                      segment for XGA data
;*****************************************************************
;*****************************************************************
XGA_DIRECT      SEGMENT
;
XGA_POS         DW      0       ; Base address of POS registers
; Contents of POS registers
POS_2           DB      0       ; POS register at offset 2
POS_4           DB      0       ; POS register at offset 4
XGA_REG_BASE    DW      -1      ; Register base for XGA system
EQUIPMENT       DW      0       ; XGA equipment in system
XGA_CUR_MODE    DW      0       ; XGA current mode
MODE            DW      0       ; Mode number during init
;
; Variable for direct access pixel draw routine (XGA_PIXEL)
XGA_CURBK       DW      0       ; Current video memory bank
;
;**********************|
;  mode setting data   |
;**********************|
;    M O D E :      NUMBER: 
; 640x480x65536  5 ---------------------|
; 640x480x256    4 ----------------|    |
; 1024x768x16    3 -----------|    |    |
; 1024x768x256   2 ------|    |    |    |
;                        |    |    |    |
; Index ------------|    |    |    |    |
; Register-----|    |    |    |    |    |
;             _|__ _|_  _|__ _|__ _|__ _|__
XGA_VAL DB    004H,000H,000H,000H,000H,000H ; Interrupt enable
        DB    005H,000H,08FH,08FH,08FH,08FH ; Interrupt status
        DB    000H,000H,004H,004H,004H,004H ; Operating mode
        DB    00AH,064H,000H,000H,000H,000H ; Palette mask
        DB    001H,000H,001H,001H,001H,001H ; Vid mem aper cntl
        DB    008H,000H,000H,000H,000H,000H ; Vid mem aper indx
        DB    006H,000H,000H,000H,000H,000H ; Virt mem ctl
        DB    009H,000H,003H,002H,003H,004H ; Mem access mode
        DB    00AH,050H,001H,001H,001H,001H ; Disp mode 1
        DB    00AH,050H,000H,000H,000H,000H ; Disp mode 1
        DB    00AH,010H,09DH,09DH,063H,063H ; Horiz tot lo.
        DB    00AH,011H,000H,000H,000H,000H ; Horiz tot hi.
        DB    00AH,012H,07FH,07FH,04FH,04FH ; Hor disp end lo
        DB    00AH,013H,000H,000H,000H,000H ; Hor disp end hi
        DB    00AH,014H,07FH,07FH,04FH,04FH ; Hor blank start lo
        DB    00AH,015H,000H,000H,000H,000H ; Hor blank start hi
        DB    00AH,016H,09DH,09DH,063H,063H ; Hor blank end lo
        DB    00AH,017H,000H,000H,000H,000H ; Hor blank end hi
        DB    00AH,018H,087H,087H,055H,055H ; Hor sync start lo
        DB    00AH,019H,000H,000H,000H,000H ; Hor sync start hi
        DB    00AH,01AH,09CH,09CH,061H,061H ; Hor sync end lo
        DB    00AH,01BH,000H,000H,000H,000H ; Hor sync end hi
        DB    00AH,01CH,040H,040H,000H,000H ; Hor sync pos
        DB    00AH,01EH,004H,004H,000H,000H ; Hor sync pos
        DB    00AH,020H,030H,030H,00CH,00CH ; Vert tot lo
        DB    00AH,021H,003H,003H,002H,002H ; Vert tot hi
        DB    00AH,022H,0FFH,0FFH,0DFH,0DFH ; Vert disp end lo
        DB    00AH,023H,002H,002H,001H,001H ; Vert disp end hi
        DB    00AH,024H,0FFH,0FFH,0DFH,0DFH ; Vert blank start lo
        DB    00AH,025H,002H,002H,001H,001H ; Vert blank start hi
        DB    00AH,026H,030H,030H,00CH,00CH ; Vert blank end lo
        DB    00AH,027H,003H,003H,002H,002H ; Vert blank end hi
        DB    00AH,028H,000H,000H,0EAH,0EAH ; Vert sync start lo
        DB    00AH,029H,003H,003H,001H,001H ; Vert sync start hi
        DB    00AH,02AH,008H,008H,0ECH,0ECH ; Vert sync end
        DB    00AH,02CH,0FFH,0FFH,0FFH,0FFH ; Vert line comp lo
        DB    00AH,02DH,0FFH,0FFH,0FFH,0FFH ; Vert line comp hi
        DB    00AH,036H,000H,000H,000H,000H ; Sprite cntl
        DB    00AH,040H,000H,000H,000H,000H ; Start addr lo
        DB    00AH,041H,000H,000H,000H,000H ; Start addr me
        DB    00AH,042H,000H,000H,000H,000H ; Start addr hi
        DB    00AH,043H,080H,040H,050H,0A0H ; Pixel map width lo
        DB    00AH,044H,000H,000H,000H,000H ; Pixel map width hi
        DB    00AH,054H,00DH,00DH,000H,000H ; Clock sel
        DB    00AH,051H,003H,002H,003H,004H ; Display mode 2
        DB    00AH,070H,000H,000H,000H,000H ; Ext clock sel
        DB    00AH,050H,00FH,00FH,0C7H,0C7H ; Display mode 1
        DB    0FFH,0FFH,0FFH,0FFH,0FFH,0FFH ; End of the list
        DB    00AH,055H,000H,000H,000H,000H ; Border color
        DB    00AH,060H,000H,000H,000H,000H ; Sprite pal lo
        DB    00AH,061H,000H,000H,000H,000H ; Sprite pal hi
        DB    00AH,062H,000H,000H,000H,000H ; Sprite pre lo
        DB    00AH,063H,000H,000H,000H,000H ; Sprite pre hi
        DB    00AH,064H,0FFH,0FFH,0FFH,0FFH ; Palette mask
        DB    0FFH,0FFH,0FFH,0FFH,0FFH,0FFH ; End of the list
;
;
XGA_DIRECT      ENDS
;
;*****************************************************************
;*****************************************************************
;                          procedures
;*****************************************************************
;*****************************************************************
P_CODE  SEGMENT PUBLIC
        ASSUME  CS:P_CODE
.386
XGA_MODE        PROC     FAR
; Procedure to initialize an XGA graphics mode by setting the
; video system registers directly
; Note: This procedure is not required if the XGA system is
; initialized by means of the AI HOPEN and HINIT functions
; On entry:
;         AL = mode number (valid range is 2 to 8)
; On exit:
;        carry clear if no error
;
;**********************|
;   save caller's DS   |
; set DS to XGA_DIRECT |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DIRECT           ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DIRECT
;
        MOV     MODE,AX         ; Mode to variable
        CMP     MODE,9          ; Mode number out of range?
        JB      TEST_MODE1      ; Go if less than 9
        JMP     BAD_MODE        ; illegal entry value for mode
; Mode 0 = VGA BIOS mode number 3
; Mode 1 = 132 column VGA text mode
; These modes are not valid
TEST_MODE1:
        CMP     MODE,1          ; 80-col vga text mode?
        JA      VALID_MODE      ; Go if range is > 1
        JMP     BAD_MODE        ; Error exit for invalid mode
;**********************|
;  graphics mode setup |
;**********************|
VALID_MODE:
; First enable VGA graphics
        MOV     DX,03C3H        ; VGA Video Enable register
        MOV     AL,1            ; Value to enable video
        OUT     DX,AL           ; Output to port
; The table al XGA_VAL contains the values to be sent to the
; XGA register in order to initialize the corresponding mode
        LEA     SI,XGA_VAL      ; Point to start of values table
        MOV     BX,MODE         ; Use mode as an offset
;**********************|
;    initialize XGA    |
;       registers      |
;**********************|
; The value at offset 0 of XGA_VAL is the register number
; The value at offset 1 is the index register number if the
; register is 0AH. The remaining entries is register data for
; each mode
REG_DATA:
        MOV     DX,XGA_REG_BASE ; XGA register base
        MOV     AH,0            ; High byte of offset is 0
        MOV     AL,[SI]         ; Low byte of offset
; Register value 0FFH marks the end of the table
        CMP     AL,0FFH         ; End of the table?
        JE      SETUP_END       ; End of register setup
        ADD     DX,AX           ; Add register offset to base
        CMP     AL,0AH          ; Test for an index register
        JE      INDEXED         ; Go if index register
; At this point register is not at offset 0AH, therefore data
; is output directly
        MOV     AL,[SI+BX]      ; Get data value from table
        OUT     DX,AL           ; and send to port
        JMP     SHORT NEXT_REG  ; Continue
INDEXED:
        MOV     AL,[SI+1]       ; Get index register number
        MOV     AH,[SI+BX]      ; Get data byte from table
        OUT     DX,AX           ; Output data to index register
NEXT_REG:
        ADD     SI,9             ; Index to next register in table
        JMP     REG_DATA
;
SETUP_END:
        MOV     AX,MODE           ; Remember the mode we're in
        MOV     XGA_CUR_MODE,AX
        MOV     AX,1              ; Return ok
        POP     DS               ; Restore caller;'s DS
        RET
BAD_MODE:
        MOV     AX,0              ; Return failure
        POP     DS               ; Restore caller;'s DS
        RET
XGA_MODE        ENDP
;*****************************************************************
;
INIT_XGA       PROC     FAR
; Initialize XGA registers and report machine setup
; On exit:
;        AL = machine setup
; AL bits 7 6 5 4 3 2 1 0
;         | | | | | | | |___ 1 = XGA in system
;         | | | | | | |      0 = no XGA found
;         | | | | | | |_____ 1 = XGA color monitor
;         | | | | | |        0 = XGA monochrome monitor
;         | | | | | |_______ 1 = high resolution (1024 x 768)
;         | | | | |          0 = no high resolution
;         | | | | |_________ 1 = RAM = 1Mb
;         | | | |            0 = RAM = 512Kb
;         | | | |___________ 1 = dual monitor system
;         | | |              0 = single monitor system
;         |_|_|_____________ UNUSED
;
;        BX = XGA register base address
;
;**********************|
;   save caller's DS   |
; set DS to XGA_DIRECT |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DIRECT           ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DIRECT
;**********************|
;   get POS address    |
;**********************|
; Use service number 196, INT 15H, with AL = 0 to determine base
; address of Programmable Option Select (POS) registers
        MOV     AX,0C400H       ; AH = C4H (service request)
                                ; AL = 0 (sub-service)
        INT     15H             ; BIOS interrupt
                                ; for microchannel machines only
        JNC     VALID_POS       ; Go if POS address returned
        JMP     NO_XGA          ; Error - not microchannel
VALID_POS:
        MOV     XGA_POS,DX   ; Save base address of POS
; An XGA system can be located on the motherboard or in one 
; of 9 possible slots. Initialize CX = 0 for motherboard XGA
; CX = 1 to 9 for XGA in adapter card
        XOR     CX,CX           ; Start with motherboard
        CLI                     ; Interrupts off
; Use BIOS service 196, sub-service number 1, to enable slot
; for setup
GET_POS_0:
        MOV     AH,0C4H         ; BIOS service
        MOV     AL,01H          ; Sub-service number
        MOV     BX,CX           ; Slot number to BX
        INT     15H
; Slot enabled for setup
        MOV     DX,XGA_POS   ; POS register 0 and 1
        IN      AX,DX           ; Read ID low and high bytes
; Valid range for XGA systems is 8FDAH to 8FDBH
        CMP    AX,08FD8H        ; Test low limit
        JAE    TEST_HIGH_LIM    ; Go if equal or greater
; At this point the POS reports that system is not and XGA adapter
NOT_XGA_POS:
        INC     CX              ; CX is options counter
        CMP     CX,9            ; Done all slots?
        JB      GET_POS_0       ; Go if not at last slot
        JMP     NO_XGA          ; No XGA exit
TEST_HIGH_LIM:
        CMP     AX,08FDBH       ; Test high limit of range
        JA      NOT_XGA_POS     ; Go if out of range
;**********************|
;     XGA found        |
;**********************|
        CLI                      ; Disable interrupts
; Test if XGA is in motherboard
        CMP    CX,0              ; 0 is motherboard value
        JNE    XGA_CARD          ; Go if not on the motherboard
;**********************|
;  motherboard XGA     |
;**********************|
; Port 94H is used to enable and disable motherboard video
        MOV    AL,0DFH          ; Bit 5 = 0 for video setup
        MOV    DX,94H           ; 94H is system board enable
        OUT    DX,AL
        JMP    SHORT GET_POS    ; Skip slot setup
;**********************|
;       XGA card       |
;**********************|
; Use service number 196, sub-service number 1 (C401H) to
; place adapter in setup mode
XGA_CARD:
        MOV    AX,0C401H        ; Place adapter in setup mode
        MOV    BX,CX            ; Slot number to BL
        INT    15H
;**********************|
;  save POS registers  |
;**********************|
GET_POS:
        MOV     DX,XGA_POS      ; Get pos record for the slot id
        ADD     DX,2            ; POS register at offset 2
        IN      AL,DX           ; Read data byte
        MOV     POS_2,AL        ; and store it
        INC     DX              ; Next POS register
        INC     DX              ; is number 4
        IN      AL,DX           ; Get contents
        MOV     POS_4,AL        ; Store it
;**********************|
;   re-enable video    |
;**********************|
; Test for XGA in motherboard
        CMP    CX,0             ; Treat the motherboard differently
        JNE    XGA_ADAPTER      ; Go if not in motherboard
; XGA in motherboard. Set bit 5 in port 94H to re-enable video
        MOV    AL,0FFH           ; All bits set
        OUT    094H,AL
        JMP    SHORT REG_BASE
XGA_ADAPTER:
        MOV    AX,0C402H         ; Enable the MCS slot for normal
        MOV    BX,CX             ; this slot
        INT    15H
;**********************|
; calculate and store  |
; XGA register base    |
;**********************|
REG_BASE:
        STI                     ; Interrupts on again
        MOV    AL,POS_2         ; Get value at POS register 2
        AND    AX,0EH           ; Mask out all bits except
                                ; instance
        SHL    AX,1             ; Multiply instance by 8
        SHL    AX,1
        SHL    AX,1
        ADD    AX,2100H         ; Add instance to base address
        MOV    XGA_REG_BASE,AX  ; Store result in variable
;**********************|
;   get monitor ID     |
;**********************|
; Bits 0 to 3 of Display ID and Comparator register (offset 52H)
; encode the display type attached to the system
        MOV     DX,XGA_REG_BASE ; Base address
        ADD     DX,0AH          ; To index register
        MOV     AL,052H         ; Select Display ID and Comparator
                                ; register
        OUT     DX,AL
; Read byte at selected register
        MOV     DX,XGA_REG_BASE
        ADD     DX,0BH           ; To data port
        IN      AL,DX            ; Read register data
        AND     AL,0FH           ; Mask off high nibble
        CMP     AL,00H           ; Test for illegal value
        JNE     ID_OK           ; Go if valid display ID
; Illegal ID code
        JMP     NO_XGA
;**********************|
; set monitor ID bit   |
;**********************|
ID_OK:
; Monitor ID is reported as follows:
; bit code    color/BW  max. address    Model
; xxxx 1101 = color     640 by 480      8503
; xxxx 1110 = color     640 by 480      8512/8513
; xxxx 1011 = color     1024 by 768     8515
; xxxx 1001 = BW        1024 by 768     8504/8507
; xxxx 1010 = color     1024 by 768     8514
; xxxx 0010 = color     1024 by 768     8514
; xxxx 1111 = no display installed
        AND     AL,00000110B     ; Preserve bits 1 and 2
        XOR     AL,00000101B     ; Reverse bits 0 and 2
        MOV     EQUIPMENT,AX  ; Save the result flag
;**********************|
;    set monitor bit   |
;**********************|
; Read bit 0 of the Operating Mode register (offset 0) to
; determine if VGA mode address decoding is enabled
        MOV     DX,XGA_REG_BASE      ; Operating Mode register
        IN      AL,DX           ; Read data byte
        TEST    AL,1            ; Test low bit
        JNZ     INT_CONTROL     ; Go single monitor in system
        OR      EQUIPMENT,00010000B      ; Set bit 5 to indicate
                                        ; 2 monitors
;**********************|
;  disable XGA system  |
;      interrupts      |
;**********************|
INT_CONTROL:
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,4            ; Select Interrupt Enable register
        XOR     AL,AL           ; All interrupts OFF
        OUT     DX,AL
;**********************|
; set extended graphics|
;         mode         |
;**********************|
; A value of 100B in bit 0 to 2 of the Operating Mode register
; selects the extended graphics mode
        MOV     DX,XGA_REG_BASE ; Register base
; Switch to extended mode
        MOV     AL,00000100B     ; Bit map is 100B
        OUT     DX,AL
;**********************|
;  select 64K aperture |
;     at A0000H        |
;**********************|
; Operating systems and real mode applications access XGA video
; memory by means of a 64K aperture. Since in 1024 by 764 mode
; there are 786,432 pixels, 12 banks of 64K are required. The
; active bank is selected by means of the Aperture Index register
; at address 21x8H
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,01H          ; Select Aperture Control register
        MOV     AL,1            ; Value 01B selects 64K at A0000H
        OUT     DX,AL           ; Write data to register
;**********************|
; clear palette mask   |
;**********************|
; The contents of the Palette Mask register (offset 64H) is ANDed
; with the screen pixels at display time. Clearing all bits disables
; the display
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
        MOV     AX,0064H        ; 00 to register at offset 64H
        OUT     DX,AX           ; Write data
; Display is now disabled by means of the Palette Mask
;**********************|
; select video page 12 |
;**********************|
        MOV     XGA_CUR_MODE,2  ; Force mode 2 (1024 by 768 in
                                ; 256 colors)
        MOV     AL,12           ; Select bank 12
        CALL    XGA_NEWBANK     ; Local select bank procedure
;**********************|
;  test memory size    |
;**********************|
; Memory size is determined by writing and then reading data at
; at bank 12 address. If the data stored is recovered, then the
; XGA system contains 1Kb of VRAM. Otherwise the system contains
; 512K
        PUSH    ES              ; Save ES segment
        MOV     AX,0A000H       ; AX = base address of video RAM
        MOV     ES,AX           ; VRAM base to ES
        MOV     AL,0A5H         ; Any value to AL
        MOV     ES:[0000H],AL   ; Store AL in VRAM
        JMP     SHORT $ + 2     ; Delay
        MOV     AH,ES:[0000H]   ; Read VRAM byte
        CMP     AH,AL           ; Compare values
        JNE     XGA_512         ; Go if value not recovered
; 1Mb VRAM in system
        OR      EQUIPMENT,00001000B     ; Set bit 3
XGA_512:
        POP    ES               ; Restore ES segment
        MOV    AL,0             ; Select bank 0
        CALL   XGA_NEWBANK      ; Restore bank 0 as active
        MOV    XGA_CUR_MODE,0   ; and current mode variable
;**********************|
;  select access mode  |
;**********************|
; Select Intel order and 8 bits per pixel in the Memory Access
; Mode register (offset + 9)
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,9            ; To Mode register
        MOV     AL,03H          ; 7 6 5 4 3 2 1 0  <= bit map
                                ; | | | | | | | |   Bits/pixel
                                ; | | | | | |_|_|__ 000 = 1 bit
                                ; | | | | |         001 = 2 bits
                                ; | | | | |         010 = 4 bits
                                ; | | | | |        *011 = 8 bits
                                ; | | | | |         100 = 16 bits
                                ; | | | | |___ *0 = Intel format
                                ; | | | |       1 = Motorola format
                                ; |_|_|_|____ RESERVED
                                ; 03H = 00000011B
        OUT     DX,AL
;**********************|
;  enable palette mask |
;**********************|
; Re-enable video by setting all bits in the Palette Mask register
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
        MOV     AX,0FF64H       ; FFH to Palette Mask register
        OUT     DX,AX
;**********************|
;  restore VGA signal  |
;**********************|
; In single-monitor systems the VGA signal must be restored on exit
        TEST    EQUIPMENT,10H   ; Test monitor bit for dual monitor
                                ; system
        JNZ     XGA_EXIT        ; Go if a dual-monitor system
; VGA signal is restored by writing 001B to the XGA Operating Mode
; register and a value of 1 to the VGA Video System Enable register
        MOV    DX,XGA_REG_BASE  ; Register base
        MOV    AL,1             ; Bit map is 001B
        OUT    DX,AL            ; Write to Operating Mode register
        MOV    DX,03C3H         ; VGA Video System Enable register
        MOV    AL,1             ; Value is 00000001B
        OUT    DX,AL            ; Write to register
        JMP    SHORT XGA_EXIT
;**********************|
;      ERROR exit      |
;**********************|
NO_XGA:
        MOV    XGA_REG_BASE,0   ; Set invalid value in variable
        STC                     ; Error flag
        POP     DS              ; Restore caller's DS
        RET
;
XGA_EXIT:
        MOV     AX,EQUIPMENT    ; Return the result
        MOV     BX,XGA_REG_BASE ; Return register base address
        CLC                     ; No error
        POP     DS              ; Restore caller's DS
        RET
INIT_XGA       ENDP
;
;******************************************************************
XGA_PIXEL_2    PROC  FAR
; Write a screen pixel accessing XGA memory directly
; On entry:
;       CX = x coordinate of pixel
;       DX = y coordinate of pixel
;       BL = pixel color in 8-bit format
; Note: code assumes that XGA is in a 1024 by 768 pixel mode
;       in 256 colors (mode number 2)

;**********************|
;   save caller's DS   |
; set DS to XGA_DIRECT |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DIRECT           ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DIRECT
; Set ES to video buffer base address
        MOV     AX,0A000H       ; Base for all graphics modes
        MOV     ES,AX           ; To ES segment
        MOV     AL,BL           ; Color to AL
; Get address in XGA system
        CLC                     ; Clear carry flag
        PUSH    AX              ; Save color value
        MOV     AX,1024         ; This many dots / line
        MUL     DX              ; DX holds line count of address
        ADD     AX,CX           ; Plus this many x-dots
        ADC     DX,0            ; Answer in DX:AX
                                ; DL = bank, AX = offset
        MOV     BX,AX           ; Save this in bx
        MOV     AX,DX           ; Move bank number to AL
;**********************|
;    change banks      |
;**********************|
        MOV    DX,XGA_REG_BASE   ; XGA base register address
        ADD    DX,08H            ; Aperture index register
        OUT    DX,AL             ; Bank number is in AL
        POP     AX              ; Restore color value
;**********************|
;   set the pixel      |
;**********************|
        MOV     ES:[BX],AL      ; Write the dot
        POP     DS              ; Restore caller's DS
        RET
XGA_PIXEL_2     ENDP
;
;******************************************************************
XGA_CLS_2       PROC    FAR
; Clear video memory while in mode number 2 using block move
;**********************|
;   save caller's DS   |
; set DS to XGA_DIRECT |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DIRECT           ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DIRECT
;
        MOV     AX,0A000H       ; Video memory base address
        MOV     ES,AX           ; To the ES register
        MOV     BL,0            ; BL is bank counter
; Select bank
NEXT_BANK:
        MOV     DX,XGA_REG_BASE ; Select Page
        ADD     DX,08H          ; To Aperture Index register
        MOV     AL,BL           ; Bank number
        OUT     DX,AL           ; Select bank in AL
; Write 65536 bytes of 00H in current bank
        MOV     CX,0FFFFH       ; CX is byte counter
        MOV     AX,0            ; Attribute to place in VRAM
        CLD                     ; Forward direction
        MOV     DI,0            ; Start of block
        REP     STOSB           ; Store 65536 bytes
; Bump bank
        INC     BL
        CMP     BL,12           ; 12 is last bank
        JNE     NEXT_BANK
        POP     DS              ; Restore caller's DS
        RET
XGA_CLS_2       ENDP
;
;*****************************************************************
XGA_OFF         PROC    FAR
; Turn off XGA video by clearing the Palette Mask register
;**********************|
;   save caller's DS   |
; set DS to XGA_DIRECT |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DIRECT           ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DIRECT
;
        MOV     DX,XGA_REG_BASE
        ADD     DX,0AH          ; Index register
        MOV     AX,00064H       ; Value is all zeros for OFF
        OUT     DX,AX           ; Write data
        POP     DS              ; Restore caller's DS
        RET
XGA_OFF         ENDP
;*****************************************************************
;
XGA_ON          PROC    FAR
; Turn on XGA video by setting the Palette Mask register
;**********************|
;   save caller's DS   |
; set DS to XGA_DIRECT |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DIRECT           ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DIRECT
;
        MOV     DX,XGA_REG_BASE
        ADD     DX,0AH          ; Index register
        MOV     AX,0FF64H       ; Value is all ones for ON
        OUT     DX,AX           ; Write data
        POP     DS              ; Restore caller's DS
        RET
XGA_ON          ENDP
;
;*****************************************************************
XGA_PALETTE     PROC    NEAR
; Set 256 XGA palette registers
; On entry:
;         SI --> 1024-byte color table in RGBx format
;
; Code assumes that XGA system is in set in a graphics mode
;
;**********************|
;   save caller's DS   |
; set DS to XGA_DIRECT |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DIRECT           ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DIRECT
;
; Select index register at offset 0AH
        MOV    DX,XGA_REG_BASE  ; Base address of controller
        ADD    DX,0AH
; Write 00H (in AH) to palette mask register (64H)
; This value is ANDed with display memory. Clearing all bits
; makes the palette invisible during setup
        MOV    AX,0064H          ; make invisible
        OUT    DX,AX
; Write 00H (in AH) to border color register (55H)
        MOV    AX,0055H          ; Border Color register
        OUT    DX,AX
; Write 00000100B (in AH) to palette sequence register (66H) to
; select four-color write mode (RGBx) and to start with the
; R color code
        MOV     AX,0466H          ; Palette sequence register
        OUT     DX,AX
; Write 00H (in AH) to palette index register low (60H)
; and high (61H) to select first DAC register
        MOV     AX,0060H          ; start at palette 0
        OUT     DX,AX
        MOV     AX,0061H          ; Sprite index high
        OUT     DX,AX
; SI --> table of palette colors
        MOV     CX,1024         ; Counter for 256 * 4
        MOV     AX,065H         ; Select data register
        OUT     DX,AL
        INC     DX              ; Point to first register
; Loop to send 4 blocks of 256 byte each to port 065H
NEW_PALETTE:
        MOV     AL,[SI]         ; Get byte from table
        OUT     DX,AL           ; Send to port
        INC     SI              ; Bump table pointer
        LOOP    NEW_PALETTE
;
        DEC     DX              ; Back to Select register
; Write FFH (in AH) to palette mask register (64H)
; This value is ANDed with display memory. Setting all bits 
; makes the palette visible again
        MOV     AX,0FF64H       ; All bits set
        OUT     DX,AX           ; To make visible
        POP     DS              ; Restore caller's DS
        RET
XGA_PALETTE     ENDP
;
;*****************************************************************
DC_PALETTE      PROC    FAR
; Set 256 XGA palette registers for the 65535 color mode
; Note: the values are those recommended by IBM
; Code assumes that XGA system is in set in a graphics mode
;
;**********************|
;   save caller's DS   |
; set DS to XGA_DIRECT |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DIRECT           ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DIRECT
;
; Select index register at offset 0AH
        MOV    DX,XGA_REG_BASE  ; Base address of controller
        ADD    DX,0AH           ; To index register
; Write 00H (in AH) to palette mask register (64H)
; This value is ANDed with display memory. Clearing all bits
; makes the palette invisible during setup
        MOV    AX,0064H          ; make invisible
        OUT    DX,AX
; Write 00H (in AH) to palette sequence register (66H) to enable
; three-color write mode (RGB) and to start with the
; R color code
        MOV     AX,0066H          ; Palette sequence register
        OUT     DX,AX
; Write 00H (in AH) to palette index register low (60H)
; and high (61H) to select first DAC register
        MOV     AX,0060H          ; start at palette 0
        OUT     DX,AX
        MOV     AX,0061H          ; Sprite index high
        OUT     DX,AX
;**********************|
; first 128 registers  |
;**********************|
; Write 80H (in AH) to border color register (55H) to select first
; group of 128 registers
        MOV     AX,8055H          ; Border Color bit 7 set
        OUT     DX,AX
        CALL    LOAD_128          ; Local procedure
;**********************|
; second 128 registers |
;**********************|
; Write 00H (in AH) to border color register (55H) to select the
; second group of 128 registers
        MOV     AX,0055H        ; Border Color bit 7 clear
        OUT     DX,AX
        CALL    LOAD_128        ; Local procedure
; Write FFH (in AH) to palette mask register (64H)
; This value is ANDed with display memory. Setting all bits 
; makes the palette visible again
        MOV     AX,0FF64H       ; All bits set
        OUT     DX,AX           ; To make visible
        POP     DS              ; Return caller's DS
        RET
DC_PALETTE      ENDP
;
;******************************************************************
LOAD_128        PROC    NEAR
; Auxiliary procedure for DC_PALETTE to load a group of 128
; DAC registers with the recommended values
;
        MOV     DX,XGA_REG_BASE ; Base address
        ADD     DX,0AH          ; Index register
        MOV     AX,0065H        ; Select data register
        OUT     DX,AL
        INC     DX              ; To data register
        MOV     BX,0            ; BX is value for blue register
        MOV     CX,128          ; Counter for 128 registers
; Loop to send 3 bytes to 128 registers
DC_128:
        MOV     AL,0            ; Send red
        OUT     DX,AL           ; Send to port
        JMP     SHORT $ + 2     ; I/O delay
        OUT     DX,AL           ; Send green
        MOV     AL,BL           ; Load blue value
        OUT     DX,AL           ; Send blue
        ADD     BL,8            ; Bump blue value in BL
                                ; Wraps around automatically
        LOOP    DC_128
        DEC     DX              ; Back to Index register
        RET
LOAD_128        ENDP
;
;*****************************************************************
;*****************************************************************
;                  coprocessor operations
;*****************************************************************
;*****************************************************************
;
INIT_COP        PROC    FAR
; Initialize XGA coprocessor
; Code assumes that the procedure INIT_XGA has been called and
; that the POS_x variables have been loaded
; Coprocessor base address is calculated as follows:
; ROM address = (ROM field + 2000H) + C0000H
; COP address = (((Instance * 128) + 1C00H) + ROM address)
; On exit:
;         GS = coprocessor base address
;         FS = base address of video memory
;
;**********************|
;   save caller's DS   |
; set DS to XGA_DIRECT |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DIRECT           ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DIRECT
; First calculate ROM address from data in POS register 2
        MOV     EAX,0           ; Clear EAX
        MOV     AL,POS_2        ; Get POS register 2
        AND     EAX,0F0H        ; Preserve ROM bits
        SHR     EAX,4           ; Shift ROM to low nibble
        MOV     ECX,2000H       ; Multiplier
        MUL     ECX             ; EAX * ECX in EAX
        ADD     EAX,0C0000H     ; Add constant
        MOV     EBX,EAX         ; Store ROM address in EBX
; EBX now holds ROM address
; Instance is stored in bits 1-3 of POS register 2
        MOV     EAX,0           ; Clear EAX
        MOV     AL,POS_2        ; Get POS register 2
        AND     EAX,0EH         ; Preserve Instance bits
        SHR     EAX,1           ; Shift right Instance bits
        MOV     ECX,128         ; Multiplier to ECX
        MUL     ECX
        ADD     EAX,1C00H       ; Add constant from formula
; Add ROM address
        ADD     EAX,EBX
        SHR     EAX,4           ; Shift right one nibble to
                                ; to obtain segment value
; Store segment value in GS
        MOV     GS,AX           ; Move segment into GS
;**********************|
;    get VRAM base     |
;**********************|
; Start of video memory is a 32-bit physical address determined
; as follows:
;            high-order word      low-order word
;          ___________________   ____________________
;          BBBB BBBi ii00 0000 | 0000 0000 0000 0000
;
; B (base) = are 7 high bits in POS register 4
; i (instance) = bits 1 to 3 in POS register 2
; The high order word of the video memory address must be stored
; at the coprocessor register at offset 16H
        MOV     AL,POS_4        ; Get B bits in above map
        AND     AL,11111110B    ; Clear low bit
        SHL     AX,8            ; Shift to high position
; Get instance bits
        MOV     BL,POS_2        ; Get POS register to
        AND     BL,00001110B    ; Mask out other bits
        MOV     BH,0            ; Clear high part of BX
        SHL     BX,5            ; Move instance bits to position
        OR      AX,BX           ; OR in with B bits (in AX)
        MOV     FS,AX           ; Store in FS segment
;**********************|
;  select access mode  |
;**********************|
; Select Intel order and 8 bits per pixel in the Memory Access
; Mode register (offset + 9)
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,9            ; To Mode register
        MOV     AL,03H          ; 7 6 5 4 3 2 1 0  <= bit map
                                ; | | | | | | | |   Bits/pixel
                                ; | | | | | |_|_|__ 000 = 1 bit
                                ; | | | | |         001 = 2 bits
                                ; | | | | |         010 = 4 bits
                                ; | | | | |        *011 = 8 bits
                                ; | | | | |         100 = 16 bits
                                ; | | | | |___ *0 = Intel format
                                ; | | | |       1 = Motorola format
                                ; |_|_|_|____ RESERVED
                                ; 03H = 00000011B
        OUT     DX,AL
        POP     DS              ; Restore caller's DS
        RET
INIT_COP        ENDP
;*****************************************************************
;
COP_RECT_2      PROC    FAR
; Use graphics coprocessor to perform a pixel Blit operation
; on a rectangular screen area
; Code assumes:
;       1. 1024 by 768 mode in 256 colors (mode number 2)
;       2. The INIT_COP routine has been previously called to
;          initialize the GS and FS segment registers
; On entry:
;         CX = x coordinate of top-left corner
;         DX = y coordinate of top_left corner
;         SI = width of rectangle, in pixels
;         DI = height of rectangle, in pixels
;         BL = 8-bit color code
;**********************|
;   test for not busy  |
;**********************|
        CALL    COP_RDY         ; Local routine
; At this point the coprocessor is not busy
;**********************|
;   prepare to pixBlt  |
;**********************|
        MOV     AL,01H          ; Data value for Map A
        MOV     GS:[+12H],AL    ; Write to pixel map index
        MOV     AX,0H           ; Data value for VRAM low
        MOV     GS:[+14H],AX    ; Write to pix map base address
; FS register holds the high order word of VRAM address. This
; value is calculated by the INIT_COP routine in this module
        MOV     AX,FS           ; Data for VRAM high
        MOV     GS:[+16H],AX    ; Write to pix map segment address
; Code assumes 1024 by 768 pixel mode and Intel format
        MOV     AX,1023         ; Value for pix map width
        MOV     GS:[+18H],AX    ; Write to width register
        MOV     AX,767          ; Value for pix map height
        MOV     GS:[+20H],AX    ; Write to height register
        MOV     AL,3            ; Select Intel order and 8 bits
                                ; per pixel
        MOV     GS:[+1CH],AL    ; Write to format register
;**********************|
;    perform pixBlt    |
;**********************|
        MOV     AL,03H          ; Select source mix mode
        MOV     GS:[+48H],AL    ; Write to Mix register
; Write color (in BL) to foreground register
        MOV     GS:[+58H],BL    ; Write to foreground color
                                ; register
; Write coordinates of rectangle's start point to coprocessor
; registers
        MOV     GS:[+78H],CX    ; Write to Destination x Address
                                ; register
        MOV     GS:[+7AH],DX    ; Write to Destination y Address
                                ; register
; Store width in Operations Dimension 1 register
        MOV     GS:[+60H],SI    ; Write to Operation Dimension 1
; Store height in Operations Dimension 2 register
        MOV     GS:[+62H],DI    ; Write to Operation Dimension 2
;**********************|
;  setup pix operation |
;      registers       |
;**********************|
; Bit map of Pixel Operations register for pixBlt operation:
; byte 3 = bbss|pppp (* = values for this operation)
;                     bb = background source
;                         *00 = fixed register pixBlt
;                          10 = VRAM to VRAM pixBlt
;                     ss = foreground source
;                         *00 = fixed register pixBlt
;                          10 = VRAM to VRAM pixBlt
;                     pppp = function
;                          0010 = draw and step read
;                          0011 = line draw read
;                          0100 = draw and step write
;                          0101 = line draw write
;                         *1000 = pixBlt
;                          1001 = inverting pixBlt
;                          1010 = area fill pixBlt
;                      BYTE 3 = 00001000B = 08H
; byte 2 = SSSS|DDDD (* = values for this operation)
;                     SSSS = source pixel map
;                         *0001 = pixel map A
;                          0010 = pixel map B
;                          0011 = pixel map C
;                     DDDD = destination pixel map
;                         *0001 = pixel map A
;                          0010 = pixel map B
;                          0011 = pixel map C
;                      BYTE 2 = 00010001B = 11H
; byte 1 = PPPP|0000 (* = values for this operation)
;                     PPPP = pattern pixel map
;                          0001 = pixel map A
;                          0010 = pixel map B
;                          0011 = pixel map C
;                         *1000 = foreground (fixed)
;                          1001 = generated from source
;                      BYTE 1 = 10000000B = 80H
; byte 0 = mm00|0oox (* = values for this operation)
;                     mm = mask pixel map
;                         *00 = mask map disabled
;                          01 = boundary enabled
;                          10 = mask map enabled
;                     oox = octant bits (x = don't care)
;                         *00 = start at top left and move
;                               right and down
;                          10 = start at top right and move
;                               left and down
;                          01 = start at bottom left and move
;                               right and up
;                          11 = start at bottom right and move
;                               left and up
;                      BYTE 0 = 00000000B = 00H
        MOV     EAX,08118000H   ; Value from bit map
        MOV     GS:[+7CH],EAX   ; Write to Pixel Operations
                                ; register
        RET
COP_RECT_2      ENDP
;*****************************************************************
;
COP_SYSVID_1    PROC    FAR
; Use graphics coprocessor to perform a pixel blit operation
; from a source in system memory to a destination in video memory
; Image map is encoded in 1 bit per pixel format
; Code assumes:
;       1. 1024 by 768 mode in 256 colors
;       2. The INIT_COP routine has been previously called to
;          initialize the GS and FS segment registers
;       3. Image bit map is 1 bit per pixel
; On entry:
;         DS:SI = offset of source bit map in RAM
;         CX = source map pixel width
;         DX = source map pixel height
;         SI = x coordinate of video image
;         DI = y coordinate of video image
;         BL = 8-bit color code to use in displaying image
;**********************|
;   test for not busy  |
;**********************|
        CALL    COP_RDY         ; Local routine
; At this point the coprocessor is not busy
;**********************|
; map A is destination |
;   (video memory)     |
;**********************|
        PUSH    AX              ; Bit map offset to stack
        MOV     AL,01H          ; Data value for Map A
        MOV     GS:[+12H],AL    ; Write to pixel map index
        MOV     AX,0H           ; Data value for VRAM low
        MOV     GS:[+14H],AX    ; Write to pix map base address
; FS register holds the high order word of VRAM address. This
; value is calculated by the INIT_COP routine in this module
        MOV     AX,FS           ; Data for VRAM high
        MOV     GS:[+16H],AX    ; Write to pix map segment address
; Destination map is 1024 by 768 pixel mode and Intel format
        MOV     AX,1023         ; Value for pix map width
        MOV     GS:[+18H],AX    ; Write to width register
        MOV     AX,767          ; Value for pix map height
        MOV     GS:[+20H],AX    ; Write to height register
; Bit map of pixel format register:
; 7 6 5 4 3 2 1 0 <= bits
; | | | | | |_|_|______ pixel image size (* = selected value)
; | | | | |             000 = 1 bit per pixel
; | | | | |             001 = 2 bits per pixel
; | | | | |             010 = 4 bits per pixel
; | | | | |            *011 = 8 bits per pixel
; | | | | |_______ format control
; | | | |           1 = Motorola order
; | | | |          *0 = Intel order
; |_|_|_|________________________________ RESERVED
        MOV     AL,3            ; Select Intel order and 8 bit
                                ; per pixel
        MOV     GS:[+1CH],AL    ; Write to format register
;**********************|
;    map B is source   |
;    (system memory)   |
;**********************|
        MOV     AL,02           ; Data value for Map B
        MOV     GS:[+12H],AL    ; Write to pixel map index
; AX = offset of source bit map (in stack)
; DS = segment of source bit map
; To convert logical address to physical address the segment value
; is shifted left 4 bits and the offset added
        MOV     EAX,0           ; Clear 32 bits
        MOV     AX,DS           ; Segment to AX
        SHL     EAX,4           ; Shift segment 4 bits
        POP     BP              ; Offset to BP
        ADD     AX,BP           ; Add offset to segment
        MOV     GS:[+14H],EAX   ; Write to pix map base address
; Dimensions of source map are in CX and DX registers
        DEC     CX
        DEC     DX
        MOV     GS:[+18H],CX    ; Write to width register
        MOV     GS:[+20H],DX    ; Write to height register
; Bit map of pixel format register:
; 7 6 5 4 3 2 1 0 <= bits
; | | | | | |_|_|______ pixel image size (* = selected value)
; | | | | |            *000 = 1 bit per pixel
; | | | | |             001 = 2 bits per pixel
; | | | | |             010 = 4 bits per pixel
; | | | | |             011 = 8 bits per pixel
; | | | | |_______ format control
; | | | |          *1 = Motorola order
; | | | |           0 = Intel order
; |_|_|_|________________________________ RESERVED
        MOV     AL,08H          ; Select Motorola order and 1 
                                ; bit per pixel
        MOV     GS:[+1CH],AL    ; Write to format register
;**********************|
;    select mix mode   |
;**********************|
        MOV     AL,03H          ; Select source mix mode
        MOV     GS:[+48H],AL    ; Write to Mix register
; Write color (in BL) to foreground register
        MOV     GS:[+58H],BL    ; Write to foreground color
                                ; register
; Write coordinates of source and destination
; Source coordinate are 0,0, destination coordinates are in SI
; and DI
        MOV     AX,0            ; Source coordinates
        MOV     GS:[+70H],AX    ; Write to Source x Address
        MOV     GS:[+72H],AX    ; Write to Source y Address
        MOV     GS:[+78H],SI    ; Write to Destination x Address
        MOV     GS:[+7AH],DI    ; Write to Destination y Address
; Store width in Operations Dimension 1 register
        MOV     GS:[+60H],CX    ; Write to Operation Dimension 1
; Store height in Operations Dimension 2 register
        MOV     GS:[+62H],DX    ; Write to Operation Dimension 2
;**********************|
;  setup pix operation |
;      registers       |
;**********************|
; Bit map of Pixel Operations register for pixBlt operation:
; byte 3 = bbss|pppp (* = values for this operation)
;                     bb = background source
;                         *00 = background color
;                          10 = source pixel map
;                     ss = foreground source
;                         *00 = foreground color
;                          10 = source pixel map
;                     pppp = function
;                          0010 = draw and step read
;                          0011 = line draw read
;                          0100 = draw and step write
;                          0101 = line draw write
;                         *1000 = pixBlt
;                          1001 = inverting pixBlt
;                          1010 = area fill pixBlt
;                      BYTE 3 = 00001000B = 08H
; byte 2 = SSSS|DDDD (* = values for this operation)
;                     SSSS = source pixel map
;                          0001 = pixel map A
;                         *0010 = pixel map B
;                          0011 = pixel map C
;                     DDDD = destination pixel map
;                         *0001 = pixel map A
;                          0010 = pixel map B
;                          0011 = pixel map C
;                      BYTE 2 = 00100001B = 21H
; byte 1 = PPPP|0000 (* = values for this operation)
;                     PPPP = pattern pixel map
;                          0001 = pixel map A
;                         *0010 = pixel map B
;                          0011 = pixel map C
;                          1000 = foreground (fixed)
;                          1001 = generated from source
;                      BYTE 1 = 00100000B = 20H
; byte 0 = mm00|0oox (* = values for this operation)
;                     mm = mask pixel map
;                         *00 = mask map disabled
;                          01 = boundary enabled
;                          10 = mask map enabled
;                     oox = octant bits (x = don't care)
;                         *00 = start at top left and move
;                               right and down
;                          10 = start at top right and move
;                               left and down
;                          01 = start at bottom left and move
;                               right and up
;                          11 = start at bottom right and move
;                               left and up
;                      BYTE 0 = 00000000B = 00H
        MOV     EAX,008212000H   ; Value from bit map
        MOV     GS:[+7CH],EAX   ; Write to Pixel Operations
                                ; register
        RET
COP_SYSVID_1    ENDP
;
;*****************************************************************
COP_SYSVID_8    PROC    FAR
; Use graphics coprocessor to perform a pixel blit operation
; from a source in system memory to a destination in video memory
; Image map is encoded in 8 bit per pixel format
; Code assumes:
;       1. 1024 by 768 mode in 256 colors
;       2. The INIT_COP routine has been previously called to
;          initialize the GS and FS segment registers
;       3. Image bit map is 8 bit per pixel
; On entry:
;         DS:SI = offset of source bit map in RAM
;         CX = source map pixel width
;         DX = source map pixel height
;         SI = x coordinate of video image
;         DI = y coordinate of video image
;
; Note: see bit maps in COP_SYSVID_1 procedure
;**********************|
;   test for not busy  |
;**********************|
        CALL    COP_RDY         ; Local routine
; At this point the coprocessor is not busy
;**********************|
; map A is destination |
;   (video memory)     |
;**********************|
        PUSH    AX              ; Bit map offset to stack
        MOV     AL,01H          ; Data value for Map A
        MOV     GS:[+12H],AL    ; Write to pixel map index
        MOV     AX,0H           ; Data value for VRAM low
        MOV     GS:[+14H],AX    ; Write to pix map base address
; FS register holds the high order word of VRAM address. This
; value is calculated by the INIT_COP routine in this module
        MOV     AX,FS           ; Data for VRAM high
        MOV     GS:[+16H],AX    ; Write to pix map segment address
; Destination map is 1024 by 768 pixel mode and Intel format
        MOV     AX,1023         ; Value for pix map width
        MOV     GS:[+18H],AX    ; Write to width register
        MOV     AX,767          ; Value for pix map height
        MOV     GS:[+20H],AX    ; Write to height register
        MOV     AL,3            ; Select Intel order and 8 bit
                                ; per pixel
        MOV     GS:[+1CH],AL    ; Write to format register
;**********************|
;    map B is source   |
;    (system memory)   |
;**********************|
; Note that entry location of map is by the ES segment register
        MOV     AL,02           ; Data value for Map B
        MOV     GS:[+12H],AL    ; Write to pixel map index
; AX = offset of source bit map (in stack)
; ES = segment of source bit map
; To convert logical address to physical address the segment value
; is shifted left 4 bits and the offset added
        MOV     EAX,0           ; Clear 32 bits
        MOV     AX,ES           ; Segment to AX
        SHL     EAX,4           ; Shift segment 4 bits
        POP     BP              ; Offset to BP
        ADD     AX,BP           ; Add offset to segment
        MOV     GS:[+14H],EAX   ; Write to pix map base address
; Dimensions of source map are in CX and DX registers
        DEC     CX
        DEC     DX
        MOV     GS:[+18H],CX    ; Write to width register
        MOV     GS:[+20H],DX    ; Write to height register
        MOV     AL,0BH          ; Select Motorola order and 8 
                                ; bit per pixel
        MOV     GS:[+1CH],AL    ; Write to format register
;**********************|
;    select mix mode   |
;**********************|
        MOV     AL,03H          ; Select source mix mode
        MOV     GS:[+48H],AL    ; Write to Mix register
; Write coordinates of source and destination
; Source coordinate are 0,0, destination coordinates are in SI
; and DI
        MOV     AX,0            ; Source coordinates
        MOV     GS:[+70H],AX    ; Write to Source x Address
        MOV     GS:[+72H],AX    ; Write to Source y Address
        MOV     GS:[+78H],SI    ; Write to Destination x Address
        MOV     GS:[+7AH],DI    ; Write to Destination y Address
; Store width in Operations Dimension 1 register
        MOV     GS:[+60H],CX    ; Write to Operation Dimension 1
; Store height in Operations Dimension 2 register
        MOV     GS:[+62H],DX    ; Write to Operation Dimension 2
;**********************|
;  setup pix operation |
;      registers       |
;**********************|
        MOV     EAX,028218000H   ; Value from bit map
        MOV     GS:[+7CH],EAX   ; Write to Pixel Operations
                                ; register
        RET
COP_SYSVID_8    ENDP
;
;*****************************************************************
COP_LINE_2      PROC    FAR
; Draw line using XGA graphics coprocessor
; Code assumes:
;       1. 1024 by 768 mode in 256 colors
;       2. The INIT_COP routine has been previously called to
;          initialize the GS and FS segment registers
; On entry:
;        CX = x pixel coordinate of line start
;        DX = y pixel coordinate of line start
;        SI = x pixel coordinate of line end
;        DI = y pixel coordinate of line end
;        BL = 8-bit color code
;
;**********************|
;   test for not busy  |
;**********************|
        CALL    COP_RDY         ; Local routine
; At this point the coprocessor is not busy
;**********************|
;   prepare to draw    |
;**********************|
; Draw line accessing coprocessor registers directly
        MOV     AL,01H          ; Data value for Map A
        MOV     GS:[+12H],AL    ; Write to pixel map index
        MOV     AX,0H           ; Data value for VRAM low
        MOV     GS:[+14H],AX    ; Write to pix map base address
; FS register holds the high order word of VRAM address. This
; value is calculated by the INIT_COP routine in this module
        MOV     AX,FS           ; Data for VRAM high
        MOV     GS:[+16H],AX    ; Write to pix map segment address
; Code assumes 1024 by 768 pixel mode and Intel format
        MOV     AX,1023         ; Value for pix map width
        MOV     GS:[+18H],AX    ; Write to width register
        MOV     AX,767          ; Value for pix map height
        MOV     GS:[+20H],AX    ; Write to height register
        MOV     AL,3            ; Select Intel order and 8 bits
                                ; per pixel
        MOV     GS:[+1CH],AL    ; Write to format register
;**********************|
;     draw the line    |
;**********************|
        MOV     AL,03H          ; Select source mix mode
        MOV     GS:[+48H],AL    ; Write to Mix register
; Write color (in BL) to foreground register
        MOV     GS:[+58H],BL    ; Write to foreground color
                                ; register
; Write coordinates of line start point to coprocessor registers
        MOV     GS:[+78H],CX    ; Write to Destination x Address
                                ; register
        MOV     GS:[+7AH],DX    ; Write to Destination y Address
                                ; register
;**********************|
;  calculate octant 0  |
;**********************|
;        CX = x pixel coordinate of line start
;        DX = y pixel coordinate of line start
;        SI = x pixel coordinate of line end
;        DI = y pixel coordinate of line end
; Octant bits in Pixel Operations register as follows:
; xxxx x210
;       |||_______ DZ bit = 0 if |x| > |y|
;       ||________ DY bit = 0 if y is positive (DI >= DX)
;       |_________ DX bit = 0 if x is positive (SI >= CX)
; BL will hold octant bits
        MOV     BL,0            ; Clear octant register
        CMP     SI,CX           ; Test for DX bit
        JGE     DX_ISOK         ; Go if horizontal line
; At this point SI < CX, therefore DX bit must be set
        OR      BL,00000100B    ; DX bit is now set in BL
        XCHG    SI,CX           ; Exchange so that CX > SI
DX_ISOK:
; Now test DX bit condition
        CMP     DI,DX           ; Test for DY bit
        JGE     DY_ISOK         ; Go if horizontal line
; At this point DI < DX, therefore DY bit must be set
        OR      BL,00000010B    ; DY bit is now set in BL
        XCHG    DI,DX           ; Exchange so that DX > DI
; Now test DX bit condition
DY_ISOK:
        SUB     DI,DX           ; Find |y|
        XCHG    DX,DI           ; |y| to DX
        SUB     SI,CX           ; and |x|
        XCHG    CX,SI           ; |x| to CX
        CMP     CX,DX           ; Is |x| > |y|
        JG      BRZ_TERMS       ; Go to leave DZ = 0
; At this point |x| <= |y|, therefore DZ bit must be set
; and |y| must be exchanged with |x|
        OR      BL,00000001B    ; Set DZ bit
        XCHG    CX,DX           ; Exchange
;**********************|
;   Bresenham terms    |
;     calculations     |
;**********************|
BRZ_TERMS:
; Bresenham terms:
;     Term E (error) = (2 * |y|) - |x|
;     Term K1        = 2 * |y|
;     Term K2        = 2 * (|y| - |x|)
; AT this point CX = |x| and DX = |y|
; First store |x| in Operations Dimensions register
        MOV     GS:[+60H],CX    ; Write to Operation Dimension 1
                                ; register
; Then calculate Term E
        PUSH    DX              ; Save |y|
        ADD     DX,DX           ; 2 * |y|
        SUB     DX,CX           ; - |x|
        MOV     SI,DX           ; Store Term E in SI
        POP     DX              ; Restore |y|
        PUSH    CX              ; and save |x|
        MOV     CX,DX           ; |y| to CX
        ADD     CX,CX           ; Calculate 2 * |y|
        MOV     DI,CX           ; Store Term K1 in DI
        POP     CX              ; Restore |x| from stack
        SUB     DX,CX           ; |y| - |x|
        ADD     DX,DX           ; times 2
; DX = Term K2
        MOV     GS:[+20H],SI    ; Write to Error Term register
        MOV     GS:[+24H],DI    ; Write to K1 register
        MOV     GS:[+28H],DX    ; Write to K2 register
; Bit map of Pixel Operations register:
; byte 3 = 0000|0101 = line draw write operation
; byte 2 = 0001      = source pixel map is map A
;               0001 = destination pixel map is map A
; byte 1 = 1000|rrrr = special code for foreground and all 1s
; byte 0 = 00   0    = Map mask disabled
;            00      = Drawing mode for all pixels drawn
;                      OCTANT DATA:
;                o   = DX = 0 for x in positive direction
;                 o  = DY = 0 for y in positive direction
;                  o = DZ = 0 for |x| > |y|
        MOV     EAX,05118000H   ; All bits except octant
; BL holds octant bits
        OR      AL,BL           ; OR-in octant bits
        MOV     GS:[+7CH],EAX   ; Write to Pixel Operations
                                ; register
        RET
COP_LINE_2      ENDP
;
;*****************************************************************
;           coprocessor support procedures (NEAR type)
;*****************************************************************
XGA_NEWBANK    PROC     NEAR
; Switch video memory bank in XGA VRAM
; Note: processing assumes that 1024 by 768 mode is active
; On entry:
;         AL = new bank number
;
        MOV    XGA_CURBK,AX      ; Save the new current bank value
        MOV    DX,XGA_REG_BASE   ; XGA base register address
        ADD    DX,08H            ; Aperture index register
        OUT    DX,AL             ; Bank number is in AL
        RET
XGA_NEWBANK    ENDP
;*****************************************************************
;
COP_RDY         PROC    NEAR
; Poll bit 7 of coprocessor Control register (offset 11H) to
; determine if coprocessor is busy, if so, wait until ready
; Code assumes that GS segment holds coprocessor base address

        PUSH    AX              ; Save context
        PUSH    CX
TEST_COP:
        MOV     AL,GS:[+11H]    ; Read control register
        TEST    AL,10000000B    ; Test bit 7
        JZ      COP_READY       ; Go if bit is clear
        MOV     CX,100          ; Counter for wait loop
; A 100 iteration wait loop is introduced so that the coprocessor
; is not polled constantly, thus slowing down execution
WAIT_100:
        NOP                     ; Delay
        NOP
        LOOP    WAIT_100        ; Wait
        JMP     TEST_COP        ; Test again after wait
COP_READY:
        POP     CX              ; Restore context
        POP     AX
        RET
COP_RDY         ENDP
;
;*****************************************************************
;*****************************************************************
;                        sprite routines
;*****************************************************************
;*****************************************************************
SPRITE_IMAGE    PROC    FAR
; Load sprite image and select color registers
; On entry:
;         XGA_REG_BASE variable holds XGA register base address
;         DS:SI --> caller's sprite image buffer and color buffer
;                   formatted as follows:
;   OFFSET    UNIT      CONTENTS
;     0       byte      6 low bits are RED for sprite color 0
;     1       byte      6 low bits are GREEN for sprite color 0
;     2       byte      6 low bits are BLUE for sprite color 0
;     3       byte      6 low bits are RED for sprite color 1
;     4       byte      6 low bits are GREEN for sprite color 1
;     5       byte      6 low bits are BLUE for sprite color 1
;     6       16 bytes per 64 rows (1024 bytes) encoding the
;             sprite image at 2 bits per pixel
;   1030      end of sprite image
;
;**********************|
;   save caller's DS   |
; set DS to XGA_DIRECT |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DIRECT           ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DIRECT
;**********************|
; set sprite color 0   |
;**********************|
; Load sprite color 0 registers using values in parameter block
; supplied by caller (DS:SI)
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 38H is Sprite Color 0, red value
        MOV     AL,38H          ; Sprite register
        MOV     AH,[SI]         ; Data from caller's buffer
        INC     SI              ; Bump pointer to next byte
        OUT     DX,AX           ; Write data
;
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 39H is Sprite Color 0, green value
        MOV     AL,39H          ; Sprite register
        MOV     AH,[SI]         ; Data from caller's buffer
        INC     SI              ; Bump pointer to next byte
        OUT     DX,AX           ; Write data
;
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 3AH is Sprite Color 0, blue value 
        MOV     AL,3AH          ; Sprite register
        MOV     AH,[SI]         ; Data from caller's buffer
        INC     SI              ; Bump pointer to next byte
        OUT     DX,AX           ; Write data
;**********************|
; set sprite color 1   |
;**********************|
; Load sprite color 1 registers to GREEN
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 3BH is Sprite Color 1, red value
        MOV     AL,3BH          ; Sprite register
        MOV     AH,[SI]         ; Data from caller's buffer
        INC     SI              ; Bump pointer to next byte
        OUT     DX,AX           ; Write data
;
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 3CH is Sprite Color 1, green value
        MOV     AL,3CH          ; Sprite register
        MOV     AH,[SI]         ; Data from caller's buffer
        INC     SI              ; Bump pointer to next byte
        OUT     DX,AX           ; Write data
;
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 3DH is Sprite Color 1, blue value 
        MOV     AL,3DH          ; Sprite register
        MOV     AH,[SI]         ; Data from caller's buffer
        INC     SI              ; Bump pointer to next byte
        OUT     DX,AX           ; Write data
;**********************|
;  load sprite image   |
;**********************|
; First set the sprite index registers to zero
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 60H is Sprite/Palette index Low
        MOV     AX,0060H        ; 00 to register at offset 60H
        OUT     DX,AX           ; Write data
; Reset to base
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 61H is Sprite/Palette index High
        MOV     AX,0061H        ; 00 to register at offset 60H
        OUT     DX,AX           ; Write data
; Select the Sprite Data register at offset 6AH
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
        MOV     AL,06AH         ; Offset of data register
        OUT     DX,AL           ; Select the Sprite Data register
;**********************|
;  load sprite image   |
;**********************|
; DS:SI --> buffer area containing the sprite bit mapped image in
;           2 bits per pixel format, as follows:
;           00 = sprite color 0
;           01 = sprite color 1
;           10 = transparent pixel
;           11 = complement pixel
        MOV     CX,512          ; Word item counter
SPRITE_DATA:
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0CH          ; To second data register
        MOV     AX,[SI]         ; Get data from buffer
        OUT     DX,AX           ; Send to data port
        INC     SI              ; Bump data pointer
        INC     SI              ; to next word
        LOOP    SPRITE_DATA     ; Repeat 512 times
        POP     DS              ; Restore caller's DS
        RET
SPRITE_IMAGE    ENDP
;*****************************************************************
;
SPRITE_AT       PROC    FAR
; Display sprite image at coordinates furnished by the caller
; On entry:
;          BX = x coordinate of sprite location (0 to 1023)
;          CX = y coordinate of sprite location (0 to 767)
;
;**********************|
;   save caller's DS   |
; set DS to XGA_DIRECT |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DIRECT           ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DIRECT
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 30H is Sprite x Start LOW register
        MOV     AH,BL           ; Value to start register
        MOV     AL,30H          ; Address of Start x Low
        OUT     DX,AX           ; Write data
;
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 31H is Sprite x Start HIGH register
        MOV     AH,BH           ; Value to start register
        MOV     AL,31H          ; Address of Start register
        OUT     DX,AX           ; Write data
; Set Sprite x Preset register to 0
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 32H is Sprite x Preset register
        MOV     AH,00           ; Value to preset register
        MOV     AL,32H          ; Address of Start register
        OUT     DX,AX           ; Write data
; Select y coordinate registers
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 33H is Sprite y Start LOW register
        MOV     AH,CL           ; Value to start register
        MOV     AL,33H          ; Address of Start x Low
        OUT     DX,AX           ; Write data
;
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 34H is Sprite y Start HIGH register
        MOV     AH,CH           ; Value to start register
        MOV     AL,34H          ; Address of Start register
        OUT     DX,AX           ; Write data
; Set Sprite y Preset register to 0
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Index register 35H is Sprite x Preset register
        MOV     AH,00           ; Value to preset register
        MOV     AL,35H          ; Address of Start register
        OUT     DX,AX           ; Write data
;**********************|
;    display sprite    |
;**********************|
; Sprite is displayed by setting bit 0 of the Sprite Control
; register at offset 36H
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Sprite control register offset is 36H
        MOV     AH,01           ; Value to start register
        MOV     AL,36H          ; Address of Start register
        OUT     DX,AX           ; Write data
        POP     DS              ; Restore caller's DS
        RET
SPRITE_AT       ENDP
;*****************************************************************
;
SPRITE_OFF      PROC    FAR
; Sprite is turned off by clearing bit 0 of the Sprite Control
; register at offset 36H
;**********************|
;   save caller's DS   |
; set DS to XGA_DIRECT |
;**********************|
        PUSH    DS
        MOV     AX,XGA_DIRECT           ; Local data segment
        MOV     DS,AX                   ; to DS
        ASSUME  DS:XGA_DIRECT
        MOV     DX,XGA_REG_BASE ; Register base
        ADD     DX,0AH          ; To index register
; Sprite control register offset is 36H
        MOV     AH,0            ; Value to start register
        MOV     AL,36H          ; Address of Start register
        OUT     DX,AX           ; Write data
        POP     DS              ; Restore caller's DS
        RET
SPRITE_OFF      ENDP
;
;
P_CODE  ENDS
        END
