$ NOPRINT ERRORPRINT
NAME MATCH00
;*******************************************************************
;*******************************************************************
;                            MATCHA.ASM
;*******************************************************************
;*******************************************************************
;
; Arcade-type game program using timed-pulse and interactive 
; animation
;
; The code has the following system requirements:
; 1. VGA graphics system
; 2. MS DOS 3.0 or later
; 3. IBM PS/2, Microsoft, or compatible mouse, and installed driver
; 4. Font and TIFF files used by the Match program must be in the
;    current path
;
; Note: This version is to be assembled and link using ASM86
;       development tools to generate an 80x87 emulated program
;
;*******************************************************************
;                    public symbols in this module
;*******************************************************************
PUBLIC  FONT_1X8
PUBLIC  FONT_1X16
PUBLIC  FONT_TR9B
PUBLIC  FONT_TR6N
PUBLIC  STACK_TOP
PUBLIC  TARGET_Y
PUBLIC  TARGET_COLOR
PUBLIC  TARGE_IMAGE
;*******************************************************************
;                   procedures in SOLUTION.LIB
;*******************************************************************
EXTRN   KBR_FLUSH:      FAR
EXTRN   KBR_STATUS:     FAR
EXTRN   KBR_WAIT:       FAR
EXTRN   DOS_STRING_OUT: FAR
EXTRN   SOUND:          FAR
EXTRN   RANDOM_NUM:     FAR
EXTRN   BIN_TO_ASC:     FAR
EXTRN   BIO_SET_MODE:   FAR
EXTRN   BIO_SET_CUR:    FAR
EXTRN   TIME_VRC:       FAR
EXTRN   COARSE_FILL:    FAR
EXTRN   COARSE_ADD:     FAR
EXTRN   PIXEL_ADD:      FAR
EXTRN   WRITE_PIX:      FAR
EXTRN   SET_WRITE_MODE: FAR
EXTRN   DATA_XOR:       FAR
EXTRN   DATA_NORMAL:    FAR
EXTRN   GRAPHIC_TEXT:   FAR
EXTRN   GRAPHIC_CUR:    FAR
EXTRN   SHOW_CHAR:      FAR
EXTRN   SHOW_BIT_MAP:   FAR
EXTRN   SET_DTA:        FAR
EXTRN   BIO_SHOW_MESS:  FAR
EXTRN   MULTI_TEXT:     FAR
EXTRN   SHOW_TIFF:      FAR
;
; From the GRAFTEXT module
EXTRN   FINE_PATTERN:   FAR
EXTRN   FINE_TEXT:      FAR
EXTRN   FINE_TEXTHP:    FAR
EXTRN   READ_HPFONT:    FAR
EXTRN   COARSE_TEXT:    FAR
EXTRN   COARSE_PATTERN: FAR
EXTRN   INDEX_HP:       FAR
EXTRN   FONT_TO_RAM:    FAR
EXTRN   FONT_X2:        FAR
;
; From Intel emulator software
EXTRN   INIT87:         FAR
;
; External symbols in other modules
EXTRN   BOAR_15X15:     FAR
;
;***************************** stack *******************************
STACK   SEGMENT stack
;
               DB      0800H DUP (00H); Default stack is 2K
STACK_TOP      DW      0
;
STACK   ENDS
;***************************** data ********************************
;
DATA    SEGMENT BYTE    PUBLIC
;
;**********************|
;   first screen       |
;**********************|
RIF1K_IMAGE     DW      30      ; x coordinate
                DW      0       ; y coordinate
                DW      174     ; Offset of bit map from start
                                ; of file
                DW      173     ; Number of pixel rows
                DB      74      ; Number of bytes per row
                DB      00000000B       ; I R G B color code
                DB      'RIF1K.TIF',00H
                DB      5 DUP (00H)      ; Padding to 15 bytes
;
RIF1B_IMAGE     DW      30      ; x coordinate
                DW      0       ; y coordinate
                DW      174     ; Offset of bit map from start
                                ; of file
                DW      173     ; Number of pixel rows
                DB      74      ; Number of bytes per row
                DB      00000110B       ; I R G B color code
                DB      'RIF1B.TIF',00H
                DB      5 DUP (00H)      ; Padding to 15 bytes
;
RIF1W_IMAGE     DW      30      ; x coordinate
                DW      0       ; y coordinate
                DW      174     ; Offset of bit map from start
                                ; of file
                DW      173     ; Number of pixel rows
                DB      74      ; Number of bytes per row
                DB      00001111B       ; I R G B color code
                DB      'RIF1W.TIF',00H
                DB      5 DUP (00H)      ; Padding to 15 bytes
;
; Pistol bit display blocks
PIS1K_IMAGE     DW      240     ; x coordinate
                DW      290     ; y coordinate
                DW      174     ; Offset of bit map from start
                                ; of file
                DW      211     ; Number of pixel rows
                DB      50      ; Number of bytes per row
                DB      00000000B       ; I R G B color code
                DB      'PIS1K.TIF',00H
                DB      5 DUP (00H)      ; Padding to 15 bytes
;
PIS1B_IMAGE     DW      240     ; x coordinate
                DW      290     ; y coordinate
                DW      174     ; Offset of bit map from start
                                ; of file
                DW      211     ; Number of pixel rows
                DB      50      ; Number of bytes per row
                DB      00000110B       ; I R G B color code
                DB      'PIS1B.TIF',00H
                DB      5 DUP (00H)      ; Padding to 15 bytes
;
PIS1W_IMAGE     DW      240     ; x coordinate
                DW      290     ; y coordinate
                DW      174     ; Offset of bit map from start
                                ; of file
                DW      211     ; Number of pixel rows
                DB      50      ; Number of bytes per row
                DB      00001111B       ; I R G B color code
                DB      'PIS1W.TIF',00H
                DB      5 DUP (00H)      ; Padding to 15 bytes
;
MATCH_IMAGE     DW      210     ; x coordinate
                DW      110     ; y coordinate
                DW      174     ; Offset of bit map from start
                                ; of file
                DW      109     ; Number of pixel rows
                DB      52      ; Number of bytes per row
                DB      00000100B       ; I R G B color code
                DB      'MAT1R.TIF',00H
                DB      5 DUP (00H)      ; Padding to 15 bytes
;
TARGE_IMAGE     DW      0       ; x coordinate
TARGET_Y        DW      242     ; y coordinate
                DW      174     ; Offset of bit map from start
                                ; of file
                DW      238     ; Number of pixel rows
                DB      42      ; Number of bytes per row
TARGET_COLOR    DB      00000000B       ; I R G B color code
                DB      'TAR1K.TIF',00H
                DB      5 DUP (00H)      ; Padding to 15 bytes
; Text for first screen
INTRO1_MS       DW      80      ; x coordinate
        DW      240     ; y coordinate
        DW      2       ; Horizontal pixel spacing
        DW      0       ; Vertical pixel spacing
        DB      00000000B       ; I R G B code
        DB      'GRAPHICS SOLUTIONS - VGA Demo',00H
;
INTRO2_MS       DW      250     ; x coordinate
        DW      270     ; y coordinate
        DW      2       ; Horizontal pixel spacing
        DW      0       ; Vertical pixel spacing
        DB      00000100B       ; I R G B code
        DB      'Press any key to start ',22H,'Match',22H,00H
;*****************************************************************
;                         RAM fonts
;*****************************************************************
;**********************|
;   symetrical fonts   |
;**********************|
; RAM storage for symetrical font tables
; 1 x 8 built in ROM font
                DB      1       ; Bit map x dimension, in bytes
                DB      8       ; Bit map y dimension, in bits
FONT_1X8        DB      2050 DUP (00H)
;
; 1 x 16 built in ROM font
                DB      1       ; Bit map x dimension, in bytes
                DB      16      ; Bit map y dimension, in bits
FONT_1X16       DB      4100 DUP (00H)
;**********************|
;    HP printer fonts  |
;**********************|
; In these fonts the bit maps are of variable size, therefore
; font dimension are not recorded
; Font stored is generated with Bitstream Fontware
; 9 pt. Dutch Bold
; Original filename: AK090UHP.SFP
; Match program filename: MATCH.FN0
FONT_TR9B       DB      11000 DUP (00H)
FONT_TR6N       DB      12400 DUP (00H)
;**********************|
;     HP font data     |
;**********************|
FONT_FILE1      DB      'MATCHB9.FNT',00H
FONT_FILE2      DB      'MATCHN6.FNT',00H
DTA_BUF         DB      128 DUP (00H)   ; Disk tranfer area
                DW      0
;**********************|
;     error messages   |
;**********************|
FONT_ERR        DB      0AH,0DH
                DB      'ERROR - Match font file not found',00H
TIFF_ERR        DB      0AH,0DH
                DB      'ERROR - Tiff file not found',00H
;
DATA    ENDS
;*******************************************************************
;*******************************************************************
;                             code
;*******************************************************************
;*******************************************************************
CODE_0  SEGMENT BYTE    PUBLIC
        ASSUME  CS:CODE_0
;**********************|
;  CODE_0 Externals    |
;**********************|
EXTRN   START_MAIN:     NEAR
EXTRN   DIRECT_EXIT:    NEAR
;
;
START:
;**********************|
;    initialize DS     |
;**********************|
        MOV     AX,DATA         ; AX to module data
        MOV     DS,AX           ; and to DS
        ASSUME  DS:DATA
; Initialize stack. Note that this instruction is necessary for
; compatibility with non-Microsoft assemblers
        LEA     SP,SS:STACK_TOP ; Stack pointer to top of stack
;**********************|
; init 8087 emulator   |
;**********************|
        CALL    INIT87          ; Procedure in Intel emulator
                                ; library
;**********************|
;  load 8 x 8 font     |
;**********************|
        MOV     AH,11H          ; Service request number
        MOV     AL,30H          ; Font table information
        MOV     BH,03H          ; 8 x 8 font
        INT     10H
; ES:BP --> 8 x 8 ROM character table
;         DS:DI --> Start of RAM buffer for font
;         CX = word length of font to store
        LEA     DI,FONT_1X8     ; RAM storage for font
        MOV     CX,1024         ; Word length of font table
; DS:DI --> Start of RAM buffer for font
; CX = word length of font to store
        CALL    FONT_TO_RAM     ; Procedure in SOLUTION.LIB
;**********************|
;  load 8 x 16 font    |
;**********************|
        MOV     AH,11H          ; Service request number
        MOV     AL,30H          ; Font table information
        MOV     BH,06H          ; 8 x 16 font
        INT     10H
; ES:BP --> 8 x 16 ROM character table
;         DS:DI --> Start of RAM buffer for font
;         CX = word length of font to store
        LEA     DI,FONT_1X16    ; RAM storage for font
        MOV     CX,2048         ; Word length of font table
; DS:DI --> Start of RAM buffer for font
; CX = word length of font to store
        CALL    FONT_TO_RAM     ; Procedure in SOLUTION.LIB
;**********************|
;     load HP font     |
;**********************|
; Font files must be in the current path
; Set Disk transfer Area at the buffer named DTA_BUF
        LEA     DX,DTA_BUF      ; DTA buffer in RAM
        CALL    SET_DTA
; Read Times Roman Bold 9 point font
        LEA     DX,FONT_FILE1   ; Buffer with ASCIIZ filename
        LEA     DI,FONT_TR9B    ; Storage buffer for HP font
        CALL    READ_HPFONT     ; Local procedure
        JC      BAD_FONT_READ   ; Error if font not read
; Read Times Roman Normal 6 point font
        LEA     DX,FONT_FILE2   ; Buffer with ASCIIZ filename
        LEA     DI,FONT_TR6N    ; Storage buffer for HP font
        CALL    READ_HPFONT     ; Local procedure
        JNC     FIRST_SCREEN    ; Font read successfully
;**********************|
;   font read error    |
;**********************|
BAD_FONT_READ:
        LEA     SI,FONT_ERR     ; Error message text
ERROR_EXIT:
        PUSH    SI              ; Save message pointer
        MOV     AL,3            ; Set alphanumeric mode 3
        CALL    BIO_SET_MODE    ; Procedure in SOLUTION.LIB
        POP     SI
        CALL    BIO_SHOW_MESS   ; Alpha display procedure
        JMP     DIRECT_EXIT     ; Go to MS DOS
;*****************************************************************
;                         first screen
;*****************************************************************
FIRST_SCREEN:
;**********************|
;   initialize video   |
;     buffer address   |
;**********************|
        MOV     AX,0A000H       ; AX to base address of video buffer
        MOV     ES,AX           ; To ES
;
        MOV     AH,0            ; BIOS service request code
        MOV     AL,18           ; VGA mode number 18. 640 by 480
                                ; pixels in 16 colors
        INT     10H             ; BIOS video interrupt
;**********************|
;   set write mode 2   |
;**********************|
; This program uses write mode 2 for graphics display. In this write
; mode the color code is contained in a CPU register
        MOV     AL,2            ; Mode number
        CALL    SET_WRITE_MODE
;**********************|
;  draw color blocks   |
;**********************|
; Fill entire screen
        MOV     CH,0            ; Start column
        MOV     CL,0            ; Start row
        MOV     DH,80           ; Horizontal bytes
        MOV     DL,60           ; Vertical bytes
        MOV     AL,00000111B    ; Color = gray
        CALL    COARSE_FILL
;**********************|
;  display TIFF images |
;**********************|
; Pistol
        LEA     SI,PIS1B_IMAGE
        CALL    SHOW_TIFF       ; Display rountine for TIFF
        LEA     SI,PIS1K_IMAGE
        CALL    SHOW_TIFF       ; Display rountine for TIFF
        LEA     SI,PIS1W_IMAGE
        CALL    SHOW_TIFF       ; Display rountine for TIFF
; The word "Match"
        LEA     SI,MATCH_IMAGE
        CALL    SHOW_TIFF       ; Display rountine for TIFF
; Target
        LEA     SI,TARGE_IMAGE
        CALL    SHOW_TIFF       ; Display rountine for TIFF
; Rifle
        LEA     SI,RIF1B_IMAGE
        CALL    SHOW_TIFF       ; Display rountine for TIFF
        LEA     SI,RIF1K_IMAGE
        CALL    SHOW_TIFF       ; Display rountine for TIFF
        LEA     SI,RIF1W_IMAGE
        CALL    SHOW_TIFF       ; Display rountine for TIFF
;**********************|
;    display text      |
;**********************|
        LEA     SI,INTRO1_MS     ; VGA demo message
        LEA     DI,FONT_TR6N    ; Hewlett Packard TR 6 pts
        CALL    FINE_TEXTHP     ; Procedure in SOLUTION.LIB
;
        LEA     SI,INTRO2_MS    ; Press any key message
        LEA     DI,FONT_TR6N    ; Hewlett Packard TR 6 pts
        CALL    FINE_TEXTHP     ; Procedure in SOLUTION.LIB
;
        CALL    KBR_WAIT
        JMP     START_MAIN
;
CODE_0  ENDS
        END     START

 Procedure in SOLUTION.LIB
;
        CALL  